/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 
package org.eclipse.actf.accservice.core.win32.msaa;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.accservice.IAccessibilityEventService;
import org.eclipse.actf.accservice.IWindowService;
import org.eclipse.actf.accservice.core.IAccessibleElement;
import org.eclipse.actf.accservice.event.AccessibilityModelEvent;
import org.eclipse.actf.model.event.IModelEventListener;
import org.eclipse.actf.model.event.ModelEventType;


public class MsaaAccessibilityEventService implements IAccessibilityEventService {


	public static final int EVENT_SYSTEM_MENUSTART = 0x0004;
	public static final int EVENT_SYSTEM_MENUEND = 0x0005;
	public static final int EVENT_SYSTEM_MENUPOPUPSTART =0x0006;
	public static final int EVENT_SYSTEM_MENUPOPUPEND =0x0007;
	public static final int EVENT_OBJECT_CREATE = 0x8000; // hwnd + ID + idChild is created item
	public static final int EVENT_OBJECT_DESTROY = 0x8001; // hwnd + ID + idChild is destroyed item
	public static final int EVENT_OBJECT_SHOW = 0x8002; // hwnd + ID + idChild is shown item
	public static final int EVENT_OBJECT_HIDE = 0x8003; // hwnd + ID + idChild is hidden item
	public static final int EVENT_OBJECT_REORDER = 0x8004;
	public static final int EVENT_OBJECT_FOCUS = 0x8005; // hwnd + ID + idChild is focused item
	public static final int EVENT_OBJECT_SELECTION = 0x8006; // hwnd + ID + idChild is selected item (if only one), or idChild is OBJID_WINDOW if complex
	public static final int EVENT_OBJECT_SELECTIONADD = 0x8007;// hwnd + ID + idChild is item added
	public static final int EVENT_OBJECT_SELECTIONREMOVE = 0x8008;// hwnd + ID + idChild is item removed
	public static final int EVENT_OBJECT_SELECTIONWITHIN = 0x8009;
	public static final int EVENT_OBJECT_STATECHANGE = 0x800A;
	public static final int EVENT_OBJECT_LOCATIONCHANGE = 0x800B;
	public static final int EVENT_OBJECT_NAMECHANGE = 0x800C; // hwnd + ID + idChild is item w/ name change
	public static final int EVENT_OBJECT_DESCRIPTIONCHANGE = 0x800D; // hwnd + ID + idChild is item w/ desc change
	public static final int EVENT_OBJECT_VALUECHANGE = 0x800E; // hwnd + ID + idChild is item w/ value change
	public static final int EVENT_OBJECT_PARENTCHANGE = 0x800F; // hwnd + ID + idChild is item w/ new parent
	public static final int EVENT_OBJECT_HELPCHANGE = 0x8010; // hwnd + ID + idChild is item w/ help change
	public static final int EVENT_OBJECT_DEFACTIONCHANGE = 0x8011; // hwnd + ID + idChild is item w/ def action change
	public static final int EVENT_OBJECT_ACCELERATORCHANGE = 0x8012;

	public static final int WINEVENT_OUTOFCONTEXT   = 0x0000 ; // Events are ASYNC
	public static final int WINEVENT_SKIPOWNTHREAD  = 0x0001 ; // Don't call back for events on installer's thread
	public static final int WINEVENT_SKIPOWNPROCESS = 0x0002 ;  // Don't call back for events on installer's process
	public static final int WINEVENT_INCONTEXT      = 0x0004 ; // Events are SYNC, this causes your dll to be injected into every process

	protected static Map _listenerMap = new HashMap();
	protected Map _hookMap = new HashMap();
	protected IWindowService _windowService;
	protected static int _flag = WINEVENT_INCONTEXT|WINEVENT_SKIPOWNPROCESS;
	protected static long timeDiff=0;
	protected static long initClockTicks =0;

	//Memory Map
	public static final String fileMappingObjName = "Event_data_Mem_Map_File";
	public static final int PAGE_READONLY = 0x02;
	public static final int PAGE_READWRITE = 0x04;
	public static final int PAGE_WRITECOPY = 0x08;

	public static final int FILE_MAP_COPY = 0x0001;
	public static final int FILE_MAP_WRITE = 0x0002;
	public static final int FILE_MAP_READ = 0x0004;
	protected static int mapFilePtr;
	//private static int viewPtr;
	public final int dwMemFileSize = 2 * 1024;

	protected static boolean isMapFileOpen = false;

	public static native int createFileMapping(int lProtect, int dwMaximumSizeHigh, int dwMaximumSizeLow, String name);
	public static native int openFileMapping(int dwDesiredAccess, boolean bInheritHandle, String name);
	public static native int mapViewOfFile(int hFileMappingObj, int dwDesiredAccess, int dwFileOffsetHigh, int dwFileOffsetLow, int dwNumberOfBytesToMap);
	public static native boolean unmapViewOfFile(int lpBaseAddress);
	public static native void writeToMem(int lpBaseAddress, int content);
	public static native String[] readFromMem(int lpBaseAddress);
	public static native boolean closeHandle(int hObject);
	public native boolean initThread();

	public MsaaAccessibilityEventService (IWindowService windowService) {
		_windowService = windowService;
	}

	public static int getWinEventFlag () {
		return _flag;
	}

	public static void setWinEventFlag (int evFlag) {
		_flag = evFlag;
	}

	public static long  getClockTicks(){
		return internalGetClockTicks();
	}
	protected native static long internalGetClockTicks();
	/**
	 * MSAA SetWinEventHook function wrapper
	 * 
	 * @param eventMin - minimum event type for hook 
	 * (use one of the pre-defined constants in this class)
	 * @param eventMax - maximum event value for hook
	 * (use one of the pre-defined constants in this class)
	 * @param idProcess - usually to be obtained from getProcessId(int)
	 * @param idThread - use id of desired thread or 0 for all threads
	 * @return hook reference to be used with unregisterWinEventHook or -1 if set was not successful
	 * @see #removeWinEventHook(int)
	 */
	protected int setWinEventHook (int eventMin, int eventMax, int idProcess, int idThread) {
		int hook = -1;
		hook = internalSetWinEventHook(eventMin, eventMax, idProcess, idThread, _flag);
		return hook;
	}

	protected native int internalSetWinEventHook (int eventMin, int eventMax, int idThread, int idProcess, int  dwFlags);

	/**
	 * remove a previously registered winEventHook
	 * 
	 * @param hook - hook reference obtained from <code>setWinEventHook</code>
	 * @return <code>true</code> if hook successfully removed, <code>false</code> otherwise
	 * @see #setWinEventHook(int, int, int, int)
	 */
	protected boolean removeWinEventHook (int hook) {
		return internalRemoveWinEventHook(hook);
	}

	protected native boolean internalRemoveWinEventHook (int hook);

	/**
	 * callback from the native Windows system for out-of-process Msaa events. The parameters are:
	 * 
	 * <p><ul>
	 * <li>event information (hwnd, child Id, Object ID, event ID)to form a new <code>MsaaAccessible</code>. 
	 * In this case, the source of the resulting event will be this <code>MsaaAccessible</code> object.
	 * </ul></p>
	 * 
	 * <p>The appropriate listeners for the given event id are notified and the <code>AccesibilityModelEvent</code> is 
	 * created from the formed <code>MsaaAccessible</code> </p>
	 * 
	 * @param eventId
	 * @param hwnd
	 * @param idObject
	 * @param idChild
	 * @param time
	 */
	protected static void winEventCallback(int eventId, int hwnd, 
			int idObject, int idChild,long time) {

		if(initClockTicks > time){
			setTimeDiff();
		}
		long dms =System.currentTimeMillis();
		long ctime = time+ timeDiff;
		AccessibilityModelEvent accEvent = null;	
		MsaaAccessible macc = null;
		int ref = createAccessibleObjectFromEvent(hwnd,idObject,idChild);
		if (ref!=0) {
			macc = new MsaaAccessible(ref);
		}else{
			return;
		}
		IAccessibleElement acc = macc;
		try {
			acc = macc.testAndConvertToIA2(macc);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
		accEvent = new AccessibilityModelEvent(acc);
		accEvent.setEventType(eventName(eventId));
		accEvent.setTimeMillis(ctime);

		fireAccessibilityModelEvent(accEvent, eventId);	
		
	}

	protected static native int createAccessibleObjectFromEvent(int hwnd, int idObject, int idChild);

	
	/**
	 * callback from the native Windows system for in-process Msaa events. 
	 * The event information is read from a memory map file as a string array.
	 *  
	 * <p>The appropriate listeners for the given event id are notified and the <code>AccesibilityModelEvent</code> is  
	 * created. Here, the source of the AccesibilityModelEvent is a string array containing information about the 
	 * accessible that fired the event</p>
	 */
	protected void winEventIPCallback () {
		int mPtr = openFileMapping(FILE_MAP_READ|FILE_MAP_WRITE, false,
				fileMappingObjName);
		int state =0;
		int eventId =0;
		long timeMillis =0;
		String[] source= null;
		if(mPtr != 0) {
			int vPtr = mapViewOfFile(mPtr,
					FILE_MAP_READ|FILE_MAP_WRITE, 0, 0, 0);
			if(vPtr != 0) {
				String props[] = readFromMem(vPtr);
				if(props!=null && props.length==5){
					int role = Integer.parseInt(props[1]);
					String res = Msaa.getMsaaActfRoleName(role);
					if (res == null || res.length() < 1) {
						try {
							res = (String) Class.forName("org.eclipse.actf.accservice.core.win32.ia2.IA2")
								.getMethod("getIA2ActfRoleName", new Class[] {long.class})
									.invoke(null, new Object[] {new Integer(role)});
						}catch (Exception e) {
						}
					}
					if(res == null || res.length() < 1){
						res= props[1];
					}
					state= Integer.parseInt(props[2]);
					eventId = Integer.parseInt(props[3]);
					timeMillis = Long.parseLong(props[4]);
					if(initClockTicks > timeMillis){
							setTimeDiff();
					}
					timeMillis = timeMillis+timeDiff;
					String eventType = eventName(eventId);
					source = new String[5];
					source[0]=props[0];
					source[1]= res;
					source[2]= Msaa.getState(state).toString();
					source[3]= eventType;
					source[4]= Long.valueOf(timeMillis).toString();
				}
				unmapViewOfFile(vPtr);
			}
			closeHandle(mPtr);
		}
		//handle data
		AccessibilityModelEvent accEvent = null;
		if (source!=null){
			try {
				accEvent = new AccessibilityModelEvent(source);		
				fireAccessibilityModelEvent(accEvent, eventId);
			} catch (RuntimeException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

	protected static void fireAccessibilityModelEvent (AccessibilityModelEvent accEvent, int eventId) {

		List listeners = (List) _listenerMap.get(new Integer(eventId));
		if (listeners != null) {
			for (Iterator iter = listeners.listIterator(); iter.hasNext(); ) {
				((IModelEventListener) iter.next()).handleEvent(accEvent);
			}
		}
	}

	/**
	 * add the listener to be notified when events of the given type are fired from the underlying system
	 * 
	 * @param listener
	 * @param eventTypes to be chosen from constants in this class
	 * @see org.eclipse.actf.accservice.core.win32.msaa.MsaaGuiModel#registerModelEventListener(IModelEventListener, Object[])
	 */
	public void addAccessibilityEventListener(IModelEventListener listener, Object[] eventTypes) {

		setTimeDiff();
		if(!isMapFileOpen  && (_flag >= WINEVENT_INCONTEXT) ){
			mapFilePtr = createFileMapping(PAGE_READWRITE,
					0, dwMemFileSize, fileMappingObjName);
			if(mapFilePtr!=0){
				isMapFileOpen = true;
				initThread();
			}
		}

		for (int i = 0; i < eventTypes.length; ++i) {
			Integer eventId = (Integer) eventTypes[i];
			List listeners = (List) _listenerMap.get(eventId);
			if (listeners == null) {
				int processId = _windowService.getProcessId(_windowService.getActiveWindow());
				int hook = setWinEventHook(eventId.intValue(), eventId.intValue(), processId, 0);
				if (hook != -1) {
					listeners = new LinkedList();
					_listenerMap.put(eventId, listeners);
					_hookMap.put(eventId, new Integer(hook));
				}
			}
			listeners.add(listener);
		}
	}

	protected static void setTimeDiff() {
		initClockTicks = getClockTicks();
		timeDiff = System.currentTimeMillis() - initClockTicks;
	}
	/** {@inheritDoc} */
	public void removeAccessibilityEventListener(IModelEventListener listener, Object[] eventTypes) {
		for (int i = 0; i < eventTypes.length; ++i) {
			Integer eventId = (Integer) eventTypes[i];
			List listeners = (List) _listenerMap.get(eventId);
			if (listeners != null) {
				listeners.remove(listener);
				if (listeners.isEmpty()) {
					_listenerMap.remove(eventId);
					removeWinEventHook(((Integer) _hookMap.get(eventId)).intValue());
					_hookMap.remove(eventId);
				}
			}
		}
		if(isMapFileOpen && (_flag >= WINEVENT_INCONTEXT) ){
			if (_hookMap.isEmpty()){
				if(closeHandle(mapFilePtr)){
					isMapFileOpen = false;
					mapFilePtr = 0;
				}
			}
		}
	}


	public static String eventName(int event){
		String evName = null;

		MsaaGuiModel march = new MsaaGuiModel();
		ModelEventType mtype= march.getModelEventType(Integer.valueOf(event));
		if(mtype!=null){
			evName = mtype.getEventName();
		}
		return evName;
	}

}
