/*******************************************************************************
 * Copyright (c) 2004, 2007 IBM Corporation.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *  Mike Squillace - initial API and implementation
 *******************************************************************************/

package org.eclipse.actf.model;

import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.event.IModelChangeListener;
import org.eclipse.actf.model.event.ModelChangeEvent;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.model.traverse.ITreeNodeWalker;
import org.eclipse.actf.model.traverse.filters.INodeFilter;
import org.eclipse.actf.model.traverse.filters.NodeIdFilter;
import org.eclipse.actf.model.traverse.filters.NodeNameFilter;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;


/**
 * provides basic services for any model instance. Note that the IGuiModel
 * interface must be implemented as well by clients that wish to validate
 * structures via validation documents using the ACTF Engine.
 * 
 * @see org.eclipse.actf.core.model.IGuiModel
 * @author Mike Squillace
 */
public abstract class AbstractModel implements IModel
{

	protected IConfiguration configuration;

	protected IRuntimeContext runtimeContext = RuntimeContextFactory.getInstance().getRuntimeContext();

	protected INodeLocator locator;

	protected ITreeNodeWalker treeNodeWalker;

	private List changeListenerList = new LinkedList();

	private String _type;

	protected String baseType;

	/**
	 * create a new model
	 * 
	 * @param type -
	 *            type of model
	 */
	public AbstractModel (String type) {
		_type = type;
		try {
			configuration = runtimeContext.getConfiguration();
		}catch (ConfigurationException e) {
			Utils.println(
				IReporter.SYSTEM_FATAL, "Trouble instantiating model of type "
						+ type + " - no configuration object", e);
		}
		configuration.setSymbolPool(IConfiguration.MODEL_ID);
		HashMap modelMap = (HashMap) configuration.getParameter(_type);
		baseType = (String) modelMap.get(IConfiguration.MODEL_BASE_TYPE);

	}

	/** {@inheritDoc} */
	public String getName () {
		return _type;
	}

	protected void setModelType (String type) {
		_type = type;
	}

	/**
	 * returns empty array
	 * 
	 * @return init packages
	 */
	public String[] getPackageNames () {
		return new String[0];
	}

	/** {@inheritDoc} */
	public void setNodeID (Object comp, String id) {
	}

	/**
	 * default implementation returns the hexadecimal representation of <code>System.identityHashCode</code> of 
	 * the given element.
	 * 
	 * @see java.lang.System#identityHashCode(Object)
	 * 
	 */
	public String getNodeId (Object element) {
		int id = System.identityHashCode(element);
		String xid = Integer.toHexString(id).toUpperCase();
		return "00000000".substring(0, 8 - xid.length()) + xid;
	}

	/**
	 * {@inheritDoc} returns the qualified class name (i.e. without the package
	 * name) of the given element
	 */
	public String getNodeName (Object element) {
		String name = element.getClass().getName();
		return name.substring(name.lastIndexOf('.') + 1);
	}

	/** {@inheritDoc} */
	public String getDefaultAliasPrefix () {
		return "";
	}

	/** {@inheritDoc} */
	public ITreeNodeWalker getTreeWalker () {
		return treeNodeWalker;
	}

	/** {@inheritDoc} */
	public INodeLocator getNodeLocator () {
		return locator;
	}

	/** {@inheritDoc} */
	public void addModelChangeListener (IModelChangeListener listener) {
		if (listener != null) {
			changeListenerList.add(listener);
		}
	}

	/** {@inheritDoc} */
	public void removeModelChangeListener (IModelChangeListener listener) {
		if (listener != null) {
			changeListenerList.remove(listener);
		}
	}

	/**
	 * fire a ModelChangeEvent notification for this model
	 * 
	 * @param mce
	 *            the {@link ModelChangeEvent} The method triggered will vary
	 *            depending on the event type: NODE_INSERTED, NODE_REMOVED,
	 *            NODE_MODIFIED
	 */
	protected void fireModelChangeEvent (ModelChangeEvent mce) {
		int eventType = mce.getEventType();
		switch (eventType) {
		case ModelChangeEvent.NODE_INSERTED: {
			for (Iterator iter = changeListenerList.iterator(); iter.hasNext();) {
				((IModelChangeListener) iter.next()).nodeInserted(mce);
			}
			break;
		}
		case ModelChangeEvent.NODE_REMOVED: {
			for (Iterator iter = changeListenerList.iterator(); iter.hasNext();) {
				((IModelChangeListener) iter.next()).nodeRemoved(mce);
			}
			break;
		}
		case ModelChangeEvent.NODE_MODIFIED: {
			for (Iterator iter = changeListenerList.iterator(); iter.hasNext();) {
				((IModelChangeListener) iter.next()).nodeModified(mce);
			}
			break;
		}
		}
	}

	protected void setFilters () {
		configuration.setSymbolPool(IConfiguration.FILTER_ID);
		HashMap filterMap = (HashMap) configuration.getParameter(_type);
		if (filterMap != null) {
			String filterClass = (String) filterMap
					.get(IConfiguration.FILTER_CLASSNAME_ATTRIBUTE);

			if (filterClass != null) {
				INodeFilter filter = null;
				try {
					filter = (INodeFilter) ClassLoaderCache.getDefault()
							.classForName(filterClass).newInstance();
				} catch (Exception e) {
					Utils.println(IReporter.SYSTEM_NONFATAL, e);
				}
				if (filter != null) {
					treeNodeWalker.addNodeFilter(filter);
				}
			} else {
				String names =
				(String) filterMap
						.get(IConfiguration.FILTER_NODENAMES_ATTRIBUTE);
				if (names != null && names.length() > 0) {
					NodeNameFilter nameFilter = new NodeNameFilter(this, names);
					treeNodeWalker.addNodeFilter(nameFilter);
				} else {
					String ids =
					(String) filterMap
							.get(IConfiguration.FILTER_NODEIDS_ATTRIBUTE);
					if (ids != null && ids.length() > 0) {
						NodeIdFilter idFilter = new NodeIdFilter(this, ids);
						treeNodeWalker.addNodeFilter(idFilter);
					}
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.model.IModel#getBaseType()
	 */
	public String getBaseType() {
		return baseType;
	}
		
	// These next methods are stubs.
	public int getOrder (Object head) {
		return 1;
	}
	
} // AbstractModel
