/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import org.eclipse.actf.util.Utils;

/**
 * The concrete class for the description of a single event execution point target at which validations 
 * are to be triggered. This specialized class defines the event listener class and method name to be used 
 * when the method execution point class is generated.
 * 
 * @see #EventExecutionPointTarget(String, boolean, int)
 *  
 * @author John Buslawski
 *
 */
public class EventExecutionPointTarget extends ExecutionPointTarget
{

	protected String listenerName;

	protected String methodName;

	/**
	 * Create event-based execution point target. The target class name is the name of the class 
	 * that will trigger the generation of the containing execution point and start the validation.
	 *   
	 * Execution point targets can be polymorphic and apply to other classes in the target class
	 * hierarchy or just to that specific target class. 
	 * 
	 * Event-based execution points define the name of the listener class and the method on that
	 * listener on which the containing execution point will be generated.
	 * 
	 * @param targetClassName - name of class whose instances are to be captured
	 * @param listenerName - name of the listener
	 * @param methodName - listener method at which an execution point is formed
	 * @param polymorphic - determines if this execution point target applies to other
	 * 			classes in the target's hierarchy
	 */
	public EventExecutionPointTarget (String targetClassName,
										String listenerName, String methodName,
										boolean polymorphic) {
		super(targetClassName, polymorphic, ExecutionPointDescriptor.EVENT_EXECUTION_POINT);
		setMethodName(methodName);
		setListenerName(listenerName);
	}

	/**
	 * 
	 * @return name of method whose invocation triggers a validation
	 */
	public String getMethodName () {
		return methodName;
	}

	/**
	 * 
	 * @param methodName - name of method whose invocation triggers a validation
	 */
	public void setMethodName (String methodName) {
		this.methodName = methodName;
	}

	/**
	 * 
	 * @return names of types for a method whose invocation triggers a validation
	 */
	public String getListenerName () {
		return listenerName;
	}

	public void setListenerName (String listenerName) {
		this.listenerName = listenerName == null ? "" : listenerName;
	}

	public void printTarget (int indent) {
		String singleTab = "\t";
		String tab = "";
		for (int i = 0; i < indent; i++) {
			tab += singleTab;
		}
		Utils.println(Utils.PROCESS_ALL, tab
				+ "Details for ExecutionPointTarget - ");
		Utils.println(Utils.PROCESS_ALL, tab + "  toString() - "
				+ this.toString());
		Utils.println(Utils.PROCESS_ALL, tab + "  Type - "
				+ getType());
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  Target class name - " + getTargetClassName());
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  Listener class name - " + getListenerName());
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  Listener method name - " + getMethodName());
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  isPointcutExpression - " + isPointcutExpression());
		Utils.println(Utils.PROCESS_ALL, tab + "  polymorphic - "
				+ getPolymorphic());
	}
}
