/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

/**
 * the base class for the description of single execution point target at which validations are to be triggered.
 * Instances of execution point targets are contained as a collection in an execution point descriptor.
 * The execution point target, along with some information from the execution point descriptor, is 
 * used to create instances of execution points. Execution points are the run-time embodiment of the 
 * execution point targets.
 * 
 * <p>Currently, there are three execution point targets that are supported:
 * 
 * <p><ul>
 * <li>method - descriptors that describe execution points that are constructor or method invocations
 * <li>event - descriptors that describe execution points that occur when an event is fired
 * <li>property - descriptors that describe execution points that specify that a property has changed
 * </ul>
 * 
 * <p>In ACTF, the first type of execution point descriptor is only meaningful in the context of an 
 * aspect-oriented programming environment in which validation code can be woven into the target class 
 * at the appropriate point of invocation. The other types can be implemented via listeners and proxies 
 * handled via the Java Reflection API.
 *
 *  <p><b>Note</b>: The API and functionality of tihs class and its subclasses is incomplete.
 *  
 *  @Asee ExecutionPoint
 * @author John Buslawski
 *
 */
public abstract class ExecutionPointTarget
{

	protected String targetClassName;

	protected int type;

	protected boolean isPolymorphic = false;

	protected boolean isPointcutExpression;

	/**
	 * Create the parent execution point target. The target class name is the name of the class 
	 * that will trigger the generation of the containing execution point and start the validation.
	 *   
	 * Execution point targets can be polymorphic and apply to other classes in the target class
	 * hierarchy or just to that specific target class. 
	 * 
	 * The type of execution point is either method-based, defined by a <code>raven:method</code> 
	 * element, or event-based, defined by a <code>raven:event</code> element.
	 *  
	 *  @see #ExecutionPointTarget(String, boolean, int)
	 *  @see #MethodExecutionPointTarget()
	 *  
	 * @param targetClassName - name of class whose instances are to be captured
	 * @param polymorphic - boolean that defines whether this execution point target applies to just
	 * 	this target or also to classes in it's hierarchy
	 * @param type - type of execution point descriptor
	 */
	public ExecutionPointTarget (String targetClassName, boolean polymorphic,
									int type) {
		setTargetClassName(targetClassName);
		setType(type);
		setPolymorphic(polymorphic);
	}

	/**
	 * 
	 * @return name of class whose instances may trigger validations
	 */
	public String getTargetClassName () {
		return targetClassName;
	}

	/**
	 * name of class whose instances may trigger validations. If an asterisk '*' appears anywhere 
	 * in this name, the name is treated as a pointcut expression for an aspect-oriented 
	 * programming (AOP) environment. All other parameters may be set to <code>null</code>.
	 * 
	 * @param targetClassName - name of class whose instances may trigger validations
	 */
	public void setTargetClassName (String targetClassName) {
		if (targetClassName.indexOf("*") >= 0) {
			isPointcutExpression = true;
		}
		this.targetClassName = targetClassName;
	}

	/**
	 * 
	 * @return type of execution point target
	 */
	public int getType () {
		return type;
	}

	/**
	 * 
	 * @param type - type of execution point target 
	 * @throws IllegalArgumentException if type is not one of the pre-defined constants for this class
	 */
	public void setType (int type) {
		if (type < ExecutionPointDescriptor.METHOD_EXECUTION_POINT
				|| type > ExecutionPointDescriptor.PROPERTYCHANGED_EXECUTION_POINT) { throw new IllegalArgumentException("Unknown type for execution point descriptor"); }
		this.type = type;
	}

	/**
	 * returns whether or not the targetClassName actually represents a pointcut expression.
	 * 
	 * @return <code>true</code> if target class name represents a pointcut expression
	 * @see #setTargetClassName(String)
	 */
	public boolean isPointcutExpression () {
		return isPointcutExpression;
	}

	/**
	 * 
	 * @return whether this execution point target is polymorphic or not
	 */
	public boolean getPolymorphic () {
		return isPolymorphic;
	}

	/**
	 * 
	 * @param polymorphic - whether this execution point descriptor is polymorphic
	 */
	public void setPolymorphic (boolean polymorphic) {
		isPolymorphic = polymorphic;
	}
}
