/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.regex.Pattern;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.IGuiModel;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.traverse.ITreeNodeWalker;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.PrintWriterReporter;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.doc.rulebase.ValidationDocumentRuleBase;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.validators.AbstractValidationContext;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.IValidator;
import org.eclipse.actf.validation.validators.ValidationContextConstants;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.ToolBar;

/**
 * FocusValidationProcess handles all focus validation processes.
 * 
 * @author John Buslawski
 */
public class FocusValidationProcess extends AbstractValidationProcess {

	private String[] defaultNonTraversableComposites = {
			"org.eclipse.ui.forms.widgets.AbstractHyperlink",
			"org.eclipse.swt.custom.AnimatedProgress",
			"org.eclipse.swt.custom.CLabel",
			"org.eclipse.ui.forms.widgets.FormText",
			"org.eclipse.swt.custom.StyledText",
			"org.eclipse.swt.custom.TableCursor",
			"org.eclipse.swt.custom.CCombo", "org.eclipse.swt.widgets.Combo",
			"org.eclipse.swt.widgets.Table",
			"org.eclipse.swt.widgets.TableTree", "org.eclipse.swt.widgets.Tree" };

	private String[] defaultNonfocusableCtrlNames = {
			"org.eclipse.swt.widgets.Label", "org.eclipse.swt.custom.CLabel",
			"org.eclipse.swt.widgets.ProgressBar",
			"org.eclipse.swt.widgets.Canvas",
			"org.eclipse.swt.widgets.Composite",
			"org.eclipse.swt.widgets.Shell", "org.eclipse.swt.widgets.Group" };

	ArrayList arrowTraversableList = new ArrayList();

	ArrayList nonTraversableComposites = new ArrayList();

	ArrayList nonFocusableCtrlNames = new ArrayList();

	HashMap compositeCtrlList = new HashMap();

	ValidationDocumentRuleBase valDocRB = null;

	public FocusValidationProcess(IRuleBase iRuleBase, ExecutionPoint execPoint,
			IValidationContext context) {
		setRuleBase(iRuleBase);
		setExecutionPoint(execPoint);
		setValidationContext(context);
	}

	protected FocusValidationProcess() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.validation.core.AccesibilityValidationProcess#init()
	 */
	public void init() throws ValidationProcessException {
		if (valDocRB == null) {
			if (ruleBase != null
					&& ruleBase.getClass().isAssignableFrom(
							ValidationDocumentRuleBase.class)) {
				valDocRB = (ValidationDocumentRuleBase) getRuleBase();
			} else {
				Utils.println(IReporter.SYSTEM_NONFATAL,
						"Incompatible IRuleBase type in FocusValidationProcess.init(). "
								+ "IRuleBase name: " + getRuleBase().getName());
				throw new ValidationProcessException(
						"Unknown IRuleBase type in FocusValidationProcess.init(). ");
			}
		}
		// Build the remainder of the IValidationContext if necessary. The
		// ICodeProcessor and
		// IModel are in the IRuleBase.
		if (context.getParameter(ValidationContextConstants.CODE_PROCESSOR) == null) {
			context.setParameter(ValidationContextConstants.CODE_PROCESSOR,
					valDocRB.getCodeProcessor());
		}
		if (context.getParameter(ValidationContextConstants.MODEL) == null) {
			context.setParameter(ValidationContextConstants.MODEL,
					valDocRB.getModel());
		}
		if (context.getParameter(ValidationContextConstants.CONFIGURATION) == null) {
			try {
				IRuntimeContext runtimeContext = RuntimeContextFactory
						.getInstance().getRuntimeContext();
				IConfiguration config = runtimeContext
						.getConfiguration();
				context.setParameter(ValidationContextConstants.CONFIGURATION,
						config);
			} catch (ConfigurationException e) {
				// Could not obtain a Configuration. We can't do much here
				// but quit.
				throw new RuntimeException(e.getMessage(), e);
			}
		}
		if (context
				.getParameter(ValidationContextConstants.VALIDATED_ELEMENTS) == null) {
			context.setParameter(
					ValidationContextConstants.VALIDATED_ELEMENTS,
					new HashSet());
		}
		if (nonFocusableCtrlNames.isEmpty()) {
			for (int i = 0; i < defaultNonfocusableCtrlNames.length; i++) {
				nonFocusableCtrlNames.add(defaultNonfocusableCtrlNames[i]);
			}
		}
		if (nonTraversableComposites.isEmpty()) {
			for (int i = 0; i < defaultNonTraversableComposites.length; i++) {
				nonTraversableComposites
						.add(defaultNonTraversableComposites[i]);
			}
		}
		// TODO need to estimate the amount of work to do. This may involve
		// complete traversal of the hierarchy.
		calculateWork(execPoint, ruleBase, context);
		initProgressEstimates();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.actf.validation.core.AccessibilityValidationProcess#run()
	 */
	public void run() throws ValidationProcessException {
		validate(execPoint, ruleBase, context);
	}

	/**
	 * validate the objects in the tree with the given target as the root. All
	 * nodes should be instances of the appropriate type(s), which includes the
	 * primary type appropriate for the model and, optionally, a type
	 * that provides additional features and properties of the primary type,
	 * called its context.
	 * 
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 * 
	 * @throws ValidationProcessException
	 */
	public void validate(ExecutionPoint execPoint, IRuleBase iRuleBase,
			IValidationContext context) throws ValidationProcessException {
		IReporter reporter = null;
		if (isTerminated()) {
			return;
		}
		Object target = execPoint.getTarget();
		if (target != null) {
			try {
				IModel model = (IModel) context
						.getParameter(ValidationContextConstants.MODEL);
				if (model instanceof IGuiModel) {
					((IGuiModel) model).requestFocusFor(target);
				}
				reporter = (IReporter) context
						.getParameter(ValidationContextConstants.REPORTER);
				reporter.startReport(target);
				IConfiguration config = (IConfiguration) context
						.getParameter(ValidationContextConstants.CONFIGURATION);
				reporter.report(IReporter.INFO,
						"Beginning validation for target: "
								+ target.getClass().getName() + "...");
				ICodeProcessor codeProcessor = (ICodeProcessor) context
						.getParameter(ValidationContextConstants.CODE_PROCESSOR);
				if (codeProcessor.getScript("init") != null) {
					codeProcessor.processCode(new String[] { "init" });
				}
				codeProcessor.setLocal("rootComponent", target, "true");
				validateComponent(target, execPoint, iRuleBase, context);
				reporter.report(IReporter.INFO,
						"Validation for component with id "
								+ target.getClass().getName() + " complete.");
				if (reporter instanceof PrintWriterReporter) {
					PrintWriterReporter pwr = (PrintWriterReporter) reporter;
					reporter.report(IReporter.INFO, "Results of validation: "
							+ pwr.getErrorCount() + " errors, "
							+ pwr.getWarningCount() + " warnings");
					pwr.resetCount();
				}
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_FATAL, e);
			} finally {
				reporter.flush();
				reporter.endReport();
			}
		} else {
			throw new ValidationProcessException(
					"No target object to validate.");
		}
	}

	/**
	 * used to validate single components.
	 * 
	 * @param target --
	 *            the target object to validate
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 * 
	 * @throws ValidationProcessException
	 */
	public void validateComponent(Object target, ExecutionPoint execPoint,
			IRuleBase iRuleBase, IValidationContext context)
			throws ValidationProcessException {
		IModel model = (IModel) context
				.getParameter(ValidationContextConstants.MODEL);
		IConfiguration config = (IConfiguration) context
				.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		boolean validatingHidden = config
				.getBooleanParameter(ValidationConstants.VALIDATE_HIDDEN_KEY);
		boolean isHiddenComponent = !(model instanceof IGuiModel)
				|| !((IGuiModel) model).isVisible(target);
		if (validatingHidden || !isHiddenComponent) {
			String valName = execPoint.getValidatorClassName();
			IRuleSet[] ruleSets = iRuleBase
					.getRuleSetsForExecutionPoint(execPoint.getName());
			// for each IRuleSet that is enabled
			// get the IValidator name from the ExecutionPoint and create it
			// invoke it with the target object from the ExecutionPoint
			for (int i = 0; i < ruleSets.length; i++) {
				if (ruleSets[i].isEnabled()) {
					IValidator validator;
					try {
						validator = (IValidator) Class.forName(valName).getConstructor(
							new Class[] { IRuleSet.class}).newInstance(new Object[] { ruleSets[i]});
					} catch (Exception e) {
						throw new ValidationProcessException(
								"Unable to create IValidator named:  " + valName);
					}
					validator.validate(context, target);
					incrementProgress();
				}
			}
			// If the parent is visible then validate it's children, if not then
			// skip them.
			// Since we're validating, set the validate flag to true.
			validateChildren(target, execPoint, iRuleBase, context, true);
		} else if (isHiddenComponent) {
			Utils.println(Utils.PROCESS_INFO,
					"Skipping hidden component of type "
							+ target.getClass().getName());
		}
	}

	/**
	 * Validate the child components of the given target. The TreeWalkerFactory
	 * is used to get the appropriate tree walker, which fetches the children of
	 * this component. The traversal mode is also automatically set.
	 * 
	 * @param target --
	 *            the target object whose children are to be validated
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 * @param validate --
	 *            validate (true) or visit only
	 */
	protected void validateChildren(Object target, ExecutionPoint execPoint,
									IRuleBase iRuleBase, IValidationContext context, boolean validate)
			throws ValidationProcessException {
		IConfiguration config = (IConfiguration) context
				.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(IConfiguration.ACTF_ID);
		
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		if (model == null) {
			Utils.println(Utils.WARNINGS,
					"IValidationContext does not have a valid IModel.");
			throw new ValidationProcessException("IValidationContext does not have a valid IModel.");
		}
		
		final ITreeNodeWalker walker = model.getTreeWalker();
		final Object[][] childrenRes = new Object[][] { new Object[0] };
		final Object targetObj = target;
		if (walker != null) {
			Runnable runnable = new Runnable() {
				public void run() {
					try {
						childrenRes[0] = walker.getChildren(targetObj);
					} catch (InvalidComponentException e) {
						Utils.println(Utils.WARNINGS, "Component " + targetObj
								+ " is invalid - no children available");
					} catch (Exception e) {
						Utils.println(IReporter.SYSTEM_FATAL,
								"Unable to retrieve children for component", e);
					}
				}
			};
		} else {
			Utils.println(Utils.WARNINGS,
					"No tree walker available for component of type " + target.getClass().getName());
		}
		
		Object[] children = childrenRes[0];
		//		// The algorithm for validating the children is written below. The only
		// difficult
		// part here is the transition to another model. For instance,
		// Swing components
		// can be embedded in an SWT gui. In such a case, the validation process
		// needs to transition
		// to the new model (and new IRuleBase) for that target. The
		// ExecutionPointDescriptors
		// define the transition target regular expressions and the IRuleBase to
		// use.
		// 
		// Transition algorithm:
		//
		// for each child
		// if child is mapped to same IRuleBase as the ExecutionPoint target
		// validateComponent( child, execPoint, ruleBase, context );
		// else
		// if
		// RuleBaseManager.getExcutionPointDescriptor(name).getModeleTransition()
		// != null
		// if target object matches a transition regular expression
		// create an instance of that ExecutionPoint with component as target
		// object
		// get the IRuleBase from RuleBaseManager
		// ??? initialize a IValidationContext ??? or ??? use the same one ???
		// validateComponent( child, newExecPoint, newRuleBase, newContext )
		// else
		// log message that there is no matching model transition
		// else
		// log message that there is no model transition
		// endfor
		//
		RuleBaseManager rbMgr = RuleBaseManager
				.getInstance();
		for (int i = 0; !isTerminated() && i < children.length; ++i) {
			String childName = children[i].getClass().getName();
			IRuleBase[] ruleBases = RuleBaseManager.getInstance()
					.getRuleBasesByTarget(children[i]);
			boolean foundRuleBaseMatch = false;
			for (int j = 0; j < ruleBases.length; j++) {
				if (iRuleBase.getName().equalsIgnoreCase(ruleBases[j].getName())) {
					// just visiting or validating? only the validate flag knows
					// for sure
					if (validate) {
						validateComponent(children[i], execPoint, iRuleBase, context);
					} else {
						visitComponent(children[i], execPoint, iRuleBase, context);
					}
					foundRuleBaseMatch = true;
					break;
				}
			}
			if (!foundRuleBaseMatch) {
				// get the execution point descriptor for this execution point.
				// The execution point descriptor has the definitions for which
				// models
				// can be transitioned to based on whether the target object
				// class name matches
				// the regular expression.
				ExecutionPointDescriptor execPointDesc = iRuleBase
						.getExecutionPointDescriptor(execPoint.getName());
				if (execPointDesc == null) {
					Utils.println(Utils.WARNINGS,
							"Unable to find execution point descriptor for execuiton point named: " + execPoint.getName());
					throw new ValidationProcessException(
							"Unable to find execution point descriptor for execution point named: " + execPoint.getName());
				}
				boolean foundModelTransMatch = false;
				// get the model transitions.
				ModelTransition[] modelTrans = execPointDesc.getModelTransitions();
				// loop through each transition and test whether the regular
				// expression matches the
				// the child class name.
				if (modelTrans != null && modelTrans.length > 0) {
					for (int k = 0; k < modelTrans.length
							&& !foundModelTransMatch; k++) {
						if (pathMatchesExpression(children[i], modelTrans[k].getRegexp())) {
							foundModelTransMatch = true;
							// get the IRuleBase from RuleBaseManager
							IRuleBase newRB = rbMgr.getRuleBase(modelTrans[k].getRuleBaseName());
							if (newRB == null) {
								Utils.println(Utils.WARNINGS,
										"RuleBaseManager cannot find IRuleBase named: " + modelTrans[k].getRuleBaseName());
								// TODO throw exception or just log it
								throw new ValidationProcessException(
										"RuleBaseManager cannot find IRuleBase named: " + modelTrans[k].getRuleBaseName());
							}
							// get the execution point descriptor from the new
							// rule base using
							// the name from the model transition of the
							// original execution point.
							ExecutionPointDescriptor newExecPointDesc = newRB
									.getExecutionPointDescriptor(modelTrans[k].getExecutionPointName());
							if (newExecPointDesc == null) {
								Utils.println(
												Utils.WARNINGS,
												"Unable to find execution point descriptor for execution point named: "
														+ modelTrans[k].getExecutionPointName());
								// TODO throw exception or just log it??
								throw new ValidationProcessException(
										"Unable to find execution point descriptor for execution point named: "
												+ modelTrans[k].getExecutionPointName());
							}
							// Create an instance of that ExecutionPoint with
							// the name and IValidator class name
							// from the ExecutionPointDescriptor. All of the
							// other fields are the same and can
							// be taken from the original execution point.
							ExecutionPoint newExecPoint = execPoint
									.cloneExecutionPoint(newExecPointDesc
											.getName(), newExecPointDesc
											.getValidatorName(), execPoint);
							IValidationContext newContext = cloneContext(newRB, context);
							// just visiting or validating? only the validate
							// flag knows for sure
							if (validate) {
								validateComponent(children[i], newExecPoint, newRB, newContext);
							} else {
								visitComponent(children[i], newExecPoint, newRB, newContext);
							}
						}
					}
					if (!foundModelTransMatch) {
						Utils.println(
										Utils.WARNINGS,
										"No matching model transitions for child target [ " + childName
												+ " ] defined for execution point descriptor named: "
												+ execPointDesc.getName());
					}
				} else {
					Utils.println(Utils.WARNINGS,
							"No model transitions defined for execution point descriptor named: "
									+ execPointDesc.getName());
				}
			} // if( !foundRuleBaseMatch )
		} // for (int i = 0; !isTerminated() && i < children.length; ++i)
	} // validateChildren

	/**
	 * 
	 * @param iRuleBase
	 * @param context
	 * @return
	 * @throws ValidationProcessException
	 */
	public IValidationContext cloneContext(IRuleBase iRuleBase,
			IValidationContext context) throws ValidationProcessException {
		ValidationDocumentRuleBase valRB = null;
		if (iRuleBase != null && iRuleBase instanceof ValidationDocumentRuleBase) {
			valRB = (ValidationDocumentRuleBase) iRuleBase;
		} else {
			Utils.println(IReporter.SYSTEM_NONFATAL,
					"Incompatible IRuleBase type in FocusValidationProcess.copyContext(). "
							+ "IRuleBase name: " + iRuleBase.getName());
			throw new ValidationProcessException(
					"Unknown IRuleBase type in FocusValidationProcess.copyContext(). ");
		}
		// create a new IValidationContext from the old one
		AbstractValidationContext baseCtx = (AbstractValidationContext) context;
		IValidationContext newCtx = (IValidationContext) baseCtx.clone();
		newCtx.setParameter(ValidationContextConstants.MODEL, valRB
				.getModel());
		newCtx.setParameter(ValidationContextConstants.CODE_PROCESSOR, valRB
				.getCodeProcessor());
		newCtx.setParameter(ValidationContextConstants.VALIDATED_ELEMENTS,
				new HashSet());
		return newCtx;
	}

	/**
	 * used by child classes within the model to determine if their path
	 * matches a reqular expression that identifies a rule base. This is
	 * typically needed when a child component has a
	 * 
	 * @param target -
	 *            target object we are trying to match
	 * @param regexp -
	 *            the regular expression that identifies a rule base
	 * @return validator for this component
	 * 
	 */
	protected boolean pathMatchesExpression(Object target, String regexp) {
		String name = null;
		boolean matchFound = false;
		Pattern pattern = Pattern.compile(regexp);
		if (target != null) {
			Class cls = target.getClass();
			do {
				name = cls.getName();
				if (pattern.matcher(name).matches()) {
					matchFound = true;
					break;
				}
			} while ((cls = cls.getSuperclass()) != null);
		}
		return matchFound;
	}

	/**
	 * visit the objects in the tree with the given target as the root. This
	 * method is used primarily to walk the component tree starting with the
	 * target and estimate the amount of work (or number of components) when the
	 * tree is walked.
	 * 
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void calculateWork(ExecutionPoint execPoint, IRuleBase iRuleBase,
			IValidationContext context) throws ValidationProcessException {
		if (isTerminated()) {
			return;
		}
		Object target = execPoint.getTarget();
		if (target != null) {
			try {
				IModel model = (IModel) context
						.getParameter(ValidationContextConstants.MODEL);
				if (model instanceof IGuiModel) {
					((IGuiModel) model).requestFocusFor(target);
				}
				workEstimate = 0;
				visitComponent(target, execPoint, iRuleBase, context);
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_FATAL, e);
			}
		} else {
			throw new ValidationProcessException(
					"No target object to validate.");
		}
	}

	/**
	 * used to visit a single component.
	 * 
	 * @param target --
	 *            the target object to validate
	 * @param execPoint --
	 *            the ExecutionPoint that triggered this validation
	 * @param iRuleBase --
	 *            the IRuleBase that matches this target
	 * @param context --
	 *            the IValidationContext to use when evaluating rules
	 */
	protected void visitComponent(Object target, ExecutionPoint execPoint,
			IRuleBase iRuleBase, IValidationContext context)
			throws ValidationProcessException {
		IModel model = (IModel) context
				.getParameter(ValidationContextConstants.MODEL);
		IConfiguration config = (IConfiguration) context
				.getParameter(ValidationContextConstants.CONFIGURATION);
		config.setSymbolPool(ValidationConstants.VALIDATION_ID);
		boolean validatingHidden = config
				.getBooleanParameter(ValidationConstants.VALIDATE_HIDDEN_KEY);
		boolean isHiddenComponent = !(model instanceof IGuiModel)
				|| !((IGuiModel) model).isVisible(target);
		if (validatingHidden || !isHiddenComponent) {
			workEstimate++;
			// If the parent is visible then validate it's children, if not then
			// skip them.
			// Since we're visiting only, set the validate flag to false.
			validateChildren(target, execPoint, iRuleBase, context, false);
		} else if (isHiddenComponent) {
			Utils.println(Utils.PROCESS_INFO,
					"Skipping hidden component of type "
							+ target.getClass().getName());
		}
	}

	/*
	 * For any Composite, create a list of children that can be traversed by the
	 * keyboard
	 */
	public boolean computeKeyboardTraversalList(Object target)
			throws ValidationProcessException {
		Composite composite = null;
		IGuiModel model = null;
		if (context.getParameter(ValidationContextConstants.MODEL) != null) {
			model = (IGuiModel) context
					.getParameter(ValidationContextConstants.MODEL);
		} else {
			Utils.println(Utils.WARNINGS,
					"IValidationContext does not have a valid IModel.");
			throw new ValidationProcessException(
					"IValidationContext does not have a valid IModel.");
		}
		if (target.getClass().isAssignableFrom(Composite.class)) {
			composite = (Composite) target;
		} else {
			Utils.println(Utils.WARNINGS,
					"Target object is not assignable from class Composite.");
			throw new ValidationProcessException(
					"Target object is not assignable from class Composite.");
		}
		final Composite comp = composite;
		Control control = getFocusControl(comp, model);
		if (control == null) {
			Utils
					.println(
							Utils.WARNINGS,
							"Acquired null Control object from getFocusControl() in FocusValidationProcess.computeKeyboardTraversalList().");
			return false;
		}
		ArrayList focusCtrlList = new ArrayList();
		focusCtrlList.add(control);
		if (isArrowTraversable(control, model)) {
			getArrowTraversables(control, focusCtrlList);
		}
		if (traverse(control, model, SWT.TRAVERSE_TAB_NEXT)) {
			control = getFocusControl(composite, model);
			while ((control != null) && (!focusCtrlList.contains(control))) {
				if (isArrowTraversable(control, model)) {
					getArrowTraversables(control, focusCtrlList);
				} else {
					focusCtrlList.add(control);
				}
				traverse(control, model, SWT.TRAVERSE_TAB_NEXT);
				control = getFocusControl(composite, model);
			}
		}
		cacheControlList(composite, focusCtrlList);
		return true;
	}

	public void getArrowTraversables(Control ctrl, List ctrlList) {
		IGuiModel model = (IGuiModel) context
				.getParameter(ValidationContextConstants.MODEL);
		if ((getStyle(ctrl, model) & SWT.RADIO) != 0) {
			Control[] children = getChildren(ctrl, model);
			for (int i = 0; i < children.length; i++) {
				if (((getStyle(children[i], model) & SWT.RADIO) != 0)
						&& getEnabled(children[i], model)) {
					ctrlList.add(children[i]);
				}
			}
		} else if (ctrl instanceof ToolBar) {
			if (setFocus(ctrl, model)) {
				if (traverse(ctrl, model, SWT.TRAVERSE_ARROW_NEXT)) {
					Control focusedCtrl = getFocusControl(ctrl, model);
					while ((focusedCtrl != null)
							&& !ctrlList.contains(focusedCtrl)) {
						ctrlList.add(focusedCtrl);
						traverse(focusedCtrl, model,
								SWT.TRAVERSE_ARROW_NEXT);
						focusedCtrl = getFocusControl(ctrl, model);
					}
				} else {
					// traversal failed, log message here
					Utils.println(Utils.PROCESS_INFO,
							"Unable to traverse(SWT.TRAVERSE_ARROW_NEXT) on Control "
									+ ctrl.getClass().getName());
				}
			} else {
				// focus failed, log message
				Utils.println(Utils.PROCESS_INFO,
						"Unable to setFocus() to Control "
								+ ctrl.getClass().getName());
			}
		}
	}

	public void cacheControlList(Composite composite, List ctrlList) {
		// int compID = java.lang.System.identityHashCode(composite);
		if ((ctrlList.size() > 0)
				&& !listContainsString(composite.getClass().getName(),
						nonTraversableComposites)
				&& !compositeCtrlList.containsKey(composite)) {
			compositeCtrlList.put(composite, ctrlList);
		}
	}

	public boolean listContainsString(String str, List strList) {
		if (strList.size() == 0) {
			return false;
		}
		Iterator listIter = strList.iterator();
		while (listIter.hasNext()) {
			String listString = (String) listIter.next();
			if (str.equalsIgnoreCase(listString)) {
				return true;
			}
		}
		return false;
	}

	public boolean isControlInFocusableList(Control ctrl) {
		Set ctrlListSet = compositeCtrlList.entrySet();
		if (ctrlListSet.size() == 0) {
			return false;
		}
		Iterator listIter = ctrlListSet.iterator();
		while (listIter.hasNext()) {
			List ctrlList = (List) listIter.next();
			if (ctrlList.contains(ctrl)) {
				return true;
			}
		}
		return false;
	}

	public boolean isNonfocusableControl(Control ctrl) {
		return listContainsString(ctrl.getClass().getName(),
				nonFocusableCtrlNames);
	}

	public boolean isArrowTraversable(Control control,
			IGuiModel model) {
		if (arrowTraversableList.contains(control.getClass().getName())
				|| (getStyle(control, model) & SWT.RADIO) != 0) {
			return true;
		} else {
			return false;
		}
	}

	protected int getStyle(Control control, IGuiModel model) {
		final int[] returnedStyle = new int[1];
		final Control ctrl = control;
		model.invokeOnUIThread(new Runnable() {

			public void run() {
				try {
					returnedStyle[0] = ctrl.getStyle();
				} catch (SWTException e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Unable to execute getStyle() with Control "
									+ ctrl.getClass().getName(), e);
				}
			}
		});
		return returnedStyle[0];
	}

	protected Control getFocusControl(Control control,
			IGuiModel model) {
		final Control ctrl = control;
		final Object[] tmpCtrl = new Object[1];
		model.invokeOnUIThread(new Runnable() {

			public void run() {
				try {
					tmpCtrl[0] = ctrl.getDisplay().getFocusControl();
				} catch (SWTException e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Unable to execute getFocusControl() with Control "
									+ ctrl.getClass().getName(), e);
				}
			}
		});
		if (tmpCtrl[0] == null) {
			Utils
					.println(
							Utils.WARNINGS,
							"Acquired null Control object from getFocusControl() in FocusValidationProcess.computeKeyboardTraversalList().");
		}
		return (Control) tmpCtrl[0];
	}

	protected Control[] getChildren(Control control, IGuiModel model) {
		final ITreeNodeWalker walker = model.getTreeWalker();
		final Object[][] children = new Object[][] { new Object[0] };
		final Control ctrl = control;
		if (walker != null) {
			model.invokeOnUIThread(new Runnable() {

				public void run() {
					try {
						children[0] = walker.getChildren(ctrl);
					} catch (InvalidComponentException e) {
						Utils
								.println(
										Utils.WARNINGS,
										"Control "
												+ ctrl
												+ " is invalid - no children available.");
					} catch (Exception e) {
						Utils.println(IReporter.SYSTEM_FATAL,
								"Unable to retrieve children for control.", e);
					}
				}
			});
		} else {
			Utils.println(Utils.WARNINGS,
					"No tree walker available for component of type "
							+ control.getClass().getName());
		}
		Control[] ctrlChildren = new Control[children[0].length];
		for (int i = 1; i < children[0].length; i++) {
			ctrlChildren[i] = (Control) children[0][i];
		}
		return ctrlChildren;
	}

	protected boolean getEnabled(Control control, IGuiModel model) {
		final boolean[] enabled = new boolean[] { false };
		final Control ctrl = control;
		model.invokeOnUIThread(new Runnable() {

			public void run() {
				try {
					enabled[0] = ctrl.getEnabled();
				} catch (SWTException e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Unable to execute getEnabled() with Control "
									+ ctrl.getClass().getName(), e);
				}
			}
		});
		return enabled[0];
	}

	protected boolean setFocus(Control control, IGuiModel model) {
		final boolean[] focus = new boolean[] { false };
		final Control ctrl = control;
		model.invokeOnUIThread(new Runnable() {

			public void run() {
				try {
					focus[0] = ctrl.setFocus();
				} catch (SWTException e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Unable to execute setFocus() with Control "
									+ ctrl.getClass().getName(), e);
				}
			}
		});
		return focus[0];
	}

	protected boolean traverse(Control control, IGuiModel model,
			int traversal) {
		final int swtTraversal = traversal;
		final boolean[] traverse = new boolean[] { false };
		final Control ctrl = control;
		model.invokeOnUIThread(new Runnable() {

			public void run() {
				try {
					traverse[0] = ctrl.traverse(swtTraversal);
				} catch (SWTException e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Unable to execute traversal(int) with Control "
									+ ctrl.getClass().getName(), e);
				}
			}
		});
		return traverse[0];
	}
}
