/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.core;

import org.eclipse.actf.util.Utils;

/**
 * The concrete class for the description of single method execution point target at which validations 
 * are to be triggered. This specialized class defines the method name, parameter types and parameter
 * values to be used when the method execution point class is generated.
 *  
 * @author John Buslawski
 *
 */
public class MethodExecutionPointTarget extends ExecutionPointTarget
{

	protected String methodName;

	protected TargetArgument[] arguments;

	/**
	 * Create method-based execution point target. The target class name is the name of the class 
	 * that will trigger the generation of the containing execution point and start the validation.
	 *   
	 * Execution point targets can be polymorphic and apply to other classes in the target class
	 * hierarchy or just to that specific target class. 
	 * 
	 * Method-based execution points define the name of the method of the target class, and any
	 * arguments, on which the containing execution point will be generated.
	 * 
	 * 
	 * @param targetClassName - name of class whose instances are to be captured
	 * @param methodName - method at which an execution point is formed
	 * @param polymorphic - determines if this execution point target applies to other
	 * 			classes in the target's hierarchy
	 */
	public MethodExecutionPointTarget (String targetClassName,
										String methodName, boolean polymorphic) {
		this(targetClassName, methodName, null, polymorphic);
	}

	/**
	 * create a descriptor for a method execution point. The target class name is the name of the class whose 
	 * instances will be captured by this descriptor. If an asterisk '*' is found anywhere within the name, 
	 * the name is assumed to be a part of an aspect-oriented environment's pointcut expression. In this case, the type 
	 * should be set to <code>METHOD_EXECUTION_POINT</code> and all other parameters may be <code>nul</code>.
	 * 
	 * @param targetClassName - name of class whose instances are to be captured
	 * @param methodName - method at which an execution point is formed
	 * @param arguments - type name/value pairs representing the arguments passed to a execution point target
	 * @param polymorphic - determines if this execution point target applies to other
	 * 			classes in the target's hierarchy
	 */
	public MethodExecutionPointTarget (String targetClassName,
										String methodName,
										TargetArgument[] arguments,
										boolean polymorphic) {
		super(targetClassName, polymorphic, ExecutionPointDescriptor.METHOD_EXECUTION_POINT);
		setMethodName(methodName);
		setArguments(arguments);
	}

	/**
	 * 
	 * @return name of method whose invocation triggers a validation
	 */
	public String getMethodName () {
		return methodName;
	}

	/**
	 * 
	 * @param methodName - name of method whose invocation triggers a validation
	 */
	public void setMethodName (String methodName) {
		this.methodName = methodName;
	}

	/**
	 * 
	 * @return arguments for a method whose invocation triggers a validation
	 */
	public TargetArgument[] getArguments () {
		return arguments;
	}

	/**
	 * 
	 * @param arguments - the arguments for a method whose invocation triggers a validation
	 */
	public void setArguments (TargetArgument[] arguments) {
		this.arguments = arguments == null ? new TargetArgument[0] : arguments;
	}

	public void printTarget (int indent) {
		String singleTab = "\t";
		String tab = "";
		for (int i = 0; i < indent; i++) {
			tab += singleTab;
		}
		Utils.println(Utils.PROCESS_ALL, tab
				+ "Details for ExecutionPointTarget - ");
		Utils.println(Utils.PROCESS_ALL, tab + "  toString() - "
				+ this.toString());
		Utils.println(Utils.PROCESS_ALL, tab + "  Type - "
				+ getType());
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  Target class name - " + getTargetClassName());
		Utils.println(Utils.PROCESS_ALL, tab + "  Method name - "
				+ getMethodName());
		if (arguments != null && arguments.length > 0) {
			Utils.println(Utils.PROCESS_ALL, tab + "  Arguments ");
			for (int i = 0; i < arguments.length; i++) {
				Utils.println(Utils.PROCESS_ALL, tab + singleTab
						+ "  type - " + arguments[i].getType() + ", values - "
						+ arguments[i].getValue());
			}
		}else {
			Utils.println(Utils.PROCESS_ALL, tab
					+ "  No parameter type names for this method.");
		}
		Utils.println(Utils.PROCESS_ALL, tab
				+ "  isPointcutExpression - " + isPointcutExpression());
		Utils.println(Utils.PROCESS_ALL, tab + "  polymorphic - "
				+ getPolymorphic());
	}
}
