/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.codeproc;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.regex.Pattern;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.ValidationConstants;

/**
 * This abstract class implements the basic functionality of a ICodeProcessor
 * whether it be a compiler or interpreter processor. This class should not be
 * extended other than by AbstractCodeInterpreter.
 * Those wishing to implement this functionality in other contexts should extend
 * one of the latter two classes.
 * 
 * @author Mike Squillace
 */
public abstract class AbstractCodeProcessor implements ICodeProcessor {

	private static final Map ENTITY_REFERENCE_MAP = new HashMap();
	static {
		ENTITY_REFERENCE_MAP.put("&amp;", "&");
		ENTITY_REFERENCE_MAP.put("&quot;", "\"");
		ENTITY_REFERENCE_MAP.put("&lt;", "<");
		ENTITY_REFERENCE_MAP.put("&gt;", ">");
		ENTITY_REFERENCE_MAP.put("&apos;", "\'");
	}

	protected IConfiguration configuration;
	protected IModel model;
		protected String language, classpath;
	private Pattern visibilityPattern = Pattern
			.compile("__________abcdefghijklmnopqrstuvwxyyz");

	public AbstractCodeProcessor(String language, IModel model) {
		this.model = model;
		this.language = language;

		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		try {
			configuration = context.getConfiguration();
		} catch (ConfigurationException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		configuration.setSymbolPool(IConfiguration.ACTF_ID);
		classpath = configuration.getStringParameter(ValidationConstants.ACTF_CLASSPATH_KEY) + File.pathSeparator;
	} // ctor

		/**
	 * set the model for this processor
	 * 
	 * @param model --
	 *            model
	 */
	public void setModel(IModel model) {
		this.model = model;
	}

	/**
	 * return model used by this processor
	 * 
	 * @return model used by this processor
	 */
	public IModel getModel() {
		return model;
	}

	/** {@inheritDoc} */
	public void setClasspath(String cp) {
		classpath += cp;
	}

	/** {@inheritDoc} */
	public String getClasspath() {
		return classpath;
	}

	/**
	 * set the language identifier for this processor. This should match one of
	 * the keys in the 'scriptlang' pool of the actf.xml file.
	 * 
	 * @param lang --
	 *            language identifier for this processor
	 * @see "resources/actf.xml"
	 */
	public void setLanguage(String lang) {
		language = lang;
	}

	/**
	 * get the language identifier for this processor
	 * 
	 * @return language identifier for this processor
	 */
	public String getLanguage() {
		return language;
	}

	/**
	 * add the code in the given stream to be executed later by this processor
	 * 
	 * @param scriptName --
	 *            name of script
	 * @param scriptStream --
	 *            stream containing code for script
	 */
	public void addScript(String scriptName, InputStream scriptStream) {
		addScript(scriptName, getFileContents(scriptStream));
	}

	/**
	 * add the given scripts to this compiler for later compilation. The map is
	 * typically keyed by script name and contains values holding the code to be
	 * executed for event handlers.
	 * 
	 * @param scriptMap --
	 *            map keyed by script names with values containing code to be
	 *            executed
	 */
	public void addScripts(Map scriptMap) {
		Iterator iter = scriptMap.keySet().iterator();
		String id = null;
		while (iter.hasNext()) {
			id = (String) iter.next();
			addScript(id, (String) scriptMap.get(id));
		}
	}

	/**
	 * prepare code for execution. This default implementation resolves entity
	 * references and removes CDATA tags. Other preparations can be made by
	 * implementations of this method in sub-classes so long as this method is
	 * called first.
	 * 
	 * @param code --
	 *            code to be prepared for execution
	 * @return code readied for execution
	 */
	public String prepareCode(String code) {
		if (code != null) {
			StringBuffer codeBuff = new StringBuffer(code);
			// strip CDATA delimiters
			if (code.trim().startsWith("[CDATA[") && code.trim().endsWith("]]")) {
				int start = codeBuff.indexOf("[CDATA[");
				int end = codeBuff.lastIndexOf("]]");
				codeBuff.replace(start, start + "[CDATA[".length(), "");
				codeBuff.replace(end, end + "]]".length(), "");
			}
			// replace entity references
			Iterator iter = ENTITY_REFERENCE_MAP.keySet().iterator();
			String entityRef;
			int next;
			while (iter.hasNext()) {
				entityRef = (String) iter.next();
				while ((next = codeBuff.indexOf(entityRef)) != -1) {
					codeBuff.replace(next, next + entityRef.length(),
							(String) ENTITY_REFERENCE_MAP.get(entityRef));
				}
			} // next entity reference
			code = codeBuff.toString();
		}
		return code;
	} // prepareCode

	/** {@inheritDoc} */
	public void setIdVisibility(String visibility)
			throws IllegalArgumentException {
		try {
			if (visibility == null || visibility.equalsIgnoreCase("ALL")) {
				visibilityPattern = Pattern.compile("_?_?[a-zA-Z0-9]+\\w*");
			} else if (visibility.equalsIgnoreCase("NONE")) {
				visibilityPattern = Pattern
						.compile("__________abcdefghijklmnopqrstuvwxyyz");
			} else {
				visibilityPattern = Pattern.compile(visibility);
			}
		} catch (Exception e) {
			throw new IllegalArgumentException("Cannot compile pattern: "
					+ visibility + " - " + e.getClass().getName() + " - "
					+ e.toString());
		}
	} // setIdVisibility

	/**
	 * should be called from within <code>setLocal</code> to determine whether
	 * or not the id to be declared is visible.
	 * 
	 * @param id --
	 *            identifier to be tested
	 * @param vParam --
	 *            visibility parameter (value of rib:visible attribute if
	 *            present)
	 */
	protected boolean isIdVisible(String id, String vParam) {
		boolean matches = visibilityPattern.matcher(id).matches();
		return (matches && (vParam == null || !vParam.equalsIgnoreCase("false")))
				|| (!matches && (vParam != null && vParam
						.equalsIgnoreCase("true")));
	}

	protected String getFileContents(InputStream file) {
		BufferedReader br = null;
		StringBuffer sb = new StringBuffer();
		String line = null;
		try {
			br = new BufferedReader(new InputStreamReader(file));
			while ((line = br.readLine()) != null) {
				sb.append(line);
				sb.append('\n');
			}
		} catch (IOException ioe) {
			Utils.println(IReporter.SYSTEM_FATAL, ioe);
		}
		return sb.toString();
	} // getFileContents

	/** {@inheritDoc} */
	public String[] makeExpressionList(String str)
			throws IllegalArgumentException {
		final char delim = getExpressionDelimiter();
		List exprList = new LinkedList();
		int start = 0, end = 0;
		boolean match = true;
		char curChar;
		int count;
		try {
			while (end < str.length()) {
				curChar = str.charAt(end);
				switch (curChar) {
				case '(':
					count = 1;
					while (++end < str.length() && count > 0) {
						if (str.charAt(end) == '(') {
							++count;
						} else if (str.charAt(end) == ')') {
							--count;
						}
					}
					match = count == 0;
					break;
				case '[':
					count = 1;
					while (++end < str.length() && count > 0) {
						if (str.charAt(end) == '[') {
							++count;
						} else if (str.charAt(end) == ']') {
							--count;
						}
					}
					match = count == 0;
					break;
				case '{':
					count = 1;
					while (++end < str.length() && count > 0) {
						if (str.charAt(end) == '{') {
							++count;
						} else if (str.charAt(end) == '}') {
							--count;
						}
					}
					match = count == 0;
					break;
				case '\'':
					while (++end < str.length()
							&& (str.charAt(end) != '\'' || (str.charAt(end) == '\'' && str
									.charAt(end - 1) == '\\'))) {
						;
					}
					match = (end == str.length() && str.charAt(end - 1) == '\'')
							|| str.charAt(end) == '\'';
					break;
				case '\"':
					while (++end < str.length()
							&& (str.charAt(end) != '\"' || (str.charAt(end) == '\"' && str
									.charAt(end - 1) == '\\'))) {
						;
					}
					match = (end == str.length() && str.charAt(end - 1) == '\"')
							|| str.charAt(end) == '\"';
					break;
				default:
					if (str.charAt(end) == delim) {
						if (match) {
							exprList.add(str.substring(start, end).trim());
							end = start = end + 1;
						}
					} else {
						++end;
					}
					break;
				} // switch
			} // while
		} catch (RuntimeException e) {
			match = false;
		}
		if (match) {
			exprList.add(str.substring(start, end).trim());
		} else {
			throw new IllegalArgumentException(
					"Symbol mismatch in or other problem processing string: "
							+ str);
		}
		return (String[]) exprList.toArray(new String[0]);
	} // makeExpressionList
} // AbstractCodeProcessor
