/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.codeproc;

import java.io.InputStream;
import java.util.Map;

import org.eclipse.actf.model.IModel;

/**
 * responsible for processing any code within a validation document. A
 * ICodeProcessor may be implemented as one type:
 * 
 * <p>
 * <ul>
 * <li> a <code>org.eclipse.actf.validation.doc.scripting.AbstractCodeInterpreter</code>
 * which is responsible for interpreting code in a variety of scripting
 * languages
 * </ul>
 * 
 * @author Mike Squillace
 */
public interface ICodeProcessor {

	/**
	 * pool id for script languages pool - value is 'scriptlang'
	 */
	public static final String SCRIPTLANG_ID = "scriptlang";

	public static final String FACTORY_PROCESSOR_KEY = "factory.processor";

	/**
	 * set the model for this processor
	 * 
	 * @param model --
	 *            model
	 */
	public void setModel(IModel model);

	/**
	 * return model used by this processor
	 * 
	 * @return model used by this processor
	 */
	public IModel getModel();

	/**
	 * returns the classpath being used from within script code or to compile
	 * source code generated during argument evaluation. This classpath is not
	 * necessarily the same as that used by the ACTF validation engine.
	 * 
	 * @return classpath being used by underlying interpreter or compiler
	 * @see #setClasspath(String)
	 */
	public String getClasspath();

	/**
	 * set the classpath for this processor. The classpath is used in one of two
	 * ways, depending upon the type of processor being used:
	 * 
	 * <p>
	 * <ul>
	 * <li>For interpreters, the classpath will be used to define where classes
	 * are found from within a script code block. For example, it will be the
	 * classpath used to aid the underlying interpreter resolve
	 * <code>import</code> statements.
	 * <li>For compilers, it is the classpath used to resolve references at
	 * compilation time. This is especially important for code that is generated
	 * by ACTF for the evaluation of arguments or wrappers for event handlers.
	 * </ul>
	 * 
	 * <p>
	 * The classpath for a processor isnot necessarily the same as that used by
	 * ACTF validation engines. A default value is supplied in the
	 * ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY property in the
	 * raven.xml file.
	 * 
	 * @param cp -
	 *            classpath to use
	 * @see "resources/raven.xml"
	 */
	public void setClasspath(String cp);

	/**
	 * set the language identifier for this processor. This should match one of
	 * the keys in the 'scriptlang' pool in the actf.xml file.
	 * 
	 * @param lang --
	 *            language identifier for this processor
	 * @see "resources/actf.xml"
	 */
	public void setLanguage(String lang);

	/**
	 * get the language identifier for this processor
	 * 
	 * @return language identifier for this processor
	 */
	public String getLanguage();

	/**
	 * set the environment for this processor. This method is primarily to be
	 * called when a new processor is instantiated and is to have the same
	 * environment (e.g. local variables, executed code) as another processor;
	 * thus, it should be called in concert with <code>getEnvironment()</code>.
	 * 
	 * @param env --
	 *            environment of another processor retreaved via
	 *            <code>getEnvironment()</code>
	 */
	public void setEnvironment(Object env);

	/**
	 * used to retreave the environment of this processor so as to be passed to
	 * the <code>setEnvironment(Object)</code> of a newly instantiated
	 * processor
	 * 
	 * @return environment of this processor
	 * @see #setEnvironment(Object)
	 */
	public Object getEnvironment();

	/**
	 * set the visibility for symbols in the XML document. Visibility determines
	 * which (if any) of the symbols declared via the rib:id attribute will be
	 * visible in scripts or compiled classes. Options for this parameter
	 * include:
	 * 
	 * <p>
	 * <ul>
	 * <li>ALL -- all symbols defined via the rib:id attribute are visible
	 * <li>NONE -- none of the symbols defined via the rib:id attribute are
	 * visible
	 * <li> &lt;regExp&gt; -- a regular expression defining which rib:id
	 * attributes are visible
	 * </ul>
	 * 
	 * <p>
	 * The <code>setLocal</code> method of implementations of this interface
	 * should take care to set the environment according to the constraints of
	 * this property.
	 * 
	 * <p>
	 * <b>Note</b>: Not utilized in ACTF
	 * 
	 * 
	 * @param visibility --
	 *            one of "ALL", "NONE", or "&lt;reg_expression&gt;"
	 * @throws IllegalArgumentException
	 *             if argument cannot be processed
	 */
	public void setIdVisibility(String visibility)
			throws IllegalArgumentException;

	/**
	 * add the specified code as a script for this processor. This code may be
	 * executed as is upon invoking this method if the processor is an
	 * implementation of an interpreter or simply added to a body of code to be
	 * compiled later and executed upon initialization or instantiation of the
	 * compiled class.
	 * 
	 * @param code --
	 *            code to be added to this processor
	 */
	public void addScript(String code);

	/**
	 * add the script code with the specified script name to this processor for
	 * later execution
	 * 
	 * @param scriptName --
	 *            name or id of script
	 * @param scriptCode --
	 *            code to be executed
	 */
	public void addScript(String scriptName, String scriptCode);

	/**
	 * add the code in the given stream to be executed later by this processor
	 * 
	 * @param scriptName --
	 *            name of script
	 * @param scriptStream --
	 *            stream containing code for script
	 */
	public void addScript(String scriptName, InputStream scriptStream);

	/**
	 * add the given scripts to this processor for later execution or
	 * compilation. The map is keyed by script name and contains values holding
	 * the code to be executed or included in the execution.
	 * 
	 * @param scriptMap --
	 *            map keyed by script names with values containing code to be
	 *            executed
	 */
	public void addScripts(Map scriptMap);

	/**
	 * get the code associated with the given script name
	 * 
	 * @param scriptName --
	 *            name of script for which code is desired
	 * @return code of script with given name or <code>null</code> if name is
	 *         not valid
	 */
	public String getScript(String scriptName);

	/**
	 * get the initialization code for this processor. This is code typically
	 * hard-coded into the implementation to be run upon initialization of the
	 * interpreter or upon instantiation of a compiled class.
	 * 
	 * @return initialization code for this processor
	 */
	public String getInitCode();

	/**
	 * declare a new identifier to this processor for use in scripts.
	 * 
	 * <p>
	 * By specifying an override parameter "either "true" or "false"), the
	 * following behavior should result:
	 * <ul>
	 * <li>if id visibility is set to "ALL" and override is "false", the local
	 * is not set
	 * <li>if id visibility is set to "NONE" and override is "true", the local
	 * is set
	 * <li>if id visibility is determined by a regular expression and override
	 * is "true", then the id is set regardless of whether or not the id matches
	 * the set pattern
	 * <li>if id visibility is determined by a regular expression and overide
	 * is "false", the local id is not set regardless of whether or not the id
	 * matches the set pattern
	 * </ul>
	 * 
	 * <p>
	 * <b>Note</b>: Only first two parameters relevant to ACTF; always pass
	 * 'true' to vParam.
	 * 
	 * 
	 * @param name --
	 *            identifier for this object
	 * @param obj --
	 *            object being declared (may be <code>null</code>)
	 * @param vParam --
	 *            either "true" or "false", the rib:visible attribute value (may
	 *            also be <code>null</code>, in which case the behavior
	 *            specified via <code>setIdVisibility</code> will be adopted
	 * @see #setIdVisibility(String)
	 */
	public void setLocal(String name, Object obj, String vParam);

	/**
	 * get the default expression delimiter for the language. This will
	 * typically be either a comma (,) or semicolon (;). This can also be set
	 * via the 'exprdelim' property in the raven.xml file.
	 * 
	 * @return expression delimiter
	 * @see "resources/raven.xml"
	 */
	public char getExpressionDelimiter();

	/**
	 * split the specified string into "expressions" bounded by the delimiter
	 * returned by <code>getExpressionDelimiter</code>. Although what
	 * constitutes an expression is language-dependent, this method will insure
	 * that each String in the given list is consistent and complete. The
	 * following tokens can be recognized:
	 * 
	 * <p>
	 * <ul>
	 * <li>numeric and boolean literals
	 * <li>strings <code>"...", '...'</code>
	 * <li>parenthesized expressions <code>(...)</code> or expressions
	 * containing parenthesized expressions (e.g.
	 * <code>new java.awt.Dimension(150, 150)</code>
	 * <li>lists enclosed in square brackets (e.g.
	 * <code>[item1, item2, ...]</code>
	 * <li>hashes or other structures that use curly braces (e.g.
	 * <code>{name="Mike Squillace", ...}</code> or
	 * <code>new String[] {"a", "b", ...}</code>
	 * </ul>
	 * 
	 * <p>
	 * The default implementation in both
	 * <code>org.eclipse.actf.validation.doc.scripting.AbstractCodeInterpreter</code> and
	 * <code>org.eclipse.actf.validation.doc.scripting.BaseCodeCompiler</code> implement
	 * this method in the manner described above.
	 * 
	 * @param str --
	 *            string to be split
	 * @return array of String objects representing tokens in specified string
	 * @throws IllegalArgumentException
	 *             if symbol mismatch occurs
	 */
	public String[] makeExpressionList(String str)
			throws IllegalArgumentException;

	/**
	 * prepare code for execution or compilation (e.g. by resolving entity
	 * references or removing CDATA tags)
	 * 
	 * @param code --
	 *            code to be prepared
	 * @return code readied for execution or compilation
	 */
	public String prepareCode(String code);

	/**
	 * attempt to evaluate the given string as an argument of the specified type
	 * 
	 * @param arg --
	 *            argument to be evaluated
	 * @param returnType --
	 *            desired type of returned object
	 * @return newly created object resulting from evaluation
	 * @throws EvaluationException
	 *             if evaluation was not successful
	 */
	public Object evaluateArguments(String arg, Class returnType)
			throws EvaluationException;

	/**
	 * attempt to evaluate the given string as a set of arguments of objects
	 * corresponding to the specified types.
	 * 
	 * @param args --
	 *            arguments to be evaluated
	 * @param paramTypes --
	 *            desired types of returned objects
	 * @return newly created objects resulting from evaluation
	 * @throws EvaluationException
	 *             if evaluation was not successful
	 */
	public Object[] evaluateArguments(String args, Class[] paramTypes)
			throws EvaluationException;

	/**
	 * tests whether or not this processor is a compiler.
	 * 
	 * @return <code>true</code> if this processor is a compiler
	 *         implementation, <code>false</code> otherwise
	 */
	public boolean isCompilerProcessor();

	/**
	 * process the code according to the implementation of this processor (i.e.
	 * whether it is a code compiler or code interpreter.
	 * 
	 * <p>
	 * This method may perform one of two processes:
	 * <p>
	 * <ul>
	 * <li>execute the code with the given script name. The environment for
	 * this process will reflect the state change caused by this execution.
	 * <li>compile and execute the code of this ICodeProcessor.
	 * </ul>
	 * </p>
	 * 
	 * @param params --
	 *            parameters for execution or for compilation
	 * @return result of process (<code>true</code> for success,
	 *         <code>false</code> otherwise)
	 */
	public boolean processCode(String[] params);
} // ICodeProcessor
