/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.rulebase;

import java.io.InputStream;
import java.net.URL;
import java.util.Map;
import java.util.Properties;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.DefaultModelFactory;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.IResourceLocator;
import org.eclipse.actf.validation.core.ExecutionPointDescriptor;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessorFactory;
import org.eclipse.actf.validation.rulebase.IDocumentRuleBase;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rulebase.IRuleBaseFactory;
import org.eclipse.actf.validation.rulebase.MalformedRuleBaseException;
import org.eclipse.actf.validation.rules.IRuleFactory;
import org.eclipse.actf.validation.rules.IRuleMapping;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.w3c.dom.Element;

/**
 * implementation responsible for processing validation documents in ACTF.
 * 
 * @author John Buslawski
 * 
 */
public class DocumentRuleBaseBuilder implements IRuleBaseFactory
{

	private ValidationDocumentRuleBase ruleBase;
	private ValidationDocumentRuleBaseMetadata rbMetadata;
	private IConfiguration config;
	private URL docUrl;
	
	public DocumentRuleBaseBuilder() {
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			config = context.getConfiguration();
		} catch (ConfigurationException e) {
			throw new RuntimeException("Could not form configuration object for rulebase construction", e);
		}
	}

	public URL getDocumentLocation() {
		return docUrl;
	}

	private ICodeProcessor createCodeProcessor(String scriptLang, IModel model) {
		ICodeProcessor codeProc = null;
		config.setSymbolPool(IConfiguration.ACTF_ID);
		if (scriptLang != null) {
			try {
				ICodeProcessorFactory cpf = (ICodeProcessorFactory) config
						.getClassParameter(ICodeProcessor.FACTORY_PROCESSOR_KEY).newInstance();
				codeProc = cpf.getCodeProcessor(ruleBase
						.getScriptingLanguage(), model);
				if (codeProc == null) {
					Utils.println(IReporter.USER,
									"IRuleBase must specify a language in the <actf:scriptlang> attribute.");
				} else if (model == null) {
					Utils.println(IReporter.USER,
									"IRuleBase must specify a <actf:model> attribute.");
				}
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_FATAL, "Error creating code processor", e);
			}
		}
		return codeProc;
	} // setCodeProcessor

	private IModel createModel(String model) {
		IModel thisModel = null;
		if (model != null) {
			try {
				thisModel = DefaultModelFactory.getInstance().resolveModel(
						model);
				if (thisModel == null) {
					Utils
							.println(IReporter.USER,
									"IRuleBase must specify a <actf:model> attribute.");
				}
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_FATAL, e);
			}
		}
		return thisModel;
	} // createModel

	/** {@inheritDoc} */
	public IRuleBase buildRuleBase(Object data) throws MalformedRuleBaseException {
		if (data != null && data instanceof URL) {
			docUrl = (URL) data;
			try {
				ReflectionValidationDocument valDoc = null;
				InputStream docStream = docUrl.openStream();
				if (docStream != null) {
					valDoc = new ReflectionValidationDocument(docStream);
					docStream.close();
					ruleBase = new ValidationDocumentRuleBase();
					parseRuleBaseAttributes(valDoc);

					IModel model = createModel(ruleBase.getModelType());
					if (model == null) {
						throw new MalformedRuleBaseException("Unable to build IRuleBase: no registered model of type " + ruleBase.getModelType());
					} else {
						ICodeProcessor codeProcessor = createCodeProcessor(ruleBase.getScriptingLanguage(), model);
						ruleBase.setModel(model);
						ruleBase.setCodeProcessor(codeProcessor);
						
						parseRuleBaseMetadata(valDoc);
						parseRuleSets(valDoc);
						parseExecutionPointDescriptors(valDoc);

						if (codeProcessor != null) {
							initCodeProcessor();
						}
}
				}
			} catch (Exception e) {
				throw new MalformedRuleBaseException("Problem parsing rulebase at URL " + docUrl, e);
			}
		}

		return ruleBase;
	}

	public IRuleBase getRuleBase() {
		return ruleBase;
	}

	private void parseRuleBaseAttributes(IValidationDocument valDoc) {
		ruleBase.setName(valDoc.getDocumentName());
		ruleBase.setEnabled(valDoc.getEnable() != null && valDoc.getEnable().equals("true"));
		ruleBase.setModelType(valDoc.getModelType());
		ruleBase.setScriptingLanguage(valDoc.getScriptLanguage());
	}

	private void parseRuleBaseMetadata(IValidationDocument valDoc)  throws MalformedRuleBaseException {
		rbMetadata = new ValidationDocumentRuleBaseMetadata();
		rbMetadata.setAliases(parseAliases(valDoc.getAliasElements()));
		rbMetadata.setScripts(valDoc.getScriptElements());
		ruleBase.setMetadata(rbMetadata);
	}

	private Properties parseAliases(AliasElement[] aliases) {
		Properties aliasProps = new Properties();
		// Get the aliases for this model from the configuration object.
		// Any aliases defined in the Validation document that have the same name
		// will override the ones from the configuration object.
		String model = ruleBase.getModelType();
		Map aliasMap = config.getSymbolPoolContents(model
				+ IConfiguration.MODEL_POOL_ID_DELIMITER
				+ IConfiguration.ALIASES_ID);
		if (aliasMap != null) {
			aliasProps.putAll(aliasMap);
		}
		
		for (int a = 0; a < aliases.length; ++a) {
			AliasElement alias = aliases[a];
			String name = alias.getName();
			String value = alias.getValue();
			String text = alias.getTextTrim();
			if (name != null) {
				if (text.length() > 0) {
					value = text;
				} else if (value == null) {
					value = ruleBase.getModel().getDefaultAliasPrefix() + "." + name;
				} else if (value.endsWith(".")) {
					value += name;
				}
				if (((Map) aliasProps).containsKey(name)) {
					aliasProps.remove(name);
				}
				aliasProps.setProperty(name, value);
				aliasMap.put(name, value);
			} else {
				Utils.println(Utils.WARNINGS,"Each <actf:alias> tag must contain a 'name' attribute");
			}
		} // next alias
		
		return aliasProps;
	}

	private void parseRuleSets(IValidationDocument valDoc) {
		Element[] rsList = valDoc.getRuleSetElements();
		for (int i = 0; i < rsList.length; ++i) {
			Element ruleSetElem = rsList[i];
			ValidationDocumentRuleSet ruleSet = new ValidationDocumentRuleSet(ruleBase);
			parseRuleSetAttributes(ruleSetElem, ruleSet);
			
			IRuleMapping ruleMapping = ruleSet.getRuleFactory().getRuleMapping(ruleSetElem);
			ruleSet.setRuleMap(ruleMapping);
			ruleBase.addRuleSet(ruleSet);
		}
	}

	private void parseRuleSetAttributes(Element rsElem, IRuleSet ruleSet) {
		ruleSet.setName(rsElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.NAME));
		ruleSet.setExecutionPoint(rsElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, "executionPoint"));
		
		String enable = rsElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.ENABLE);
		ruleSet.setEnabled(enable != null || enable.equals("true"));
		
		String rfName = rsElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, "ruleFactory");
		IRuleFactory ruleFactory = null;
		try {
			ruleFactory = (IRuleFactory) Class.forName(rfName).newInstance();
			ruleFactory.setRuleBase(ruleBase);
			ruleFactory.setRuleSet(ruleSet);
			// TODO should be represented by separate interface
			if (ruleFactory instanceof ReflectionDocumentRuleFactory) {
				((ReflectionDocumentRuleFactory) ruleFactory).setCodeProcessor(ruleBase.getCodeProcessor());
			}
		} catch (Exception e) {
			throw new RuntimeException("Unknown ruleFactory class: " + rfName, e);
		}
		ruleSet.setRuleFactory(ruleFactory);
	}

	private void parseExecutionPointDescriptors(IValidationDocument valDoc) {
		ExecutionPointDescriptor[] execDescriptors = valDoc.getExecutionPointDescriptors();
		for (int i = 0; i < execDescriptors.length; ++i) {
			execDescriptors[i].setRuleBaseName(ruleBase.getName());
			ruleBase.addExecutionPointDescriptor(execDescriptors[i]);
		}
	}

	private void initCodeProcessor() {
		Utils.println(Utils.PROCESS_ALL, "Initializing ICodeProcessor.");
		ICodeProcessor codeProcessor = ruleBase.getCodeProcessor();
		ScriptElement[] scriptList = rbMetadata.getScripts();
		
		codeProcessor.addScript(codeProcessor.getInitCode());
		for (int s = 0; s < scriptList.length; ++s) {
			ScriptElement script = scriptList[s];
			String name = script.getName();
			Utils.println(Utils.ALL, "Evaluating script - " + (name == null ? "<nameless>" : name));

			String imports = script.getImports();
			String srcFile = script.getSourceFile();
			if ((name == null || name.length() == 0) && (srcFile == null || srcFile.length() == 0)) {
				codeProcessor.addScript(script.getText());
				Utils.println(Utils.ALL, "Script added - assignned temporary name " + script.getName());
			} else if (srcFile != null && !codeProcessor.isCompilerProcessor()) {
				try {
					IResourceLocator locator = RuntimeContextFactory.getInstance().getRuntimeContext().getResourceLocator();
					URL url = locator.getResource(srcFile);
					InputStream is = null;
					if (url != null) {
						is = url.openStream();
						codeProcessor.addScript( null, is);
						is.close();
						Utils.println(Utils.ALL, "Script added - " + script.getName());
					}
				} catch (Exception e) {
					Utils.println(IReporter.SYSTEM_FATAL,"Could not load source file " + srcFile , e);
				}
			} else if (name != null && !codeProcessor.isCompilerProcessor()) {
				codeProcessor.addScript(name, script.getText());
				Utils.println(Utils.ALL, "Script added - " + script.getName());
			} else if (codeProcessor.isCompilerProcessor()) {
				Utils.println(Utils.WARNINGS,
								"The actf:src and actf:name attributes are ignored for compiled languages");
				codeProcessor.addScript(script.getText());
				Utils.println(Utils.ALL, "Script added - " + name);
			}
			
			if (imports != null && imports.length() > 0 && !codeProcessor.isCompilerProcessor()) {
				Utils.println(Utils.WARNINGS,
								"The actf:imports attribute is ignored for non-compiled languages");
			}
		}
	}
/* TODO - this needs to be fixed so that there is no hard-coded path to the rulebase files	
	public static void main (String[] args) throws Exception {
		DocumentRuleBaseBuilder builder = new DocumentRuleBaseBuilder();
		builder.buildRuleBase(new File("c:/eclipse-constellation-321/eclipse/actf-0.1.0/com.ibm.haac.raven.model.java.core/resources/swing_accessibility.xml").toURL());
		System.out.println("rb:"+builder.getRuleBase());
	}
*/
}