/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.doc.rulebase;

import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.rulebase.IRuleBase;
import org.eclipse.actf.validation.rules.AndRule;
import org.eclipse.actf.validation.rules.ComponentCriteria;
import org.eclipse.actf.validation.rules.EqRule;
import org.eclipse.actf.validation.rules.FocusRule;
import org.eclipse.actf.validation.rules.IRule;
import org.eclipse.actf.validation.rules.IRuleFactory;
import org.eclipse.actf.validation.rules.IRuleMapping;
import org.eclipse.actf.validation.rules.IRuleSet;
import org.eclipse.actf.validation.rules.InstanceofRule;
import org.eclipse.actf.validation.rules.MaxRule;
import org.eclipse.actf.validation.rules.MinRule;
import org.eclipse.actf.validation.rules.NeqRule;
import org.eclipse.actf.validation.rules.OrRule;
import org.eclipse.actf.validation.rules.RangeRule;
import org.eclipse.actf.validation.rules.ReflexiveCriterion;
import org.eclipse.actf.validation.rules.ReflexiveRule;
import org.eclipse.actf.validation.rules.RegexpRule;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.eclipse.actf.validation.rules.ScriptRule;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

/**
 * The IRuleFactory for ACTF rules.
 * 
 * @author John Buslawski
 * 
 */
public class ReflectionDocumentRuleFactory implements IRuleFactory {

	protected static final Map ruleMap = new HashMap();
	static {
		ruleMap.put(RuleConstants.AND, AndRule.class.getName());
		ruleMap.put(RuleConstants.CRITERION, ReflexiveCriterion.class.getName());
		ruleMap.put(RuleConstants.EQ, EqRule.class.getName());
		ruleMap.put(RuleConstants.FOCUS, FocusRule.class.getName());
		ruleMap.put(RuleConstants.INSTANCEOF, InstanceofRule.class.getName());
		ruleMap.put(RuleConstants.MIN, MinRule.class.getName());
		ruleMap.put(RuleConstants.MAX, MaxRule.class.getName());
		ruleMap.put(RuleConstants.NEQ, NeqRule.class.getName());
		ruleMap.put(RuleConstants.OR, OrRule.class.getName());
		ruleMap.put(RuleConstants.RANGE, RangeRule.class.getName());
		ruleMap.put(RuleConstants.REGEXP, RegexpRule.class.getName());
		ruleMap.put(RuleConstants.REFLEXIVE, ReflexiveRule.class.getName());
		ruleMap.put(RuleConstants.SCRIPT, ScriptRule.class.getName());
		ruleMap.put(RuleConstants.SCRIPTTEST, ScriptRule.class.getName());
	}

	private ClassLoaderCache clCache = ClassLoaderCache.getDefault();
	private IRuleBase ruleBase;
	private ICodeProcessor codeProcessor;
	private Set polymorphicTypes = new HashSet();
	private Set nonPolymorphicTypes = new HashSet();
	private IValidationDocument validationDoc;
	private IRuleSet ruleSet;

	public ICodeProcessor getCodeProcessor () {
		return codeProcessor;
	}
	
	public void setCodeProcessor (ICodeProcessor codeProc) {
		codeProcessor = codeProc;
	}
	
	public IRuleBase getRuleBase () {
		return ruleBase;
	}
	
	public void setRuleBase (IRuleBase ruleBase) {
		this.ruleBase = ruleBase;
	}
	
	public IRuleSet getRuleSet () {
		return ruleSet;
	}
	
	public void setRuleSet (IRuleSet ruleSet) {
		this.ruleSet = ruleSet;
	}
	
	/** 
	 * {@inheritDoc}
	 * <p>For a <code>ReflectionDocument</code>,the mapping will consist of a Java type 
	 * mapped to a set of rules to be applied to instances of that type. The type is typically aliased and is the name 
	 * of the element defining a particular set of rules within a ruleset.
	 * 
	 * <p>Hence, the 'rulesetToken' should be an instance of <code>org.w3c.dom.Element</code>, 
	 * having a name that specifies the type for which rules represented by its child elements are to be applied.
	 *  
	 */
	public IRuleMapping getRuleMapping(Object rulesetToken) {
		// Map containing list of criteria for each class
		Map componentCriteriaMap = new HashMap();
		
		if (rulesetToken instanceof Element) {
			Element ruleSetElem = (Element) rulesetToken;
			String ruleSetName = ruleSetElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.NAME);
			Element[] ruleElements = ReflectionValidationDocument.getChildElements(ruleSetElem);
			for (int child = 0; child < ruleElements.length; ++child) {
				processRuleElement(ruleElements[child], componentCriteriaMap);
			}
		} else {
			throw new IllegalArgumentException("Ruleset token for a ReflectionDocument must be of type org.w3c.dom.Element");
		}
		return new ValidationDocumentRuleMapping(
				componentCriteriaMap, polymorphicTypes, nonPolymorphicTypes);
	}

	protected void processRuleElement (Element ruleElem, Map componentCriteriaMap) {
		// TODO MAS->Phani
		// ignored when ruleElem is child of actf:ruleset element
		// ie all of these will be "" and poly will be true
		String ruleName = ruleElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.NAME);
		String poly = ruleElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.POLYMORPHIC);
		boolean polymorphic = !poly.equals("false");

		// TODO MAS->Phani
		// empty criterion representing what we are now calling a TypeCriterion
		ComponentCriteria compCriteria = new ComponentCriteria();
		String className = getAliasValue(ruleElem.getTagName());
		Class cls = clCache.classForName(className);
		Map criteriaToRulesMap = componentCriteriaMap.containsKey(className)
			? (Map) componentCriteriaMap.get(className) : new HashMap();
			
		if (cls != null) {
			List criteriaRules = createComponentCriteria(ruleElem);
			if (!criteriaRules.isEmpty()) {
				compCriteria = new ComponentCriteria(criteriaRules);
			}
			List compRules = createRulesForComponent(ruleElem);
			criteriaToRulesMap.put(compCriteria, compRules);
			componentCriteriaMap.put(className, criteriaToRulesMap);
			
			if (polymorphic) {
				polymorphicTypes.add(cls);
				if (nonPolymorphicTypes.contains(cls)) {
					nonPolymorphicTypes.remove(cls);
				}
			} else {
				nonPolymorphicTypes.add(cls);
			}
		} else {
			Utils.println(Utils.WARNINGS, "Unknown class: " + className);
		}
	}

	protected List createRulesForComponent(Element comp) {
		List compRuleList = new LinkedList();
		Element[] rules = ReflectionValidationDocument.getChildElements(comp);
		for (int r = 0; r < rules.length; ++r) {
			IRule rule = createRule(rules[r]);
			if (rule != null) {
				compRuleList.add(rule);
				Utils.println(Utils.PROCESS_ALL,
						"Adding rule: " + rules[r].getTagName() + " to rule list for component: " + comp.getTagName());
				//((AbstractRule) rule).printRule(1);
			} else {
				Utils.println(Utils.PROCESS_ALL,
						"Unable to create rule: " + rules[r].getTagName() + " for component: " + comp.getTagName());
			}
		}
		return compRuleList;
	}

	protected IRule createRule(Element ruleElement, IRule parent) {
		IRule vRule = makeRuleInstance(ruleElement, parent);
		if (vRule != null) {
			try {
				Utils.println(Utils.PROCESS_ALL,
						"Initializing rule for element: " + ruleElement.getTagName());
				vRule.setRuleSet(ruleSet);
				vRule.initialize(ruleElement);
				Utils.println(Utils.PROCESS_ALL,
						"IRule: " + vRule.getName() + " initialized.");
				
				// Get the children of this rule.
				Element[] childRuleElems = ReflectionValidationDocument.getChildElements(ruleElement);
				for (int c = 0; c < childRuleElems.length; ++c) {
					IRule childRule = createRule(childRuleElems[c], vRule);
					if (childRule != null) {
						Utils.println(Utils.PROCESS_ALL,
								"Adding rule part: " + childRule.getName() + " to parent: " + vRule.getName());
						vRule.addRulePart(childRule);
					}
				}
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_NONFATAL,
						"Unable to initialize rule: " + vRule.getName(), e);
			}
		}
		
		return vRule;
	}

	protected IRule createRule(Element ruleElem) {
		return createRule(ruleElem, null);
	}

	protected String getAliasValue(String name) {
		return ((ValidationDocumentRuleBaseMetadata) ruleBase.getMetadata())
				.getAliasValue(name);
	}

	protected List createComponentCriteria(Element comp) {
		List criteriaList = new LinkedList();
		// showElementAttributes( comp, 0);
		// TODO In release 0.1.0, criteria are a single attribute on the first
		// level children (a component) of a <actf:ruleset>.
		NamedNodeMap criteria = comp.getAttributes();
		for (int c = 0; c < criteria.getLength(); ++c) {
			Attr compAttr = (Attr) criteria.item(c);
			IRule rule = createCriteria(compAttr);
			if (rule != null) {
				criteriaList.add(rule);
				Utils.println(Utils.PROCESS_ALL,
						"Adding criteria: " + compAttr.getNodeName() + " to rule list for component: " + comp.getTagName());
				//((AbstractRule) rule).printRule(1);
			} else {
				Utils.println(Utils.PROCESS_ALL,
						"Unable to create criteria: " + compAttr.getNodeName() + " for component: " + comp.getTagName());
			}
		}

		return criteriaList;
	}

	protected IRule createCriteria(Attr criteriaAttr) {
		IRule vRule = makeRuleInstance(criteriaAttr, null);
		if (vRule != null) {
			try {
				Utils.println(Utils.PROCESS_ALL,
						"Initializing rule for element: " + criteriaAttr.getNodeName());
				vRule.setRuleSet(ruleSet);
				vRule.initialize(criteriaAttr);
				Utils.println(Utils.PROCESS_ALL,
						"IRule: " + vRule.getName() + " initialized.");
			} catch (Exception e) {
				Utils.println(IReporter.SYSTEM_NONFATAL,
						"Unable to initialize rule: " + (vRule.getName() != null ? vRule.getName() : "<Unknown name>"), e);
			}
		}
		
		return vRule;
	}
	
	public ReflectionDocumentRuleFactory() {
	}

	/** {@inheritDoc} */
	public void registerRuleClassName(String name, String ruleClassName) {
		if (name != null && ruleClassName != null) {
			if (!ruleMap.containsKey(name)) {
				ruleMap.put(name, ruleClassName);
			}
		}
	}

	/** {@inheritDoc} */
	public IRule getRuleInstance(String name) {
		IRule rule = null;
		if (name != null && name.length() > 0) {
			// Try looking up the rule by name.
			String ruleClassName = (String) ruleMap.get(name);
			if (ruleClassName != null && ruleClassName.length() > 0) {
				try {
					Utils.println(Utils.PROCESS_ALL,
							"ReflectionDocumentRuleFactory.getRuleInstance(String), Creating rule for element: "
									+ name + ", instantiating class: " + ruleClassName);
					rule = (IRule) Class.forName(ruleClassName).newInstance();
				} catch (Exception e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"ReflectionDocumentRuleFactory.getRuleInstance(String), unable to create rule: " + ruleClassName, e);
				}
			}
		}
		
		return rule;
	}

	/** {@inheritDoc} */
	public IRule getRuleInstance(String name, IRule parent) {
		IRule rule = null;
		if (name != null && name.length() > 0) {
			rule = getRuleInstance(name);
		}
		if (rule != null && parent != null) {
			Utils.println(Utils.PROCESS_ALL,
					"ReflectionDocumentRuleFactory.getRuleInstance(Element, IRule), Setting parent:" + parent.getName());
			rule.setParent(parent);
		}
		return rule;
	}

	/** {@inheritDoc} */
	public IRule makeRuleInstance(Object obj, IRule parent) {
		IRule rule = null;
		Element elem = null;
		Attr attr = null;
		String name = null;
		boolean isActfNamespace = false;
		
		if (obj instanceof Element) {
			elem = (Element) obj;
			name = elem.getTagName();
			isActfNamespace = elem.getNamespaceURI() != null && elem.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI);
		} else if (obj instanceof Attr) {
			// TODO MAS->Phani
			// currently, the only time this is invoked is during processing of validation
			// documents for MSAA and IA2; attribute of immediate child of <actf:ruleset>
			// indicates that a property is to be tested
			attr = (Attr) obj;
			name = attr.getLocalName();
			isActfNamespace = attr.getNamespaceURI() != null && attr.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI);
		}
		
		if (elem != null) {
			String localName = elem.getLocalName();
			if (isActfNamespace && !localName.equals(RuleConstants.CRITERION)) {
				rule = getRuleInstance(localName);
			} else if (isActfNamespace && localName.equals(RuleConstants.CRITERION)) {
				// TODO MAS->Phani
				// fill in when actf:criterion is introduced
			}
			
			// getRuleInstance(String) will try to instantiate a rule by name and,
			// if not successful, will try to look the name up in the ruleNames Map
			// and if a name is found, will try to instantiate that class.
			// If a rule can't be created and the element is not in the ACTF namespace, then 
			// the default is to create a Reflexive rule
			if (rule == null && !isActfNamespace) {
						String ruleClassName = (String) ruleMap.get(RuleConstants.REFLEXIVE);
						if (ruleClassName != null && ruleClassName.length() > 0) {
							try {
								Utils.println(Utils.PROCESS_ALL,
										"ReflectionDocumentRuleFactory.makeRuleInstance(), Creating rule for element: "
										+ name + ", instantiating class: " + ruleClassName);
								rule = (IRule) Class.forName(ruleClassName).newInstance();
							} catch (Exception e) {
								Utils.println(IReporter.SYSTEM_NONFATAL,
										"ReflectionDocumentRuleFactory.makeRuleInstance(), unable to create rule: " + ruleClassName, e);
							}
						}
					}
			
					if (rule != null && parent != null) {
						Utils.println(Utils.PROCESS_ALL,
								"ReflectionDocumentRuleFactory.getRuleInstance(Element, ComponentRule), Setting parent:" + parent.getName());
						rule.setParent(parent);
					}
		} else if (attr != null) {
			if (!isActfNamespace) {
				// TODO MAS->Phani
				// create a PropertyCriterion
				String ruleClassName = (String) ruleMap.get(RuleConstants.CRITERION);
				if (ruleClassName != null && ruleClassName.length() > 0) {
					try {
						Utils.println(Utils.PROCESS_ALL,
								"ReflectionDocumentRuleFactory.makeRuleInstance(), Creating criteria for attribute: " + name + ", instantiating class: " + ruleClassName);
						rule = (IRule) Class.forName(ruleClassName).newInstance();
					} catch (Exception e) {
						Utils.println(IReporter.SYSTEM_NONFATAL,
								"ReflectionDocumentRuleFactory.makeRuleInstance(), unable to create rule: " + ruleClassName, e);
					}
				}
			}
		}

		return rule;
	}

	/** {@inheritDoc} */
	public IRule makeRuleInstance(Object obj) {
		return makeRuleInstance(obj, null);
	}

	public boolean isRuleAttribute(String name) {
		return name.equals(RuleConstants.ID)
		|| name.equals(RuleConstants.NAME)
		|| name.equals(RuleConstants.SEVERITY)
		|| name.equals(RuleConstants.POLYMORPHIC)
		|| name.equals(RuleConstants.MESSAGE)
		|| name.equals(RuleConstants.DESCRIPTION)
		|| name.equals(RuleConstants.HELPURL)
		|| name.equals(RuleConstants.PARAM)
		|| name.equals(RuleConstants.ITERATOR)
		|| name.equals(RuleConstants.IGNOREIFNULL)
		|| name.equals(RuleConstants.ENABLE);
	}

}
