/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.events;

import java.util.EventObject;
import java.util.LinkedList;

import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReportRecord;


/**
 * used to capture validation report records and creation contexts
 * 
 * @author Mike Squillace
 * 
 */
public class ReportEvent extends EventObject {

	private LinkedList reports = new LinkedList();

	private LinkedList contexts = new LinkedList();

	private String reportID;

	/**
	 * create a new report event
	 * 
	 * @param source
	 *            source of event
	 */
	public ReportEvent(Object source) {
		super(source);
	}

	/**
	 * get the most recently fetched set of report records. Note that the most
	 * recently <b>fetched</b> report is not necessarily the most recently
	 * <b>generated</b> report.
	 * 
	 * @return records from most recently fetched report or <code>null</code>
	 *         if no report records are available
	 */
	public ValidationReportRecord[] getReportRecords() {
		ValidationReportRecord[] report = null;
		if (!reports.isEmpty()) {
			report = (ValidationReportRecord[]) reports.removeFirst();
		}
		return report;
	}

	/**
	 * get the most recently fetched creation contexts. Note that the most
	 * recently <b>fetched</b> contexts are not necessarily the most recently
	 * <b>generated</b> contexts.
	 * 
	 * @return most recently fetched contexts or <code>null</code> if no
	 *         creation contexts are available
	 */
	public CreationContext[] getCreationContexts() {
		CreationContext[] contexts = null;
		if (!this.contexts.isEmpty()) {
			contexts = (CreationContext[]) this.contexts.removeFirst();
		}
		return contexts;
	}

	/**
	 * add report records to the reports queue. This method, although public, is
	 * used by the resource server only.
	 * 
	 * @param records
	 */
	public void queueReportRecords(ValidationReportRecord[] records) {
		reports.addLast(records);
	}

	/**
	 * add creation contexts to the contexts queue. This method, although
	 * public, is used by the resource server only.
	 * 
	 * @param contexts
	 */
	public void queueCreationContexts(CreationContext[] contexts) {
		this.contexts.addLast(contexts);
	}

	/**
	 * set the report ID. This method should only be called by servers.
	 * 
	 * @param id
	 */
	public void setReportID(String id) {
		reportID = id;
	}

	/**
	 * get the report ID of the current report. This id will be valid once a
	 * REPORT_STARTED event has been fired.
	 * 
	 * @return report ID
	 */
	public String getReportID() {
		return reportID;
	}

	/**
	 * clear the contents of the queues
	 * 
	 */
	public void clear() {
		contexts.clear();
		reports.clear();
		reportID = null;
	}
}
