package org.eclipse.actf.validation.javapp.aspects;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.Vector;

import org.eclipse.actf.validation.core.ValidationProcessException;


public class AspectGenerator
{
	
	//indentation
	public static final String indent = "      ";
	//line separator
	private static final String  div = System.getProperty("line.separator");
	
	private static final String PACKAGE = "PACKAGE";
	private static final String ASPECT = "ASPECT";
	private static final String IMPORT = "IMPORT";
	private static final String MEMBER = "MEMBER";
	private static final String POINTCUT = "POINTCUT";
	private static final String ADVICE = "ADVICE";
	
	private static final String legal = "/*******************************************************************************"
                                      + div + " * Copyright (c) 2004, 2007 IBM Corporation." 
                                      + div + " * All rights reserved. This program and the accompanying materials\n"
                                      + div + " * are made available under the terms of the Eclipse Public License v1.0\n"
                                      + div + " * which accompanies this distribution, and is available at\n"
                                      + div + " * http://www.eclipse.org/legal/epl-v10.html\n"
                                      + div + " * \n"
                                      + div + " * Warning: DON'T MODIFY THIS PROGRAM, IT IS AUTOMATICALLY GENERATED\n"
                                      + div + " *******************************************************************************/"
	                                  + div;
	                                  
	private StringBuffer buffer = new StringBuffer(legal);
	
	//end of constructor mark
	private final static String ENDOFCONSTOR = "//end of default construct";
	
	/**
	 * 
	 * @param packageName: package name of the aspect class to be generated
	 * @param aspectName: 
	 * @param superClass: should be InterJvmValidationAspect for Inter-JVM validation, and IntraJvmValidation for inter-JVm validation
	 */
	public void createAspect(String packageName, String aspectName, String superClass) throws ValidationProcessException {
	    
		int index = findInsertPosition(PACKAGE);
		StringBuffer sb = new StringBuffer();
		
		sb.append(div);
		sb.append("package "+ packageName +";");
		sb.append(div);
		if (index ==-1) {
			buffer.append(sb);
		} else {
			
			buffer.insert(index, sb.toString());
		}
		
		findInsertPosition(ASPECT);
		StringBuffer aBuffer = new StringBuffer();
		aBuffer.append(div);
		aBuffer.append("public aspect "+ aspectName );
		
		if ((superClass != null)&&(!superClass.equals(""))) {
			aBuffer.append(" extends " + superClass + div);
			aBuffer.append("{" +div + div);
			
			//generate default constructor
			aBuffer.append(indent + "public " + aspectName +"() {");
			aBuffer.append(div +indent + indent);
			aBuffer.append("super(\"" + aspectName +"\");");
			aBuffer.append(div + indent);
			aBuffer.append("} " + ENDOFCONSTOR);
			aBuffer.append(div +div );
			aBuffer.append("}" + div );
		}
		
		buffer.append(aBuffer);
	}
	
	public void addImports(Vector imports) throws ValidationProcessException {
		
		if (imports.size() != 0)  {
		   int index = findInsertPosition(IMPORT);
		   StringBuffer sb = new StringBuffer(); 
		
		   for (int i=0; i< imports.size(); i++)  {
			
		      sb.append("import " + imports.elementAt(i) + ";" +div);
		   }   
		 
		   if (index == -1) {
		      buffer.append(sb.toString());
		   } else {
			   buffer.insert(index, sb.toString());
		   }
		}				
	}
	
    public void addImport(String packageName) throws ValidationProcessException {
		
       if ((packageName != null)&&(!packageName.equals(""))) {
	        
    	   int index = findInsertPosition(IMPORT);
    	   StringBuffer sb = new StringBuffer();
    	   
    	   sb.append(div);
    	   sb.append("import " + packageName);
    	   sb.append(div);
    	   
    	   if (index == -1) {
 		      buffer.append(sb.toString());
 		   } else {
 			   buffer.insert(index, sb.toString());
 		   }
			
		}
	}
    
    public void addClassMember(String modifier, String type, String name, String initialValue) throws ValidationProcessException {
    	
    	if ((name == null)||(name.equals(""))||(type == null)||(type.equals("")))  {
    	
    	   throw new ValidationProcessException("Failed to specify aspect member name and/or type, exist generation process");	
    	}
    	
    	modifier = nomalizeParameter(modifier);
    	initialValue = nomalizeParameter(initialValue);
    	
    	int index = findInsertPosition(MEMBER);
     	StringBuffer sb = new StringBuffer();
     	   
     	sb.append(div);
     	sb.append(indent + "modifier " + "type " + name);
     	if (!initialValue.equals("")) {
     		sb.append(" = " + initialValue);
     	}
     	
     	buffer.insert(index, sb.toString());	
    	
    	
    }

    /**
     * generate pointcut.  Assume all the parameter passed are valid
     * a pointcut takes the form of:
     *     pointcut name(parameterList) : expression;
     *  
     * @param name: pointcut name, optional
     * @param parameterList: optional, a comma-separated type-parameter pairs same with parameter list in a java method
     * @param expression: required
     */
    public void createPointcut(String name, String parameterList, String expression) throws ValidationProcessException {
		
    	int index = findInsertPosition(POINTCUT);
     	StringBuffer sb = new StringBuffer();
     	
    	parameterList = nomalizeParameter(parameterList);
    	name = nomalizeParameter(name);
    	expression = nomalizeParameter(expression);
    	
    	if (expression.equals("")) {
    		//shouldn't get here
    		throw new ValidationProcessException("The specified pointcut expression is null");
    	}
    	
    	sb.append(div);
    	sb.append(indent);
	    sb.append("pointcut " + name);
	    sb.append("(" + parameterList + ") : ");
	    sb.append(div);
	    sb.append(indent + expression);
	    sb.append(div);
	    
	    buffer.insert(index, sb.toString());
	    
            		
	}
	
    /**
     * generate advice for pointcuts
     * an advice takes form of:
     *     type (parameterList) [returning/throwing(formal)] : 
     *     pointcutExpression { body };
     *     
     * @param type: required, one of the before, after and around 
     * @param parameterList: optional, a comma-separated type-parameter pairs same with parameter list in a java method
     * @param pointcutExpression: optional, pointcut name and parameterlist, can be combined using &&, !, || etc.
     * @param body: required, advice body
     * @param afterAction: one of the throwing(formal) or returning(formal). required only if the advice type is after
     */
    public void createAdvice(String type, String parameterList, String pointcutExpression, String body, String afterAction) throws ValidationProcessException {
		
    	int index = findInsertPosition(ADVICE);
     	StringBuffer sb = new StringBuffer();
     	
    	type = nomalizeParameter(type);
    	
    	if (!type.equals("before")&&(!type.equals("after"))&&(!type.equals("around"))) {
    		//shoudn't get here
			throw new ValidationProcessException("The specified advice type is not supported");
    	}
    	
    	pointcutExpression = nomalizeParameter(pointcutExpression);
    	body = nomalizeParameter(body);
    	if (body.equals("")) {
    		//shouldn't get here
    		throw new ValidationProcessException("The specified advice body is null");
    	}
    	
    	afterAction = nomalizeParameter(afterAction);
    	if (!type.equals("after")&&(!afterAction.equals(""))) { 
    		//shoudn't get here
			throw new ValidationProcessException("The specified after action is not valid");
    	}
    	
    	sb.append(div + div);
    	sb.append(indent + type +" ");
    	sb.append("(" + parameterList +") ");
    	sb.append(afterAction);
    	sb.append(" : ");
    	sb.append(div);
    	sb.append(indent + pointcutExpression);
    	sb.append(" {");
    	sb.append(div);
    	sb.append(indent +indent + body);
    	sb.append(div);
    	sb.append(indent + "}");
    	sb.append(indent);

    	buffer.insert(index, sb.toString());
	}
	
    /**
     * Create a aspect source file in system's temp directory or specified trace directory  
     * @param path
     * @return
     */
	public File generateAspectFile(String workingDir, String aspectName) throws IOException {
		
		String ajFileName = aspectName +".aj";
		File ajFile = setupAjFile(workingDir, ajFileName);
		
		try {
			
		    FileOutputStream fos = new FileOutputStream(ajFile);
		    Writer out = new OutputStreamWriter(fos);
		    out.write(buffer.toString());
		    out.close();
		
		} catch (Exception exc) {
		   System.out.println("Failed to write aspect file, " + ajFileName);	
		}
		
		return ajFile;
		
	}
	
	public void compileAspectFile(String workingDir, File file) {
	
		String path = file.getPath();
		//String actfDir = System.getProperty("java.io.tmpdir") + IConfiguration.ACTF_ID;
		
		//outjar -d options don't work together well
		//String[] args = {"-outjar", "ActfAspect.jar", "-d", actfDir, path };
		//String[] args = {"-classpath ", "ajde.jar"+File.pathSeparator+"aspectjweaver.jar"+File.pathSeparator+"aspectjrt.jar", "-d", workingDir, path };
		//String[] args = {"-classpath ", "D:\\aspectj1.5\\lib\\aspectjtools.jar", "-d", workingDir, path };
		String[] args = {"-verbose", "-d", workingDir, path };
		String base = "c:/eclipse-constellation-331/eclipse/plugins/";
		String[] ajJars = {
				base + "org.aspectj.ajde_1.5.4.20080111211120/ajde.jar",
				base + "org.aspectj.runtime_1.5.4.20080111211120/aspectjrt.jar",
				base + "org.aspectj.weaver_1.5.4.20080111211120/aspectjweaver.jar"
		};
		try {
			URL[] ajUrls = new URL[ajJars.length];
			for (int f = 0; f < ajJars.length; ++f) {
				ajUrls[f] = new File(ajJars[f]).toURL();
			}
			URLClassLoader ajcl = new URLClassLoader(ajUrls);
			Class compCls = Class.forName("org.aspectj.tools.ajc.Main", true, ajcl);
			Class handlerCls = Class.forName("org.aspectj.bridge.MessageHandler", true, ajcl);
			Class holderCls = Class.forName("org.aspectj.bridge.IMessageHolder", true, ajcl);
			Class kindCls = Class.forName("org.aspectj.bridge.IMessage$Kind", true, ajcl);
			Object compiler = compCls.newInstance();
			Object m = handlerCls.newInstance();
			compCls.getMethod("run", new Class[]{String[].class, holderCls})
				.invoke(compiler, new Object[]{args, m});
			
//		Main compiler = new Main();
//	    MessageHandler m = new MessageHandler();
//	    compiler.run(args, m);compiler.getClass().getClassLoader()
			Object[] ms = (Object[]) handlerCls.getMethod("getMessages", new Class[]{kindCls, Boolean.TYPE})
			.invoke(m, new Object[]{null, Boolean.TRUE});
			System.out.println("messages: " + Arrays.asList(ms));
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClassNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InstantiationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}
	
	private String nomalizeParameter(String parameter) {
		
		if ((parameter==null)||(parameter.trim().equals(""))) {
    		parameter = "";
    	} else {
    		parameter = parameter.trim();
    	}
		
		return parameter;
	}
	
	private int findInsertPosition(String artifact) throws ValidationProcessException {
		
		int pos = buffer.indexOf("public aspect");
				
		if (artifact.equals("ASPECT")) {
		   if (pos != -1) {			    
			  throw new ValidationProcessException("Aspect already generated, exit the generation");
		
		   } else {
		  	   return pos;
		   }	   
		}   
		
		if (pos == -1) {
			if (artifact.equals("IMPORT")) {
				
				return pos;
				
			} else if (artifact.equals("PACKAGE")) {
				
				pos = buffer.indexOf("import");
				
				return pos-1;
			
			} else {			
				throw new ValidationProcessException("Failed to find the aspect name, exit the generation");
			}
		} 
		
		if (artifact.equals("IMPORT")) {
			
			return pos-1;
		}
		
		int index= -1;
		if ((artifact.equals("POINTCUT"))||(artifact.equals("ADVICE"))) {
		
		   //find the closing '}' of the aspect
		   index= buffer.lastIndexOf("}");
		   if (index == -1)  {
			  throw new ValidationProcessException("Failed to find the end of aspect mark '}', exit the generation");	
		   }
		   
		   return index-1;
		}
		
		if (artifact.equals("MEMBER")) {
			
			//find the end of constor mark '{'
			   index= buffer.indexOf(ENDOFCONSTOR, pos);
			   if (index == -1)  {
				  throw new ValidationProcessException("Failed to find the start of aspect mark '}', exit the generation");	
			   }
			   
			   return index+1;
		}
		
		return index;
	}
	
	private File setupAjFile(String workingDir, String name) {
		//String actfDir = System.getProperty("java.io.tmpdir") + IConfiguration.ACTF_ID;
		
		File actfDirFile = new File(workingDir);
		
		if (!actfDirFile.exists()) {
			try {
				actfDirFile.mkdir();
			}catch (Exception e) {
				System.out.println("Failed to create aspect compile directory in system temp directory:" + workingDir);
				
			}
		}
		
		File ajFile = new File(workingDir + File.separator + name);
				
		return ajFile;
	}
}
