/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.aspects;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.DefaultModelFactory;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.AbstractReporter;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.javapp.launch.ValidationLauncher;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReportRecord;

/**
 * base class for all aspects in ACTF. Direct subclasses of this class should be extended by
 * clients writing their own aspects.
 * 
 * @author Mike Squillace
 *
 */
public abstract class BaseValidationAspect
{

	protected IConfiguration valConfig;
	protected IModel model;

	private Map _contextMap = new HashMap();
	private Set _processedContextIdSet = new HashSet();

	private class PreCreationContext
	{
		private String _id;
		private String _member;
		private Exception _exception;

		public PreCreationContext (String id, String member, Exception exception) {
			_id = id;
			_member = member;
			_exception = exception;
		}

		public String getObjectID () {
			return _id;
		}

		public String getMember () {
			return _member;
		}

		public StackTraceElement[] getStackTrace () {
			StackTraceElement[] stackTrace = _exception.getStackTrace();
			List trace = new LinkedList();
			for (int s = 0; s < stackTrace.length; ++s) {
				if (!isIgnorable(stackTrace[s])) {
					trace.add(stackTrace[s]);
				}
			}
			return (StackTraceElement[]) trace.toArray(new StackTraceElement[trace.size()]);
		}
	}

	/**
	 * instantiated by AOP engine
	 *
	 */
	public BaseValidationAspect (String modelName) {
		String externalDump = System.getProperty(ValidationLauncher.EXTERNAL_JVM_DUMP);
		String levelStr = System.getProperty(IConfiguration.TRACE_LEVEL_KEY);
		int traceLevel = IReporter.INFO;
		if (externalDump == null) {
			externalDump = "actf.external";
		}
		if (levelStr != null) {
			try {
				traceLevel = Integer.parseInt(levelStr);
			}catch (NumberFormatException e) {
			}
		}
		Utils.setTracer(AbstractReporter.getReporter(traceLevel, "c:/actf-external.out"));
		Utils.println(Utils.WARNINGS,"testing");
		model = DefaultModelFactory.getInstance().resolveModel(modelName);
	}

	/**
	 * forms and caches the creation context for an instantiated element. The context can be
	 * retrieved later via the object id, which is <code>System.identityHashCode(newObj)</code>. This method
	 * also forces the server to fire a creationContextStarted event.
	 * 
	 * @param newObj - newly instantiated object
	 * @param member - member used to instantiated object
	 * @param exception - exception holding trace leading to instantiation
	 */
	protected String getObjectId (Object obj) {
		String id = null;
		if (model != null) {
			id = model.getNodeId(obj);
		} else {
			int iid = System.identityHashCode(obj);
			String xid = Integer.toHexString(iid).toUpperCase();
			id = "00000000".substring(0, 8 - xid.length()) + xid;
		}
		return id;
	}
	
	protected void cacheCreationContext (Object newObj, Object member, Exception exception) {
		String id = getObjectId(newObj);
		if (!_contextMap.containsKey(id)) {
			//Utils.println(Utils.ALL,"Caching "+newObj.getClass().getName()+"@"+id+" from call: " + member);
			PreCreationContext context = new PreCreationContext(id, member.toString(), exception);
			_contextMap.put(id, context);
		}
	}

	/**
	 * returns whether or not this trace element should be included in the stack trace used to
	 * form the creation context. By default, stack trace elements involving ACTF classes
	 * or classes found in an AOP engine are not included in the stack trace.
	 * 
	 * @param element - stack trace element to be tested
	 * @return <code>true</code> if element can be left out of stack trace
	 */
	protected boolean isIgnorable (StackTraceElement element) {
		String clsName = element.getClassName();
		String methName = element.getMethodName();
		return clsName.startsWith("org.jboss")
				|| clsName.startsWith("org.aspectj")
				|| clsName.startsWith("org.eclipse.actf.core")
				|| methName.indexOf('$') >= 0 || methName.indexOf("aop") >= 0;
	}

	/**
	 * find the creation contexts for the given records.
	 * 
	 * @param records - report records
	 */
	protected CreationContext[] findCreationContexts (ValidationReportRecord[] records) {
		ArrayList contexts = new ArrayList();
		for (int r = 0; r < records.length; ++r) {
			ValidationReportRecord record = records[r];
			String id = record.getNodeId();
			if (id != null && id.length() > 0 && !_processedContextIdSet.contains(id)) {
				PreCreationContext pContext = (PreCreationContext) _contextMap.get(id);
				//Utils.println(Utils.ALL, "Got pContext " + pContext + " for " + id);
				if (pContext == null) {
					continue;
				}
				CreationContext context = new CreationContext(pContext.getObjectID(), pContext.getMember(), pContext.getStackTrace());
				if (context != null) {
					contexts.add(context);
					_processedContextIdSet.add(id);
				}
			}
		}
		return (CreationContext[]) contexts.toArray(new CreationContext[contexts.size()]);
	}
}
