/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.aspects;

import java.util.Iterator;
import java.util.LinkedList;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.util.thread.ThreadUtils;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.core.ExecutionPoint;
import org.eclipse.actf.validation.core.ValidationProcessConstants;
import org.eclipse.actf.validation.core.ValidationProcessException;
import org.eclipse.actf.validation.core.ValidationProcessManager;
import org.eclipse.actf.validation.core.ValidationThread;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.javapp.server.ResourceClient;
import org.eclipse.actf.validation.javapp.server.ResourceServer;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReportRecord;

/**
 * provides basic services and validation advice to classes that are to be
 * validated by ACTF. This class is the base implementation for any ValidationAspect
 * class that is to be used by a distinct JVM from that JVM
 * hosting the launching framework. The model-specific ValidationAspect implementations are instantiated
 * by the AOP engine and should not be instantiated by client applications.
 * 
 * @author Mike Squillace
 */
public abstract class InterJvmValidationAspect extends BaseValidationAspect
{

	public static final long DEFAULT_TIMEOUT = 10000;

	public static final String REPORT_ID_FIELDNAME = "reportid";
	static {
		ValidationReportRecord.addAttributeNames(new String[] {REPORT_ID_FIELDNAME});
	}

	private static LinkedList reportQueue = new LinkedList();
	private static LinkedList contextsQueue = new LinkedList();
	private static LinkedList threadList = new LinkedList();

	protected ClassLoaderCache clCache;
	protected ResourceClient client;
	protected String creationContextsProcessing;

	private String _resHostName;
	private int _resPort;
	private String _reportID;
	
	// Use this code for synchronizing validation thread count in jdk1.4.
	private boolean _shutdown, _isShuttingDown;

	static private double initTime = System.currentTimeMillis();

	/**
	 * instantiate an aspect. Notice that aspects are usually instantiated by AOP engines and not
	 * directly from within code. This base constructor does the following:
	 * 
	 *  <p><ul>
	 *  <li>adds a shutdown hook to the JVM to insure clean closing of sockets and the AOP engine
	 *  <li>establishes the tracing facility (if parameters dictate one)
	 *  <li>sets up a ResourceClient for receiving resource from the launching JVM
	 *  <li>initializes other essential members such as the configuration object and the ClassLoaderCache
	 *  </ul>
	 */
	public InterJvmValidationAspect (String modelName) {
		super(modelName);
		
		Runtime.getRuntime().addShutdownHook(new Thread() {
			public void run () {
				if (!_shutdown) {
					doShutdown();
				}
			}
		});
		
		try {
			_resHostName = System.getProperty(
				JavaValidationConstants.RESOURCE_SERVER_HOST_KEY,
				JavaValidationConstants.RESOURCE_SERVER_HOST_DEFAULT);
			_resPort = Integer.parseInt(System.getProperty(
				JavaValidationConstants.RESOURCE_SERVER_PORT_KEY,
				Integer.toString(JavaValidationConstants.RESOURCE_SERVER_PORT_DEFAULT)));
			
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			valConfig = context.getConfiguration();
			client = new ResourceClient(_resHostName, _resPort);
		}catch (Exception e2) {
			Utils.println(IReporter.SYSTEM_FATAL, e2);
			System.exit(1);
		}
		if (valConfig == null) {
			throw new RuntimeException("Could not find ACTF configuration");
		}
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		creationContextsProcessing = valConfig.getStringParameter(JavaValidationConstants.CREATION_CONTEXTS_PROCESSING_KEY);
		clCache = ClassLoaderCache.getDefault();
	} // ctor

	/**
	 * find the creation contexts for components with violations only. This method should only be called
	 * if the <code>Configuration.CREATION_CONTEXTS_PROCESSING_KEY</code> is set to
	 * <code>Configuration.PROCESS_VIOLATING_COMPONENTS</code>.
	 * 
	 * @param records - report records
	 */
	protected void processCreationContexts (ValidationReportRecord[] records) {
		CreationContext[] contexts = findCreationContexts(records);
		for (int c = 0; c < contexts.length; ++c) {
			contextsQueue.addLast(new CreationContext[] {contexts[c]});
			synchronized (client.client_lock) {
				client.sendMessage(ResourceServer.CREATION_CONTEXT_COMPLETED);
				client.sendReportElements(
					client.getChannel(), contextsQueue.removeFirst());
			}
		}
	}

	protected void startValidationThread (final Object target) {
		Utils.println(
			Utils.PROCESS_INFO, "Elapsed time since aspect creation: "
					+ (System.currentTimeMillis() - initTime) / 1000.0);
		// This is for debugging multi-threaded issues. It is conditionally compiled 
		// based on the THREAD_DEBUG constant.
		if (ThreadUtils.THREAD_DEBUG) {
			ThreadUtils.dumpThreadState("Thread state from InterJvmValidationAspect.startValidationThread()");
		}
		Thread valThread = new ValidationThread(new Runnable() {

			public void run () {
				Utils.println(
					Utils.PROCESS_ALL,
					"In validation thread.run(),  target: [" + target + "]");
				doRunValidation(target);
				ValidationThread.decrementThreadCount();
			}
		});
		Object perfVal = null, reVal = null, valHidden = null;
		try {
			perfVal = client.requestConfigurationParameter(
				ValidationConstants.VALIDATION_ID,
				ValidationConstants.PERFORMING_VALIDATION_KEY);
			reVal = client.requestConfigurationParameter(
				ValidationConstants.VALIDATION_ID,
				ValidationConstants.REVALIDATING_COMPONENTS_KEY);
			valHidden = client.requestConfigurationParameter(
				ValidationConstants.VALIDATION_ID,
				ValidationConstants.VALIDATE_HIDDEN_KEY);
			Utils.println(Utils.ALL, "Toggles: [perform=" + perfVal
					+ ",valHidden=" + valHidden + ",revalidate=" + reVal);
			if (perfVal instanceof String) {
				perfVal = Boolean.valueOf((String) perfVal);
			}
			if (reVal instanceof String) {
				reVal = Boolean.valueOf((String) reVal);
			}
			if (valHidden instanceof String) {
				valHidden = Boolean.valueOf((String) valHidden);
			}
		}catch (Exception e) {
			Utils.println(
				IReporter.SYSTEM_FATAL, "Error getting toggle values: ", e);
		}
		boolean performValidation = perfVal != null
				&& ((Boolean) perfVal).booleanValue();
		boolean revalidate = reVal != null && ((Boolean) reVal).booleanValue();
		boolean validateHidden = valHidden != null
				&& ((Boolean) valHidden).booleanValue();
		valConfig.setSymbolPool(ValidationConstants.VALIDATION_ID);
		valConfig.setParameter(
			ValidationConstants.VALIDATE_HIDDEN_KEY, validateHidden);
		valConfig.setParameter(
			ValidationConstants.REVALIDATING_COMPONENTS_KEY, revalidate);
		if (performValidation) {
			Utils.println(
				Utils.PROCESS_ALL,
				"InterJvmValidationAspect.startValidationThread(), creating validation thread");
			threadList.add(valThread);
			valThread.start();
		}else {
			Utils.println(
				Utils.PROCESS_ALL,
				"InterJvmValidationAspect.startValidationThread(), aborting validation thread "
						+ valThread.getName());
			ValidationThread.decrementThreadCount();
		}
	} // startValidationThread

	protected void runValidationWorker (Object target) {
		Utils.println(Utils.PROCESS_ALL, "Validation started: " + target);
		synchronized (client.client_lock) {
			client.sendMessage(ResourceServer.REPORT_STARTED + "{"
					+ getReportID() + "}");
		}
		ValidationReportRecord[] records = null;
		String clsName = target.getClass().getName();
		String shortName = clsName.substring(clsName.lastIndexOf('.') + 1);
		String targetId = shortName + "@" + System.identityHashCode(target);
		double startTime = System.currentTimeMillis(), elapsedTime = 0.0;
		ValidationProcessManager vpm = ValidationProcessManager.getInstance();
		ExecutionPoint eventExPt = vpm.createMethodExecutionPointForTarget(
			ValidationProcessConstants.VISIBILITY, target, null, null);
		try {
			vpm.initiateValidation(eventExPt);
		}catch (ValidationProcessException vpe) {
			Utils.println( IReporter.SYSTEM_NONFATAL, "Unable to complete validation.", vpe);
		}
		
		records = (vpm.getReporter()).getRecords();
		elapsedTime = (System.currentTimeMillis() - startTime) / 1000.0;
		Utils.println( Utils.PROCESS_ALL, "Validation finished, elapsed time - " + elapsedTime);
		
		for (int r = 0; r < records.length; ++r) {
			records[r].setAttribute(REPORT_ID_FIELDNAME, targetId);
		}
		reportQueue.addLast(records);
		synchronized (client.client_lock) {
			client.sendMessage(ResourceServer.REPORT_COMPLETED);
			client.sendReportElements(client.getChannel(), reportQueue.removeFirst());
		}
		Utils.println(Utils.PROCESS_ALL, "Validation complete: " + target);
		
		if (creationContextsProcessing.equals(JavaValidationConstants.PROCESS_VIOLATION_CONTEXTS)) {
			Utils.println(Utils.PROCESS_ALL, "Processing contexts of components with violations");
			processCreationContexts(records);
		}
	}

	protected Object getActualTarget (Object target) {
		return target;
	}

	protected void doRunValidation (final Object target) {
		try {
			synchronized (this) {
				runValidationWorker(target);
			}
		}catch (Throwable e) {
			Utils.println(
				IReporter.SYSTEM_FATAL,
				"Could not execute Validation on target object.", e);
		}finally {
			// This code is valid for jdk 1.5 only. 
			//Utils.println(Utils.PROCESS_ALL, "Ending thread id - [" + Thread.currentThread().getId() 
			//				+ ", name - " + Thread.currentThread().getName() +"]");					
			Utils.println(Utils.PROCESS_ALL, "Ending thread name - "
					+ Thread.currentThread().getName());
			// This is for debugging multi-threaded issues. It is conditionally compiled 
			// based on the THREAD_DEBUG constant.
			if (ThreadUtils.THREAD_DEBUG) {
				//ThreadUtils.dumpThreadState("InterJvmValidationAspect.doRunValidation, Ending thread id - [" + Thread.currentThread().getId() 
				//+ ", name - " + Thread.currentThread().getName() +"]");
			}
		}
	}

	protected String getReportID () {
		return _reportID;
	}

	public void setReportID (String id) {
		_reportID = id;
	}

	protected void checkThreadCount () {
		while (ValidationThread.getThreadCount() > 0) {
			try {
				long start = System.currentTimeMillis();
				long end = start;
				synchronized (ValidationThread.VALIDATION_THREAD_LOCK) {
					ValidationThread.VALIDATION_THREAD_LOCK.wait(DEFAULT_TIMEOUT);
					end = System.currentTimeMillis();
					if (Math.abs(end - start - DEFAULT_TIMEOUT)
							/ (double) DEFAULT_TIMEOUT < 0.001) {
						for (Iterator iter = threadList.iterator(); iter.hasNext();) {
							ValidationThread vt = (ValidationThread) iter.next();
							//							StackTraceElement[] stack = vt.getStackTrace();
							//							StringBuffer sb = new StringBuffer();
							//							for (int t = 0; t < stack.length; ++t) {
							//								sb.append(stack[t]);				
							//							}
							if (vt.isAlive()) {
								vt.terminate();
								Utils.println(
									Utils.ALL,
									"Terminated validation thread " + vt);
								//Utils.println(Utils.ALL, "Dump for thread\n" + sb.toString());
							}
						}
					}
				}
			}catch (InterruptedException e) {
			}
		}
	}

	protected void doShutdown () {
		if (!_isShuttingDown) {
			_isShuttingDown = true;
			Utils.println(Utils.PROCESS_ALL, "Starting doShutdown().");
			checkThreadCount();
			
			try {
				Utils.println(Utils.PROCESS_ALL, "Sending VALIDATION_JVM_TERMINATING message.");
				
				if (client != null) {
					synchronized (client.client_lock) {
						client.sendMessage(ResourceServer.VALIDATION_JVM_TERMINATING);
					}
					Thread.sleep(500);
					Utils.println(Utils.PROCESS_ALL, "Issuing _client.requestClose()");
					client.requestClose();
					Utils.println(Utils.PROCESS_ALL, "Calling _valServer.shutdown().");
				}
			}catch (Throwable e) {
				Utils.println(IReporter.SYSTEM_FATAL, "Error in shutdown", e);
			} finally {
				Utils.println(Utils.PROCESS_ALL, "doShutdown() completed.");
				_shutdown = true;
				Utils.getTracer().close();
			}
		}
	}
	
} // InterJvmValidationAspect
