/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.events;

import java.util.EventObject;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

/**
 * @author Mike Squillace
 */
public class LaunchEvent extends EventObject
{

	/** used to specify I/O messages */
	public static final String IO_STATUS_KEY = "io";

	/** used to specify messages during pre-instrumentation process */
	public static final String PREINSTRUMENTATION_STATUS_KEY = "preinst";

	/** used to specify messages during execution of distinct JVM */
	public static final String EXECUTION_STATUS_KEY = "exec";

	/** perform the underlying action only if <code>true</code> */
	public boolean doit = true;

	private Map _msgMap = new HashMap();

	/**
	 * create a new launch event
	 * 
	 * @param source source of event
	 */
	public LaunchEvent (Object source) {
		super(source);
	}

	/**
	 * get any messages generated during unpacking or transferring of archives and/or
	 * directories. Messages are placed into the string array, each element of which
	 * represents one line of the original output.
	 * 
	 * @return messages generated during unpacking/transfer of resources
	 */
	public String[] getIOMessages () {
		List list = (List) _msgMap.get(IO_STATUS_KEY);
		return list == null ? null
				: (String[]) list.toArray(new String[list.size()]);
	}

	/**
	 * get any messages generated during preinstrumentation of .class files.
	 * Messages are placed into the string array, each element of which
	 * represents one line of the original output.
	 * 
	 * @return messages generated during pre-instrumentation
	 */
	public String[] getPreinstrumentationMessages () {
		List list = (List) _msgMap.get(PREINSTRUMENTATION_STATUS_KEY);
		return list == null ? null
				: (String[]) list.toArray(new String[list.size()]);
	}

	/**
	 * get any messages generated during execution of application or
	 * launch of GUI. Messages are placed into the string array, each element of which
	 * represents one line of the original output.
	 * 
	 * @return messages generated during execution of app
	 */
	public String[] getExecutionMessages () {
		List list = (List) _msgMap.get(EXECUTION_STATUS_KEY);
		return list == null ? null
				: (String[]) list.toArray(new String[list.size()]);
	}

	/**
	 * used by launcher to place a message into this event object
	 * 
	 * @param statusKey
	 * @param msg
	 */
	public void addMessage (String statusKey, String msg) {
		if (statusKey.equals(IO_STATUS_KEY)
				&& _msgMap.get(IO_STATUS_KEY) == null) {
			_msgMap.put(IO_STATUS_KEY, new LinkedList());
		}else if (statusKey.equals(PREINSTRUMENTATION_STATUS_KEY)
				&& _msgMap.get(PREINSTRUMENTATION_STATUS_KEY) == null) {
			_msgMap.put(PREINSTRUMENTATION_STATUS_KEY, new LinkedList());
		}else if (statusKey.equals(EXECUTION_STATUS_KEY)
				&& _msgMap.get(EXECUTION_STATUS_KEY) == null) {
			_msgMap.put(EXECUTION_STATUS_KEY, new LinkedList());
		}
		((List) _msgMap.get(statusKey)).add(msg);
	}
}
