/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.launch;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.regex.Pattern;

import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.logging.PrintWriterReporter;
import org.eclipse.actf.validation.javapp.events.LaunchEvent;

/**
 * used to process standard error and standard out streams when
 * running a distinct process.
 * 
 * @author Mike Squillace
 */
public class RuntimeStreamHandlerThread extends Thread
{

	private static final Pattern REPORT_RECORD_START = Pattern.compile("^[a-zA-Z]+\\.[A-Z]{4,7}:\\s.*$");

	private InputStream _inStream;

	private IReporter _reporter;

	private boolean _isErrorStream, _isUsingAllOutput = true;

	private StringBuffer _lastReportBuffer;

	private PrintWriter _writer;

	private ValidationLauncher _launcher;

	/**
	 * create a thread to process stderr/stdout from a separate process
	 * 
	 * @param is - input stream from process
	 * @param dest - destination
	 * @param isErrorStream - <code>if this is an error stream from a process
	 */
	public RuntimeStreamHandlerThread (InputStream is, IReporter dest,
										boolean isErrorStream) {
		this(is, dest, isErrorStream, null);
	}

	/**
	 * create a thread to process stderr/stdout from a separate process
	 * 
	 * @param is - input stream from process
	 * @param dest - destination
	 * @param isErrorStream - <code>if this is an error stream from a process
	 * @param launcher - callbacks to launcher to update launch events
	 */
	public RuntimeStreamHandlerThread (InputStream is, IReporter dest,
										boolean isErrorStream,
										ValidationLauncher launcher) {
		_inStream = is;
		_reporter = dest;
		_isErrorStream = isErrorStream;
		_launcher = launcher;
		_lastReportBuffer = new StringBuffer();
	}

	/**
	 * set a writer for debugging. All contents of all streams are
	 * sent to this stream if set.
	 * 
	 * @param writer - destination for debugging
	 */
	public void setDebugWriter (PrintWriter writer) {
		_writer = writer;
	}

	/**
	 * return the write for debugging
	 * 
	 * @return writer being used for debugging
	 */
	public PrintWriter getDebugWriter () {
		return _writer;
	}

	public void setUseAllOutput (boolean output) {
		_isUsingAllOutput = output;
	}

	/** {@inheritDoc} */
	public void run () {
		BufferedReader inReader = new BufferedReader(new InputStreamReader(_inStream));
		String line = null;
		try {
			while ((line = inReader.readLine()) != null) {
				boolean lineIsRecord = REPORT_RECORD_START.matcher(line).matches();
				if (_writer != null) {
					_writer.println(line);
				}
				if (_isErrorStream && _launcher != null) {
					_launcher.updateLaunchEvent(
						LaunchEvent.EXECUTION_STATUS_KEY, line);
				}else {
					if ((!_isUsingAllOutput && lineIsRecord)
							|| _isUsingAllOutput) {
						if (lineIsRecord) {
							_lastReportBuffer.append(line);
							_lastReportBuffer.append('\n');
						}
						((PrintWriterReporter) _reporter).getPrintWriter().println(
							line);
					}
				}
			}
		}catch (Exception e) {
			if (_launcher != null) {
				_launcher.updateLaunchEvent(
					LaunchEvent.EXECUTION_STATUS_KEY,
					e.getMessage() == null ? e.toString() : e.getMessage());
			}
		}finally {
			_reporter.flush();
		}
	} // run
} // RuntimeStreamHandlerThread
