/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.launch;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.util.FileUtils;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.jar.ZipExploder;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.IResourceLocator;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.events.IReportListener;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.javapp.events.ILaunchListener;
import org.eclipse.actf.validation.javapp.events.LaunchEvent;
import org.eclipse.actf.validation.javapp.server.ResourceServer;

/**
 * a launcher used to launch and simultaneously validate Java rich-client applications. The primary
 * purpose of this launcher is to form the command line that launches the AOP-based validation of a Java application. A Java application
 * may be launched via either a <code>main(String[])</code> method of a Java class or via an
 * executable file, which is platform dependent. The sole criterion of the executable is that
 * it must provide a way for this launcher to pass arguments to its underlying JVM.
 * 
 * <p>Currently, ACTF supports AspectJ AOP. 
 * 
 * @author Mike Squillace
 *
 */
public abstract class ValidationLauncher
{

	public static final String JAVA_HOME_KEY = "java.home";
	
	/** system property to indicate distinct JVM (i.e. running apart from Eclipse environment */
	public static final String DISTINCT_JVM_KEY = "actf.distinct.jvm";

	/** system property indicating location of dump from launched JVM */
	public static final String EXTERNAL_JVM_DUMP = "actf.external.dump";

	protected Path aopPath;
	protected Path bootClasspath_p, bootClasspath_a, bootClasspath;
	protected Path classpath;
	protected IConfiguration valConfig;
	
	private boolean _java14;
	private String _executableName, _execArgs;
	private String _jvmSubParam;
	private String _workingDir;
	private List _launchListeners = new LinkedList();
	private Map _systemProperties = new HashMap();
	private Map _jvmArgs = new HashMap();
	private LaunchEvent _launchEvent;
	private Process _p = null;

	/**
	 * Once get the process, set the process object to be the process that been launched.
	 * @param p process
	 */
	public void setProcess (Process p) {
		_p = p;
		if (_p == null) {
			fireLaunchCompletedEvent();
		}
	}

	/**
	 * Return the process object that has been launched.
	 * @return process
	 */
	public Process getProcess () {
		return _p;
	}

	protected class Path
	{

	
		private LinkedList _path = new LinkedList();

		public Path () {
			this(null);
		}

		public Path (String pathEntry) {
			append(pathEntry);
		}

		public String[] getPathEntries () {
			return (String[]) _path.toArray(new String[_path.size()]);
		}

		public void append (String pathEntry) {
			pathEntry = pathEntry != null ? pathEntry.trim() : "";
			if (pathEntry.length() > 0) {
				if (pathEntry.charAt(0) == '\"'
						&& pathEntry.charAt(pathEntry.length() - 1) == '\"'
						&& pathEntry.indexOf('\"', 2) == pathEntry.length() - 1) {
					pathEntry = pathEntry.substring(1, pathEntry.length() - 1);
				}
				StringTokenizer entries = new StringTokenizer(pathEntry, File.pathSeparator);
				while (entries.hasMoreTokens()) {
					String entry = entries.nextToken();
					if (entry.charAt(0) == '\"'
							&& entry.charAt(entry.length() - 1) == '\"') {
						entry = entry.substring(1, entry.length() - 1);
					}
					if (!_path.contains(entry)) {
						_path.add(entry);
					}
				}
			}
		}

		public void append (Path path) {
			String[] entries = path.getPathEntries();
			for (int e = 0; e < entries.length; ++e) {
				append(entries[e]);
			}
		}

		public void prepend (String pathEntry) {
			pathEntry = pathEntry != null ? pathEntry.trim() : "";
			if (pathEntry.length() > 0) {
				if (pathEntry.charAt(0) == '\"'
						&& pathEntry.charAt(pathEntry.length() - 1) == '\"'
						&& pathEntry.indexOf('\"', 2) == pathEntry.length() - 1) {
					pathEntry = pathEntry.substring(1, pathEntry.length() - 1);
				}
				StringTokenizer entries = new StringTokenizer(pathEntry, File.pathSeparator);
				while (entries.hasMoreTokens()) {
					String entry = entries.nextToken();
					if (entry.charAt(0) == '\"'
							&& entry.charAt(entry.length() - 1) == '\"') {
						entry = entry.substring(1, entry.length() - 1);
					}
					if (!_path.contains(entry)) {
						_path.addFirst(entry);
					}
				}
			}
		}

		public void prepend (Path path) {
			String[] entries = path.getPathEntries();
			for (int e = 0; e < entries.length; ++e) {
				prepend(entries[e]);
			}
		}

		public String toString () {
			StringBuffer sb = new StringBuffer();
			int size = _path.size();
			sb.append('\"');
			for (int p = 0; p < size; ++p) {
				sb.append(_path.get(p).toString());
				if (p < size - 1) {
					sb.append(File.pathSeparator);
				}
			}
			sb.append('\"');
			return sb.toString();
		}
	} // Path

	/**
	 * create a new launcher using the specified executable.
	 * 
	 * @param executableName - name of executable
	 * @see #ValidationLauncher(String, String)
	 */
	public ValidationLauncher (String executableName) {
		this(executableName, null);
	}

	/**
	 * create a validation launcher. The format of the AOP file
	 * named by the filename will be AOP-platform dependent. The working directory will default
	 * to the current directory of the launched process.
	 * 
	 * @param executableName - name of executable 
	 * @param aopFilename - name of AOP file specifying aspects, pointcuts, and/or binding advice
	 */
	public ValidationLauncher (String executableName, String aopFilename) {
		this(executableName, aopFilename, ".");
	}

	/**
	 * create a validation launcher. The format of the AOP
	 * file will be AOP-platform dependent.
	 *
	 * @param executableName - name of executable 
	 * @param aopFilename - AOP file containing bindings, aspects, pointcuts, etc.
	 * @param workingDir - working directory
	 * @see #setWorkingDirectory(String)
	 */
	public ValidationLauncher (String executableName, String aopFilename,
								String workingDir) {
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			valConfig = context.getConfiguration();			
		}catch (ConfigurationException e) {
			//Could not obtain a Configuration. We can't do much here but quit.
			throw new RuntimeException(e.getMessage(), e);
		}
		setWorkingDirectory(workingDir);
		configureResourceServer();
		_executableName = executableName;
		aopPath = new Path(aopFilename);
	}

	protected void configureResourceServer () {
		valConfig.setSymbolPool(ValidationConstants.VALIDATION_ID);
		String hostName = valConfig.getStringParameter(JavaValidationConstants.RESOURCE_SERVER_HOST_KEY);
		if (hostName == null || hostName.length() == 0) {
			hostName = JavaValidationConstants.RESOURCE_SERVER_HOST_DEFAULT;
		}
		String portx = valConfig.getStringParameter(JavaValidationConstants.RESOURCE_SERVER_PORT_KEY);
		int port = JavaValidationConstants.RESOURCE_SERVER_PORT_DEFAULT;
		try {
			port = Integer.parseInt(portx);
		}catch (NumberFormatException nfe) {
			throw new RuntimeException("Illegal port value: " + portx);
		}
		
		try {
			ResourceServer resServer = ResourceServer.getInstance();
			resServer.setHostName(hostName);
			resServer.setPort(port);
			resServer.removeAllReportListeners();
		} catch (IOException ioe) {
			throw new RuntimeException("Could not instantiate/configure resource server");
		}
	}

	/**
	 * add a ReportListener to be notified of report-related events
	 * 
	 * @param listener - listener to be called upon report event
	 */
	public void addReportListener (IReportListener listener) {
		try {
			ResourceServer.getInstance().addReportListener(listener);
		}catch (IOException e) {
		}
	}

	/**
	 * remove the specified listener from the list of listeners to be notified
	 * when report events take place.
	 * 
	 * @param listener - listener to be removed
	 */
	public void removeReportListener (IReportListener listener) {
		try {
			ResourceServer.getInstance().removeReportListener(listener);
		}catch (IOException e) {
		}
	}

	/**
	 * add a LaunchListener to be notified when the start or completion of
	 * a launch takes place or when a launch has somehow measurably progressed.
	 * 
	 * @param listener - listener to be called upon launch event
	 */
	public void addLaunchListener (ILaunchListener listener) {
		if (listener != null) {
			_launchListeners.add(listener);
		}
	}

	/**
	 * remove the specified listener from the list of listeners to be notified
	 * when launch events take place.
	 * 
	 * @param listener - listener to be removed
	 */
	public void removeLaunchListener (ILaunchListener listener) {
		_launchListeners.remove(listener);
	}

	/**
	 * add an argument to be passed to the JVM. The switch will automatically be prepended with a hyphen (-)
	 * and added without any arguments if the value is <code>null</code>.
	 * 
	 * <p><b>Note</b>: The JVM arguments -classpath/-cp, -Xbootclasspath and its variants, and
	 * system properties should <i>not</i> be added in this way.
	 *
	 * @param swtch - JVM switch
	 * @param value - value of switch or <code>null</code> if switch is to occur
	 * on the command line without an argument
	 * @see #addSystemProperty(String, String)
	 */
	public void addJvmArgument (String swtch, String value) {
		if (swtch != null) {
			_jvmArgs.put(swtch, value);
		}
	}

	/**
	 * add a system property to be added to the arguments to the JVM launched by
	 * this process.
	 * 
	 * @param key
	 * @param value
	 */
	public void addSystemProperty (String key, String value) {
		if (key != null && key.length() > 0) {
			_systemProperties.put(key, value);
		}
	}

	/**
	 * add all of the system properties contained in the map
	 * 
	 * @param props - system properties with keys and values of type <code>java.lang.String</code>
	 */
	public void addSystemProperties (Map props) {
		for (Iterator iter = props.keySet().iterator(); iter.hasNext();) {
			String key = (String) iter.next();
			Object value = props.get(key);
			_systemProperties.put(key, value instanceof String ? value
					: value.toString());
		}
	}

	/**
	 * return the working directory for the process launched
	 * 
	 * @return working directory
	 * @see #setWorkingDirectory(String)
	 */
	public String getWorkingDirectory () {
		return _workingDir;
	}

	/**
	 * set the working directory. This directory is the working directory
	 * for the launched process.
	 * 
	 * @param dir - working directory (may not be <code>null</code>)
	 * @throws IllegalArgumentException if specified directory already exists as a file or
	 * if the specified directory does not exist and cannot be created 
	 */
	public void setWorkingDirectory (String dir) throws IllegalArgumentException {
		if (dir != null) {
			File workdir = new File(dir);
			if (workdir.exists() && !workdir.isDirectory()) {
				throw new IllegalArgumentException("Working directory " + dir + " already exists as a file");
			}else if (!workdir.exists()) {
				if (!workdir.mkdir()) {
					throw new IllegalArgumentException("Unable to create working directory " + dir);
				}
			}
		}
		_workingDir = dir;
	}

	/**
	 * return the JVM substitution parameter
	 * 
	 * @return JVM substitution parameter
	 * @see #setJvmSubstitutionParameter(String)
	 */
	protected String getJvmSubstitutionParameter () {
		return _jvmSubParam;
	}

	/**
	 * set the JVM substitution parameter to be used by this launcher.  A JVM substitution parameter is
	 * used by launches in which executables with underlying JVMs (rather than Java classes) are being validated.
	 * This parameter is substituted by the arguments that ACTF must pass to the underlying JVM.
	 * 
	 * <p>For example, if we are to validate the Eclipse workbench, we must pass arguments to the JVM
	 * launched by eclipse.exe on the Windows platform. The execution arguments for this launcher, then,
	 * must include:
	 * 
	 * <p><code>-vmargs <JVMSubParam></code>
	 * 
	 * <p>so that ACTF knows where on the command line to place the JVM arguments. This argument may be left <code>null</code>
	 * for launchers launching Java classes.
	 * 
	 * @param jvmSub - substitution parameter for this launcher
	 */
	public void setJvmSubstitutionParameter (String jvmSub) {
		_jvmSubParam = jvmSub;
	}

	protected String getExecutableName () {
		return _executableName;
	}

	/**
	 * return the execution arguments
	 * 
	 * @return execution arguments
	 * @see #setExecutionArguments(String)
	 */
	protected String getExecutionArguments () {
		return _execArgs;
	}

	/**
	 * set the execution arguments for the application being launched/validated. If a JVM substitution parameter
	 * is provided and is non-<code>null</code>, then these arguments are taken
	 * as arguments to the executable being launched. For cases in which the JVM substitution parameter
	 * is <code>null</code>, it is assumed that a standard Java class is being launched and that
	 * these arguments are being passed to the <code>main(String[])</code> method.
	 * 
	 * @param args - execution arguments
	 * @see #setJvmSubstitutionParameter(String)
	 */
	public void setExecutionArguments (String args) {
		_execArgs = args;
	}

	/**
	 * return the classpath 
	 *  
	 * @return classpath
	 */
	public String getClasspath () {
		return classpath == null ? "" : classpath.toString();
	}

	/**
	 * set the classpath used by this launcher. The Java process will be launched with the -cp switch followed
	 * by the value passed to this method and those
	 * prepended or appended to it.
	 * 
	 * <p>The specified classpath must be a legitimate set of paths with the syntax specified of the underlying
	 * operating system.
	 * 
	 * @param cp - classpath for JVM
	 * @see #prependToClasspath(String)
	 * @see #appendToClasspath(String)
	 */
	public void setClasspath (String cp) {
		classpath = cp == null || cp.trim().length() == 0 ? new Path()
				: new Path(cp);
	}

	/**
	 * prepend the given path to the formed classpath for the launcher. If no classpath has been
	 * specified via the <code>setClasspath</code> or <code>appendToClasspath</code>
	 * methods, a new classpath is formed.
	 * 
	 * @param path - path to prepend
	 * @see #setClasspath(String)
	 * @see #appendToClasspath(String)
	 */
	public void prependToClasspath (String path) {
		if (classpath != null) {
			classpath.prepend(path);
		}else {
			classpath = new Path(path);
		}
	}

	/**
	 * append the given path to the formed classpath for the launcher. If no classpath has been
	 * specified via the <code>setClasspath</code> or <code>prependToClasspath</code>
	 * methods, a new classpath is formed.
	 * 
	 * @param path - path to append
	 * @see #setClasspath(String)
	 * @see #prependToClasspath(String)
	 */
	public void appendToClasspath (String path) {
		if (classpath != null) {
			classpath.append(path);
		}else {
			classpath = new Path(path);
		}
	}

	/**
	 * return whether or not this launcher is using a JVM of version 1.4.2.
	 * 
	 * @return <code>true</code> if JVM being launched by this process is 1.4.2,
	 * <code>false</code> otherwise
	 * @see #setUsingJava14(boolean)
	 */
	protected boolean isUsingJava14 () {
		return _java14;
	}

	/**
	 * set whether or not this launcher is using a JVM v. 1.4.2. When set to
	 * <code>false</code> (the default), the JVM being used is assumed
	 * to be version 1.5.0 or higher. In this case, the 'javaagent' command line
	 * option will be used to take advantage of the instrumentation capabilities of JDK 5.0 and
	 * higher whereas earlier versions must rely upon manipulating the system class loader or the bytecode of <code>java.lang.ClassLoader</code>
	 * to effect weaving of aspect code.
	 * 
	 * @param java14 - set to <code>true</code> if JVM is version 1.4.2 only
	 */
	public void setUsingJava14 (boolean java14) {
		_java14 = java14;
	}

	/**
	 * called to launch the AOP-based validation of the Java application. This method:
	 * 
	 * <p><ul>
	 * <li>sets up a new <code>LaunchEvent</code> object and fires a <code>launchInitiated</code> event 
	 * <li>launches the process by calling the <code>launchProcess</code> method
	 * <li>captures any exceptions and puts them in the <code>LaunchEvent</code> object
	 * <li>fires a <code>launchCompleted</code> event
	 * </ul>
	 *
	 */
	public void launch () {
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		_launchEvent = new LaunchEvent(this);
		fireLaunchInitiatedEvent();
		try {
			launchProcess();
		}catch (Throwable e) {
			Utils.println(IReporter.SYSTEM_FATAL, e);
			StringBuffer sb = new StringBuffer();
			StackTraceElement[] stack = e.getStackTrace();
			for (int ee = 0; ee < stack.length; ++ee) {
				sb.append(stack[ee]);
				sb.append('\n');
			}
			_launchEvent.addMessage(
				LaunchEvent.EXECUTION_STATUS_KEY,
				e.getMessage() == null ? e.toString() : e.getMessage());
			_launchEvent.addMessage(
				LaunchEvent.EXECUTION_STATUS_KEY, sb.toString());
		}finally {
			fireLaunchCompletedEvent();
		}
	} // launch

	/**
	 * launches a process for running/validating the application. This method prepares the execution command,
	 * calls <code>preLaunch()</code>, and launches the JVM.
	 *  
	 * @throws Exception
	 */
	protected void launchProcess () throws Exception {
		prepareForLaunch();
		String execArgs = getExecutionArguments();
		String jvmSubParam = getJvmSubstitutionParameter();
		List exec = new LinkedList();
		if (jvmSubParam == null) {
			exec.add(getJavaPath());
			exec.addAll(Arrays.asList(allJvmArguments()));
			exec.add(getExecutableName());
			if (execArgs != null && execArgs.length() > 0) {
				exec.add(execArgs);
			}
		}else {
			int pos = execArgs.indexOf(jvmSubParam);
			if (pos == -1) { throw new IllegalArgumentException("Cannot find JVM substitution parameter: "
					+ jvmSubParam); }
			String[] jvmArgs = allJvmArguments();
			StringBuffer execBuff = new StringBuffer(execArgs);
			StringBuffer allJvmArgs = new StringBuffer();
			for (int a = 0; a < jvmArgs.length; ++a) {
				allJvmArgs.append(jvmArgs[a]);
				allJvmArgs.append(' ');
			}
			execBuff.replace(
				pos, pos + jvmSubParam.length(), allJvmArgs.toString());
			exec.add(getExecutableName());
			exec.add(execBuff.toString());
		}
		File workDir = new File(getWorkingDirectory());
		int exit = launchExternalProcess(
			(String[]) exec.toArray(new String[exec.size()]), workDir,
			new File(workDir, "launch.out").getAbsolutePath(),
			new File(workDir, "launch.err").getAbsolutePath(), this);
		Utils.println(
			Utils.PROCESS_INFO, "Exit value for JVM launch: " + exit);
	} // launchProcess

	/**
	 * prepares for the launch by adding system properties used by ACTF and starting the <code>ResourceServer</code>.
	 * 
	 * <p>Clients may overwrite, but should always call <code>super.prepareForLaunch()</code>.
	 * 
	 * @throws Exception
	 */
	public void prepareForLaunch () throws Exception {
		if (aopPath == null) {
			throw new IllegalArgumentException("AOP file has not been set or created");
		} else if (getWorkingDirectory() == null) {
			throw new IllegalArgumentException("Working directory must be specified");
		}
		
		ResourceServer resServer = ResourceServer.getInstance();
		if (resServer.isSuspended()) {
			resServer.setSuspended(false);
		}
		// TODO should be parameterized
		addJvmArgument("Xmx128M", null);
		//addJvmArgument("verbose", null);
		
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		addSystemProperty(
			IConfiguration.TRACE_LEVEL_KEY,
			valConfig.getParameterAsString(IConfiguration.TRACE_LEVEL_KEY));
		addSystemProperty(
			JavaValidationConstants.RESOURCE_SERVER_HOST_KEY,
			resServer.getHostName());
		addSystemProperty(
			JavaValidationConstants.RESOURCE_SERVER_PORT_KEY,
			new Integer(resServer.getPort()).toString());
		addSystemProperty(DISTINCT_JVM_KEY, Boolean.TRUE.toString());
		processSwtResources(classpath, new Path(getSystemProperty("java.library.path")));
	}

	void updateLaunchEvent (String key, String msg) {
		_launchEvent.addMessage(key, msg);
	}

	protected void fireLaunchInitiatedEvent () {
		for (Iterator iter = _launchListeners.iterator(); iter.hasNext();) {
			((ILaunchListener) iter.next()).launchInitiated(_launchEvent);
		}
	}

	protected void fireLaunchCompletedEvent () {
		for (Iterator iter = _launchListeners.iterator(); iter.hasNext();) {
			((ILaunchListener) iter.next()).launchCompleted(_launchEvent);
		}
	}

	protected void fireLaunchProgressedEvent () {
		for (Iterator iter = _launchListeners.iterator(); iter.hasNext();) {
			((ILaunchListener) iter.next()).launchProgressed(_launchEvent);
		}
	}

	/**
	 * get the fully-qualified path to the java executable. If no path for the JDK/JRE
	 * has been set, only the 'java' command is returned.
	 * 
	 * @return fully-qualified path to java executable
	 */
	protected String getJavaPath () {
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		String path = valConfig.getStringParameter(JavaValidationConstants.JAVA_HOME_KEY);
		if (path == null) {
			path = "";
		}else {
			path = path + File.separator + "bin" + File.separator;
		}
		return path + "java";
	}

	protected String getSystemProperty (String key) {
		return (String) _systemProperties.get(key);
	}

	/**
	 * get the registered system properties, each as a string prepared for the command line. Each system property
	 * key will be prepended by the requisite '-D' switch and followed by
	 * an equals sign '='. The quoted value then follows. If a key has a value of
	 * <code>null</code>, it is not included in the returned string.
	 * 
	 * @return array of system properties, each as it should appear on the command line
	 */
	protected String[] allSystemProperties () {
		List props = new LinkedList();
		for (Iterator iter = _systemProperties.keySet().iterator(); iter.hasNext();) {
			String key = (String) iter.next();
			String value = (String) _systemProperties.get(key);
			if (value != null) {
				StringBuffer sb = new StringBuffer();
				sb.append("-D");
				sb.append(key);
				sb.append('=');
				//sb.append(value.startsWith("\"") ? value.substring(
				//	1, value.length() - 1) : value);
				//bug fix #220
				if (value != null) {
					sb.append(value.startsWith("\"") ? value
							: quoteString(value));
				}
				props.add(sb.toString());
			}
		}
		return (String[]) props.toArray(new String[props.size()]);
	}

	/**
	 * quote the given string
	 * 
	 * @param str
	 * @return <code>&quot;&quot;str&quot;&quot;</code>
	 */
	protected String quoteString (String str) {
		if (str == null) {
			str = "";
		}
		return '\"' + str + '\"';
	}

	/**
	 * process the SWT .jar and .dll files. This will vary depending upon the version of Eclipse
	 * being used:
	 * 
	 * <p><ul>
	 * <li>if 3.0.x is being used, add swt.jar to classpath and add
	 * directory of dll files to library path
	 * <li>if 3.1 or higher is used, add swt .jar to classpath and extract dll files
	 * to working directory
	 * </ul>
	 * 
	 * @param classPath - classpath to be modified
	 * @param libPath - library path to be modified
	 * @throws IOException
	 */
	protected void processSwtResources (Path classPath, Path libPath)
		throws IOException {
		valConfig.setSymbolPool(IConfiguration.ACTF_ID);
		String swtHome = valConfig.getStringParameter(SwtGuiModel.SWT_RUNTIME_HOME_KEY);
		if (swtHome != null && swtHome.length() > 0) {
			if (swtHome.endsWith(File.pathSeparator)) {
				swtHome = swtHome.substring(0, swtHome.length() - File.pathSeparator.length());
			}
			int dot = swtHome.lastIndexOf('.');
			String ext = swtHome.substring(dot + 1);
			if (ext.equalsIgnoreCase("jar") || ext.equalsIgnoreCase("zip")) {
				// Eclipse 3.1 style:
				// extract dlls to working dir and add this
				// jar/zip to classpath
				ZipExploder exploder = new ZipExploder();
				ZipFile zf = new ZipFile(swtHome);
				Enumeration en = zf.entries();
				while (en.hasMoreElements()) {
					ZipEntry ze = (ZipEntry) en.nextElement();
					String name = ze.getName();
					dot = name.lastIndexOf('.');
					ext = name.substring(dot + 1);
					if (ext.equalsIgnoreCase("dll")) {
						exploder.copyFileEntry(getWorkingDirectory(), zf, ze);
					}
				}
				classPath.append(swtHome);
				libPath.append(getWorkingDirectory());
			}else {
				// Eclipse 3.0.x style:
				// find swt.jar and append to classpath and add
				// dir of dlls to library path
				File[] swtJar = FileUtils.findFiles(
					"swt.jar", new File(swtHome)); //akf
				for (int f = 0; f < swtJar.length; ++f) {
					classPath.append(swtJar[f].getAbsolutePath());
				}
				File[] swtDlls = FileUtils.findFiles("*.dll", new File(swtHome)); //akf
				for (int f = 0; f < swtJar.length; ++f) {
					libPath.append(swtDlls[f].getParent());
				}
			}
			addSystemProperty("java.library.path", libPath.toString());
		}
	} // processSwtResources

	/**
	 * returns  all the JVM arguments 
	 * held by this launcher. JVM arguments will, in order, include:
	 * 
	 * <p><ul>
	 * <li>any arguments to any of the Xbootclasspath variants
	 * <li>classpath argument (argument to -cp)
	 * <li>system properties
	 * <li>other JVM arguments added via the <code>addJvmArgument</code> method
	 * </ul>
	 * 
	 * @return all JVM arguments
	 */
	public String[] allJvmArguments () {
		List args = new LinkedList();
		// 1 - bootclasspath variants
		if (bootClasspath_p != null) {
			args.add("-Xbootclasspath/p:" + bootClasspath_p);
		}
		if (bootClasspath != null) {
			args.add("-Xbootclasspath:" + bootClasspath);
		}
		if (bootClasspath_a != null) {
			args.add("-Xbootclasspath/a:" + bootClasspath_a);
		}
		// 2 - classpath
		if (classpath != null) {
			args.add("-cp");
			String resPath = (String) _systemProperties.get(IResourceLocator.ACTF_RESOURCES_PATHS_KEY);
			classpath.append(File.pathSeparator+resPath);
			args.add(classpath.toString());
		}
		// 3 - all system properties
		String[] sysProps = allSystemProperties();
		if (sysProps.length > 0) {
			args.addAll(Arrays.asList(sysProps));
		}
		// 4 - all other JVM arguments
		for (Iterator iter = _jvmArgs.keySet().iterator(); iter.hasNext();) {
			String key = (String) iter.next();
			String value = (String) _jvmArgs.get(key);
			StringBuffer sb = new StringBuffer();
			sb.append('-');
			sb.append(key);
			if (value != null) {
				sb.append(value.startsWith("\"") ? value : quoteString(value));
			}
			args.add(sb.toString());
		}
		return (String[]) args.toArray(new String[args.size()]);
	} // allJvmArguments
	
	
	/**
	 * launch an external OS process
	 * 
	 * @param exec - command line to be executed
	 * @param workingDir - working directory
	 * @param out - destination of output of process
	 * @param err - destination for error stream from process
	 * @return exit code of process
	 */
	public static int launchExternalProcess (String[] exec, File workingDir,
												String out, String err) {
		return launchExternalProcess(exec, workingDir, out, err);
	}

	/**
	 * launch an external OS process
	 * 
	 * @param exec - command line to be executed
	 * @param workingDir - working directory
	 * @param out - destination of output of process
	 * @param err - destination for error stream from process
	 * @param launcher - launcher launching process
	 * @return exit code of process
	 */
	public static int launchExternalProcess (String[] exec, File workingDir,
												String out, String err,
												ValidationLauncher launcher) {
		String[] exec2 = new String[exec.length];
		if (System.getProperty("os.name").startsWith("Win")) {
			for (int i = 0; i < exec.length; i++) {
				exec2[i] = exec[i].replaceAll("[\\\\][\"]$", "\\\\\\\\\"");
			}
		}else { // assuming linux
			for (int i = 0; i < exec.length; i++) {
				exec2[i] = exec[i].replaceAll("[\"]", "");
			}
		}
		exec = exec2;
		Utils.println(Utils.PROCESS_INFO, "Command:" + Utils.arrayAsString(exec)
				+ ", working dir: " + workingDir.getAbsolutePath());
		int rc = 0;
		RuntimeStreamHandlerThread outputThread = null, errorThread = null;
		PrintWriter outWriter = null, errWriter = null;
		IReporter reporter = Utils.getTracer();
		try {
			Process p = Runtime.getRuntime().exec(exec, null, workingDir);
			launcher.setProcess(p);
			outWriter = new PrintWriter(new FileWriter(out));
			errWriter = new PrintWriter(new FileWriter(err));
			outputThread = new RuntimeStreamHandlerThread(p.getInputStream(), reporter, false, launcher);
			errorThread = new RuntimeStreamHandlerThread(p.getErrorStream(), Utils.getTracer(), true, launcher);
			outWriter.println("Output for process:");
			outWriter.println(Utils.arrayAsString(exec));
			errWriter.println("Error stream for process:");
			errWriter.println(Utils.arrayAsString(exec));
			outputThread.setDebugWriter(outWriter);
			errorThread.setDebugWriter(errWriter);
			outputThread.start();
			errorThread.start();
			rc = p.waitFor();
		}catch (Throwable e) {
			Utils.println(IReporter.SYSTEM_FATAL, e);
			rc = 1;
		}finally {
			//Utils.println(Utils.ALL, "External JVM terminated, gathering thread info. \n" + ThreadUtils.getThreadState());        	
			//System.out.println("Utils.launchExternalProcess().");
			//ThreadUtils.dumpThreadState();
			if (outputThread != null) {
				outputThread.getDebugWriter().close();
			}
			if (errorThread != null) {
				errorThread.getDebugWriter().close();
			}
		}
		Utils.println(
			Utils.ALL, "External JVM terminated, exec string - " + exec
					+ ", working directory - " + workingDir.getName());
		return rc;
	}	
} // ValidationLauncher
