/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.runtime;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.util.resources.DefaultResourceLocator;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.javapp.server.ResourceClient;
import org.eclipse.actf.validation.rulebase.IRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;

/**
 * a resource locator for retrieving resource over a socket connection between
 * two JVMs. This locator is used by the validating JVM (i.e. launched JVM) in
 * an AOP-based validation. Resources are retrieved from the launching JVM,
 * which is either launched by the Eclipse environment or via command line.
 * 
 * <p>
 * Note: The ClassLoader arguments are ignored in this implementation.
 * 
 * @author Mike Squillace
 */
public class InterJvmResourceLocator extends DefaultResourceLocator
{

	private Map _resourceCache = new HashMap();
	private ResourceClient _client;
	private String _hostName = JavaValidationConstants.RESOURCE_SERVER_HOST_DEFAULT;
	private int _port = JavaValidationConstants.RESOURCE_SERVER_PORT_DEFAULT;

	/**
	 * set the host name for the connection to the source of the resources to be
	 * retrieved by this locator
	 * 
	 * @param hostName - the host name
	 */
	public void setHostName (String hostName) {
		_hostName = hostName;
	}

	/**
	 * set the port of the connection that will furnish the source of the
	 * resources to be retrieved by this locator
	 * 
	 * @param port
	 */
	public void setPort (int port) {
		_port = port;
	}

	/** {@inheritDoc} */
	public InputStream getResourceAsStream (String id) {
		return getResourceAsStream(id, null);
	}

	/** {@inheritDoc} */
	public InputStream getResourceAsStream (String id, ClassLoader loader) {
		return getResourceAsStream(id, null, null, loader);
	}

	/** {@inheritDoc} */
	public InputStream getResourceAsStream (String id, String base, String ext,
											ClassLoader loader) {
		loadClient();
		InputStream stream = null;
		try {
			stream = _client.requestResource(id, base, ext);
			Utils.println(Utils.ALL, "Contents of stream for " + id
					+ "." + ext);
			if (stream != null) {
				BufferedReader br = new BufferedReader(new InputStreamReader(stream));
				String line = null;
				while ((line = br.readLine()) != null) {
					if (line.length() > 0) {
						Utils.println(Utils.ALL, line);
					}
				}
				Utils.println(Utils.ALL, line);
				br.close();
				stream.reset();
			}
		}catch (Exception e) {
			Utils.println(
				IReporter.SYSTEM_FATAL,
				"Could not find resource on resource server - " + id);
		}
		return stream;
	}

	/**
	 * request the active ACTF configuration object from the resource server
	 * 
	 * @return active ACTF configuration object
	 * @throws Exception
	 */
	public IConfiguration requestConfiguration () throws Exception {
		IConfiguration config = null;
		loadClient();
		config = _client.requestConfiguration();
		return config;
	}
	
	private void loadClient() {
		if (_client == null) {
			try {
				_client = new ResourceClient(_hostName, _port);
				loadRuleBaseManager();
				loadAccessibilityServiceManager();
			} catch (Exception e) {
				throw new RuntimeException(
						"Could not create connection to resource server - "
								+ (e.getMessage() == null ? "" : e.getMessage()));
			}
		}
	}
	
	private void loadRuleBaseManager() {
		try {
			IRuleBaseDescriptor[] bases = _client.requestRuleBaseDescriptors();
			RuleBaseManager manager = RuleBaseManager.getInstance();
			
			for( int i=0; i < bases.length; i++ ) {
				manager.addRuleBase( bases[i] );
			}
		} catch ( Exception e ) {
			Utils.println(IReporter.SYSTEM_FATAL, "loadRuleBaseManager failed", e);
		}
		
	}
	
	protected void finalize () throws Throwable {
		if (_client != null) {
			_client.requestClose();
		}
	}
	
	private void loadAccessibilityServiceManager() {
		Object accServiceManager = null;
		Map accservices = null;
		try {
			accservices = _client.requestAccessibilityServices();
		} catch (Exception e) {
			Utils.println(
					"Unable to get accservice Map", e);
		}
		Class accService = ClassLoaderCache.getDefault().classForName(
				"org.eclipse.actf.accservice.AccessibilityServiceManager");
		if (accService != null) {
			try {
				Method getAccServicesInstance = accService.getMethod(
						"getInstance", (Class[]) null);
				accServiceManager = getAccServicesInstance.invoke(null);

			} catch (Exception e) {
				Utils.println(
						"Unable to get AccessibilityServiceManager instance", e);
			}
			

			if (accServiceManager != null && accservices != null) {
				try {
					Method getRegisterAccServiceMeth = accService.getMethod(
							"registerAccessibilityService",
							(new Class[] {String.class, String.class}));
					for (Iterator iter = accservices.keySet().iterator(); iter.hasNext();) {
						String name = (String)iter.next();
						String classname = (String) accservices.get(name);
						getRegisterAccServiceMeth.invoke(accServiceManager, new Object [] {name, classname});
					}

				} catch ( Exception e ) {
					Utils.println(IReporter.SYSTEM_FATAL, "loadAccessibilityServices failed", e);
				}
			}
		}
		
			
			
			
		
		
	}
}


