/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.server;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.ByteBuffer;
import java.nio.channels.SocketChannel;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CharsetEncoder;

import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;

/**
 * basic services for a singleton server in ACTF
 * 
 * @author Mike Squillace
 *
 */
public abstract class AbstractSingletonServer 
{

	public static int transaction_id = 0;

	public static final String RESOURCE_SERVER_HOST_KEY = "resource.server.host";
	public static final String RESOURCE_SERVER_PORT_KEY = "resource.server.port";
	public static final String RESOURCE_SERVER_HOST_DEFAULT = "localhost";
	public static final String RESOURCE_SERVER_PORT_DEFAULT = "20013";	
	
	public static final Charset CHARSET = Charset.forName("UTF-8");

	public static final CharsetEncoder CHARSET_ENCODER = CHARSET.newEncoder();

	public static final CharsetDecoder CHARSET_DECODER = CHARSET.newDecoder();

	public static final String REQUEST_CLOSE = "requestClose";

	private int _port = JavaValidationConstants.RESOURCE_SERVER_PORT_DEFAULT;
	private String _hostName = JavaValidationConstants.RESOURCE_SERVER_HOST_DEFAULT;
	private boolean _suspended, _shutdown;

	/**
	 * 
	 * @return host name for this server
	 */
	public String getHostName () {
		return _hostName;
	}

	/**
	 * set the host name for this server. The default host name is 'localhost'.
	 * 
	 * @param name -
	 *            the host name
	 */
	public void setHostName (String name) {
		if (name == null || name.length() == 0) {
			try {
				name = InetAddress.getLocalHost().getHostName();
			}catch (UnknownHostException e) {
			}
		}
		_hostName = name;
	}

	/**
	 * returns the port on which this server is accepting connection requests
	 * 
	 * @return the port for this server
	 */
	public int getPort () {
		return _port;
	}

	/**
	 * set the port for this server to receive conection requests
	 * 
	 * @param port -
	 *            port number
	 */
	public void setPort (int port) {
		_port = port;
	}

	public boolean isSuspended () {
		return _suspended;
	}

	public void setSuspended (boolean suspended) {
		_suspended = suspended;
	}

	/**
	 * shut down the server
	 * 
	 *
	 */
	public void shutdown () {
		_shutdown = true;
		
	}

	/**
	 * returns whether or not this server has been shutdown. This method returns <code>true</code>
	 * as soon as <code>shutdown()</code> is invoked.
	 * 
	 * @return <code>true</code> if server has been shut down
	 */
	public boolean isShutdown () {
		return _shutdown;
	}

	protected synchronized String receiveMessage (SocketChannel client) throws IOException {
		String received = "";
		try {
			ByteBuffer buf = ByteBuffer.wrap(requestAsBytes(client));
			received = CHARSET_DECODER.decode(buf).toString().trim();
			Utils.println(Utils.PROCESS_ALL, "Received:" + received);
		}catch (Exception e) {
			Utils.println(IReporter.SYSTEM_NONFATAL, "Error reading message from client", e);
		}
		return received;
	}

	public static synchronized void sendResource (SocketChannel client, byte[] bytes,
										int size) throws IOException {
		ByteBuffer buffer = ByteBuffer.allocateDirect(8);
		buffer.putInt(transaction_id++);
		buffer.putInt(size);
		buffer.flip();
		client.write(buffer);
		if (size > 0) {
			buffer = ByteBuffer.wrap(bytes);
			client.write(buffer);
		}
		Utils.println(Utils.PROCESS_ALL, "Successfully wrote "
				+ bytes.length + " bytes to client");
	} // sendResource

	public static synchronized byte[] requestAsBytes (SocketChannel client)
		throws IOException {
		int offset = 0;
		ByteBuffer buffer = ByteBuffer.allocateDirect(8);
		while (offset == 0) {
			offset += client.read(buffer);
			if (offset > 0) {
				Utils.println(Utils.PROCESS_ALL, "Read " + offset
						+ " total bytes for transactionId and size");
			}
		}
		if (offset != 8) { throw new RuntimeException("Illegal packet size"); }
		offset = 0;
		buffer.flip();
		buffer.getInt(); //transaction_id
		byte[] bytes = null;
		int length = buffer.getInt();
		if (length > 0) {
			bytes = new byte[length];
			buffer = ByteBuffer.wrap(bytes);
			while (offset < length) {
				offset += client.read(buffer);
				if (offset > 0) {
					Utils.println(Utils.PROCESS_ALL, "Read " + offset
							+ " total bytes");
				}
			}
			Utils.println(Utils.PROCESS_INFO, "Read " + offset + "/"
					+ bytes.length + " bytes");
		}
		return bytes;
	}
}
