/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.server;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.SocketChannel;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.rulebase.IRuleBaseDescriptor;

/**
 * a client used to access ACTF configurations and other resources. This class is used
 * in inter-JVM valaidations to provide the launched (i.e. validating) JVM with access to
 * resources in the launching JVM. An "inter-JVM" validation is one in which the application or 
 * content to be validated is launched in a distinct JVM from that in which ACTF is running.
 * 
 * @see ResourceServer
 * @author Mike Squillace
 */
public class ResourceClient
{

	private static int transaction_id = 0;

	private SocketChannel _client;

	private Selector _selector;

	public Object client_lock = new Object();

	/**
	 * create a client that connects to the ACTF <code>ResourceServer</code> on the specified port. This client
	 * can be used during an inter-JVM validation to request configurations
	 * and other resources from the source JVM.
	 * 
	 * @param port
	 * @throws IOException
	 */
	public ResourceClient (int port) throws IOException {
		this(InetAddress.getLocalHost().getHostName(), port);
	}

	/**
	 * create a client that connects to the specified host and port. This client
	 * can be used during an inter-JVM validation to request configurations
	 * and other resources from the source JVM
	 * 
	 * @param hostName
	 * @param port
	 * @throws IOException
	 */
	public ResourceClient (String hostName, int port) throws IOException {
		_client = SocketChannel.open();
		_selector = Selector.open();
		_client.configureBlocking(false);
		_client.register(_selector, SelectionKey.OP_CONNECT);
		_client.connect(new InetSocketAddress(hostName, port));
		
		while (_selector.select(1000) > 0) {
			Set keys = _selector.selectedKeys();
			for (Iterator iter = keys.iterator(); iter.hasNext();) {
				SelectionKey key = (SelectionKey) iter.next();
				iter.remove();
				if (key.isConnectable()) {
					Utils.println(Utils.PROCESS_INFO,"Client connected, using host - " + hostName + ", port - " + port);
					SocketChannel channel = (SocketChannel) key.channel();
					if (channel.isConnectionPending()) {
						channel.finishConnect();
						Utils.println(Utils.PROCESS_INFO, "Connection complete");
					}
				}
			}
			// needed to work-around a bug with IBM 1.4.0 where Selector.select() keeps returning 1 key 
			break;
		}
	} // ctor

	/**
	 * request the active ACTF configuration object from the resource server
	 * 
	 * @return active ACTF configuration object
	 * @throws Exception
	 */
	public IConfiguration requestConfiguration () throws Exception {
		IConfiguration config = null;
		synchronized (client_lock) {
			sendMessage(ResourceServer.REQUEST_CONFIGURATION);
			ByteArrayInputStream bais = new ByteArrayInputStream(AbstractSingletonServer.requestAsBytes(_client));
			config = (IConfiguration) new ObjectInputStream(bais).readObject();
		}
		Utils.println(Utils.PROCESS_INFO, "Received configuration:" + config);
		return config;
	} // requestConfiguration

	/**
	 * request the active rule base descriptors from the resource server
	 * 
	 * @return array of rule bases descriptors
	 * @throws Exception
	 */
	public IRuleBaseDescriptor[] requestRuleBaseDescriptors () throws Exception {
		IRuleBaseDescriptor[] bases = null;
		synchronized (client_lock) {
			sendMessage(ResourceServer.REQUEST_RULEBASE_DESCRIPTORS);
			ByteArrayInputStream bais = new ByteArrayInputStream(AbstractSingletonServer.requestAsBytes(_client));
			bases = (IRuleBaseDescriptor[]) new ObjectInputStream(bais).readObject();
		}
		
		String count  = bases==null ? "null" : String.valueOf( bases.length );		
		Utils.println(Utils.PROCESS_INFO, "Received rule bases: count="+count );
		
		return bases;
	}
	
	public Map requestAccessibilityServices () throws Exception {
		Map accservices = null;
		synchronized (client_lock) {
			sendMessage(ResourceServer.REQUEST_ACCESSIBILITY_SERVICES);
			ByteArrayInputStream bais = new ByteArrayInputStream(AbstractSingletonServer.requestAsBytes(_client));
			accservices = (Map) new ObjectInputStream(bais).readObject();
			
		}
		return accservices;
	}
	/**
	 * request a resource from the resource server. The method is directly called
	 * from the <code>org.eclipse.actf.validation.javapp.resources.InterJvmResourceLocator</code> in order to
	 * fetch resources on the client-side of an inter-JVM validation.
	 * 
	 * @param id
	 * @param base
	 * @param ext
	 * @return stream to resource
	 * @throws IOException
	 * @see org.eclipse.actf.validation.javapp.runtime.InterJvmResourceLocator
	 */
	public InputStream requestResource (String id, String base, String ext)
		throws IOException {
		StringBuffer requestBuff = new StringBuffer(ResourceServer.REQUEST_RESOURCE);
		byte[] bytes = new byte[0];
		ByteArrayInputStream bais = null;
		requestBuff.append(':');
		requestBuff.append(id);
		requestBuff.append(',');
		requestBuff.append(base == null ? ResourceServer.NULL_STRING : base);
		requestBuff.append(',');
		requestBuff.append(ext == null ? ResourceServer.NULL_STRING : ext);
		synchronized (client_lock) {
			sendMessage(requestBuff.toString());
			bytes = AbstractSingletonServer.requestAsBytes(_client);
			bais = new ByteArrayInputStream(bytes);
		}
		return bais;
	}

	/**
	 * request a configuration parameter from the server
	 * 
	 * @param pool pool id
	 * @param id id of parameter
	 * @return parameter corresponding to pool/id combination
	 * @throws Exception
	 * @see org.eclipse.actf.core.config.IConfiguration
	 */
	public Object requestConfigurationParameter (String pool, String id)
		throws Exception {
		StringBuffer requestBuff = new StringBuffer(ResourceServer.REQUEST_CONFIGURATION_PARAMETER);
		requestBuff.append(':');
		requestBuff.append(pool == null ? IConfiguration.ACTF_ID : pool);
		requestBuff.append(',');
		requestBuff.append(id == null ? ResourceServer.NULL_STRING : id);
		synchronized (client_lock) {
			sendMessage(requestBuff.toString());
			return new ObjectInputStream(new ByteArrayInputStream(AbstractSingletonServer.requestAsBytes(_client))).readObject();
		}
	}

	/**
	 * request an end to the session
	 * 
	 *
	 *@throws IOException
	 */
	public void requestClose () throws IOException {
		synchronized (client_lock) {
			sendMessage(AbstractSingletonServer.REQUEST_CLOSE);
			if (_client != null && _client.isConnected()) {
				_client.close();
			}
			if (_selector != null && _selector.isOpen()) {
				_selector.close();
			}
		}
	}

	/**
	 * send a message to the resource server. The message should be one of
	 * the pre-defined messages in the <code>ResourceServer</code> class else the
	 * message will be ignored.
	 * 
	 * @param msg message to resource server
	 * @see ResourceServer
	 */
	public void sendMessage (String msg) {
		try {
			++transaction_id;
			Utils.println(Utils.PROCESS_INFO, "Sending message: " + msg);
			byte[] a = msg.getBytes();
			ByteBuffer buffer = ByteBuffer.allocateDirect(a.length + 8);
			buffer.putInt(transaction_id);
			buffer.putInt(a.length);
			buffer.put(a);
			buffer.flip();
			_client.write(buffer);
		}catch (IOException e) {
			Utils.println(
				IReporter.SYSTEM_NONFATAL, "Could not send message: " + msg);
		}
	}

	public void sendReportElements (SocketChannel client, Object reportElements) {
		try {
			byte[] bytes = null;
			if (reportElements != null) {
				ByteArrayOutputStream bos = new ByteArrayOutputStream();
				ObjectOutputStream oss = new ObjectOutputStream(bos);
				oss.writeObject(reportElements);
				oss.flush();
				oss.close();
				bytes = bos.toByteArray();
			}else {
				bytes = new byte[0];
			}
			Utils.println(Utils.ALL, "Sending " + bytes.length
					+ " bytes for resource " + reportElements);
			AbstractSingletonServer.sendResource(client, bytes, bytes.length);
		}catch (IOException e) {
			Utils.println(
				IReporter.SYSTEM_NONFATAL, "Error sending report elements: "
						+ e.getMessage(), e);
		}
	}

	public SocketChannel getChannel () {
		return _client;
	}
}
