/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.server;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.lang.reflect.Method;
import java.net.InetSocketAddress;
import java.net.ServerSocket;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.channels.SelectionKey;
import java.nio.channels.Selector;
import java.nio.channels.ServerSocketChannel;
import java.nio.channels.SocketChannel;
import java.nio.charset.CharacterCodingException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.config.XmlConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.ClassLoaderCache;
import org.eclipse.actf.util.resources.IResourceLocator;
import org.eclipse.actf.validation.events.IReportListener;
import org.eclipse.actf.validation.events.ReportEvent;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.rulebase.IRuleBaseDescriptor;
import org.eclipse.actf.validation.rulebase.RuleBaseManager;

/**
 * a server to provide resources to the launched JVM during an inter-JVM 
 * validation. This server will also keep track of a variety of events occurring
 * during the validation process, notifying any registered listeners of these
 * events.
 * 
 * @see org.eclipse.actf.core.server.ResourceClient
 * @author Mike Squillace
 */
public class ResourceServer extends AbstractSingletonServer
{

	public static final String NULL_STRING = "__<NULL>__";

	public static final String REQUEST_CONFIGURATION = "requestConfig";
	public static final String REQUEST_RULEBASE_DESCRIPTORS = "requestRulebaseDescriptors";
	public static final String REQUEST_CONFIGURATION_PARAMETER = "requestConfigParam";
	public static final String REQUEST_RESOURCE = "requestResource";

	public static final String REPORT_STARTED = "reportStarted";
	public static final String REPORT_COMPLETED = "reportCompleted";
	public static final String CREATION_CONTEXT_STARTED = "creationContextsStarted";
	public static final String CREATION_CONTEXT_COMPLETED = "creationContextsCompleted";
	public static final String VALIDATION_JVM_TERMINATING = "valJVMTerminating";

	public static final String REQUEST_ACCESSIBILITY_SERVICES = "requestAccessibilityServices";

	private static ResourceServer server;
	private ServerSocketChannel _serverChannel;
	private Selector _selector;
	private IConfiguration _config;
	private List _reportListeners = new LinkedList();
	private ReportEvent _reportEvent = new ReportEvent(this);

	private ResourceServer () throws IOException {
		_serverChannel = ServerSocketChannel.open();
		_serverChannel.configureBlocking(false);
		_selector = Selector.open();
		_serverChannel.register(_selector, SelectionKey.OP_ACCEPT);
		
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			_config = context.getConfiguration();			
		}catch (ConfigurationException e) {
			throw new RuntimeException(e.getMessage(), e);
		}
		
		Thread serverThread = new Thread(new Runnable() {
			public void run () {
				startServer();				
			}
		});
		serverThread.setName("resource server");
		serverThread.setDaemon(true);
		serverThread.start();
	} // ctor

	/**
	 * get a resource server for fetching resources from the launching JVM. The server is automatically started.
	 * 
	 * @return resource server instance
	 * @throws IOException
	 */
	public static ResourceServer getInstance () throws IOException {
		if (server == null) {
			server = new ResourceServer();
		}
		return server;
	}

	/**
	 * bind the server to a socket and start the server. Once this method is
	 * invoked successfully, use the <code>isSuspended</code> method to monitor whether or not 
	 * any data is being read.
	 * 
	 * @see org.eclipse.actf.core.server.AbstractSingletonServer#isSuspended()
	 */
	public void startServer () {
		SelectionKey key = null;
		ServerSocket socket = _serverChannel.socket();
		try {
			socket.bind(new InetSocketAddress(getHostName(), getPort()));
			socket.setReuseAddress(true);
		}catch (IOException e) {
			Utils.println(IReporter.SYSTEM_FATAL, "Unable to start resource server", e);
			throw new RuntimeException("Unable to start resource server");
		}
		while (!isShutdown()) {
			try {
				if (isSuspended()) {
					Thread.sleep(1000);
					continue;
				}
				
				_selector.select();
				Set keys = _selector.selectedKeys();
				for (Iterator iter = keys.iterator(); iter.hasNext();) {
					key = (SelectionKey) iter.next();
					iter.remove();
					if (key.isAcceptable()) {
						SocketChannel client = _serverChannel.accept();
						client.configureBlocking(false);
						client.register(_selector, SelectionKey.OP_READ);
					}else if (!isSuspended() && key.isValid() && key.isReadable()) {
						SocketChannel client = (SocketChannel) key.channel();
						String request = receiveMessage(client);
						handleRequest(request, client);
					}
				}
			}catch (Exception e) {
				Utils.println(IReporter.SYSTEM_NONFATAL, e);
				if (key != null) {
					key.cancel();
				}
			}
		}
	}

	protected ByteBuffer toByteBuffer (String msg)
		throws CharacterCodingException {
		return CHARSET_ENCODER.encode(CharBuffer.wrap(msg));
	}

	protected void handleRequest (String request, SocketChannel client)
		throws Exception {
		if (request.equals(REQUEST_CLOSE)) {
			Utils.println(Utils.ALL, "ResourceServer processing REQUEST_CLOSE request..");
			client.close();
		}else if (request.equals(REQUEST_CONFIGURATION)) {
			Utils.println(Utils.ALL, "ResourceServer processing REQUEST_CONFIGURATION request.");
			send(client, asXmlConfiguration(_config));
		} else if (request.equals(REQUEST_RULEBASE_DESCRIPTORS)) {
			Utils
					.println(Utils.ALL,
							"ResourceServer processing REQUEST_RULEBASE_DESCRIPTORS request.");
			IRuleBaseDescriptor[] bases = RuleBaseManager.getInstance()
					.getRuleBaseDescriptors();
			String count = bases == null ? "null" : String
					.valueOf(bases.length);
			Utils.println(Utils.ALL,
					"ResourceServer processing REQUEST_RULEBASE_DESCRIPTORS request. count="
							+ count);
			send(client, bases);
		} else if (request.equals(REQUEST_ACCESSIBILITY_SERVICES)) {
			Utils
					.println(Utils.ALL,
							"ResourceServer processing REQUEST_ACCESSIBILITY_SERVICES request.");
			Map accservices = requestAccessibilityServices();
			send(client, accservices);
		} else if (request.startsWith(REQUEST_RESOURCE)) {
			Utils.println(Utils.ALL,
					"ResourceServer processing REQUEST_RESOURCE request.");
			StringTokenizer st = new StringTokenizer(request.substring(request
					.indexOf(':') + 1), ",");
			String id = st.nextToken();
			String base = st.nextToken();
			String ext = st.nextToken();
			Utils.println(Utils.ALL, "ResourceServer getting resource from locator " + id);
			
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();			
			IResourceLocator locator = context.getResourceLocator();
			InputStream resource = locator.getResourceAsStream(
				id, base.equals(NULL_STRING) ? null : base,
				ext.equals(NULL_STRING) ? null : ext, null);
			byte[] buff = new byte[0];
			if (resource != null) {
				ByteArrayOutputStream boutput = new ByteArrayOutputStream();
				buff = new byte[32768];
				int read;
				while ((read = resource.read(buff)) > 0) {
					boutput.write(buff, 0, read);
				}
				buff = boutput.toByteArray();
			}
			sendResource(client, buff, buff.length);
		}else if (request.startsWith(REQUEST_CONFIGURATION_PARAMETER)) {
			Utils.println(Utils.ALL, "ResourceServer processing REQUEST_CONFIGURATION_PARAMETER request.");
			StringTokenizer st = new StringTokenizer(request.substring(request.indexOf(':') + 1), ",");
			String poolId = st.nextToken();
			String symbolId = st.nextToken();
			if (poolId != null && symbolId != null) {
				Utils.println(Utils.ALL, "ResourceServer getting parameter " + poolId + ":" + symbolId);
				_config.setSymbolPool(poolId);
				send(client, _config.getParameter(symbolId));
			}else {
				Utils.println(Utils.ERRORS,
				"Error in ids for configuration parameter request: " + "pool=" + poolId + ", symbolId=" + symbolId);
			}
		}else if (request.startsWith(REPORT_STARTED)) {
			Utils.println(Utils.ALL, "ResourceServer processing REPORT_STARTED request.");
			int start = request.indexOf('{') + 1;
			int closing = request.lastIndexOf('}');
			int end = closing == -1 ? request.length() : closing;
			_reportEvent.setReportID(request.substring(start, end));
			fireReportStarted();
		}else if (request.equals(CREATION_CONTEXT_STARTED)) {
			Utils.println(Utils.ALL, "ResourceServer processing CREATION_CONTEXT_STARTED request.");
			fireCreationContextStarted();
		}else if (request.equals(REPORT_COMPLETED)) {
			Utils.println(Utils.ALL, "ResourceServer processing REPORT_COMPLETED request.");
			_reportEvent.queueReportRecords(requestReport(client));
			fireReportCompleted();
			_reportEvent.clear();
		}else if (request.equals(CREATION_CONTEXT_COMPLETED)) {
			Utils.println(Utils.ALL, "ResourceServer processing CREATION_CONTEXT_COMPLETED request.");
			_reportEvent.queueCreationContexts(requestCreationContexts(client));
			fireCreationContextCompletedE();
		}else if (request.equals(VALIDATION_JVM_TERMINATING)) {
			Utils.println(Utils.ALL, "ResourceServer processing VALIDATION_JVM_TERMINATING request.");
			setSuspended(true);
		}
	}

	public ValidationReportRecord[] requestReport (SocketChannel client) throws Exception {
		ByteArrayInputStream bos = new ByteArrayInputStream(requestAsBytes(client));
		ValidationReportRecord[] report = (ValidationReportRecord[]) new ObjectInputStream(bos).readObject();
		return report;
	}

	public CreationContext[] requestCreationContexts (SocketChannel client)
		throws Exception {
		ByteArrayInputStream bais = new ByteArrayInputStream(requestAsBytes(client));
		CreationContext[] contexts = (CreationContext[]) new ObjectInputStream(bais).readObject();
		return contexts;
	}

	protected void send (SocketChannel client, Object obj) throws IOException {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(65536);
		ObjectOutputStream oos = new ObjectOutputStream(baos);
		oos.writeObject(obj);
		baos.flush();
		oos.close();
		byte[] ba = baos.toByteArray();
		sendResource(client, ba, ba.length);
	} // send(Object)

	protected void finalize () throws Throwable {
		if (_serverChannel != null) {
			_serverChannel.close();
			_selector.close();
		}
	}

	protected void fireReportStarted () {
		for (Iterator iter = _reportListeners.iterator(); iter.hasNext();) {
			((IReportListener) iter.next()).reportStarted(_reportEvent);
		}
	}

	protected void fireReportCompleted () {
		for (Iterator iter = _reportListeners.iterator(); iter.hasNext();) {
			((IReportListener) iter.next()).reportCompleted(_reportEvent);
		}
	}

	protected void fireCreationContextStarted () {
		for (Iterator iter = _reportListeners.iterator(); iter.hasNext();) {
			((IReportListener) iter.next()).creationContextStarted(_reportEvent);
		}
	}

	protected void fireCreationContextCompletedE () {
		for (Iterator iter = _reportListeners.iterator(); iter.hasNext();) {
			((IReportListener) iter.next()).creationContextCompleted(_reportEvent);
		}
	}

	/**
	 * add a ReportListener to be notified when a report has started or completed
	 * 
	 * @param listener -
	 *            listener to be called upon report event
	 */
	public void addReportListener (IReportListener listener) {
		_reportListeners.add(listener);
	}

	/**
	 * remove the specified listener from the list of listeners to be notified
	 * when report events take place.
	 * 
	 * @param listener -
	 *            listener to be removed
	 */
	public void removeReportListener (IReportListener listener) {
		_reportListeners.remove(listener);
	}

	/**
	 * remove all report listeners
	 * 
	 */
	public void removeAllReportListeners () {
		_reportListeners.clear();
	}
	
	private XmlConfiguration asXmlConfiguration (IConfiguration config) throws ConfigurationException {
		String[] poolNames = config.getSymbolPools();
		XmlConfiguration xmlConfig = new XmlConfiguration();
		for (int p = 0; p < poolNames.length; ++p) {
			xmlConfig.createSymbolPool(poolNames[p]);
			xmlConfig.addConfigurationData(config.getSymbolPoolContents(poolNames[p]));
		}
		return xmlConfig;
	}
	
	/** 
	 * Get the accservice Map from the AccessibilityServiceManager  via reflection
	 * @return  accservice Map
	 */
	private Map requestAccessibilityServices() {
		Object accServiceManager = null;
		Map accservices = null;
		Class accService = ClassLoaderCache.getDefault().classForName(
				"org.eclipse.actf.accservice.AccessibilityServiceManager");
		if (accService != null) {
			try {
				Method getAccServicesInstance = accService.getMethod(
						"getInstance", (Class[]) null);
				accServiceManager = getAccServicesInstance.invoke(null);

			} catch (Exception e) {
				Utils.println(
						"Unable to get AccessibilityServiceManager instance", e);
			}
			

			if (accServiceManager != null) {
				try {
					Method getAccServicesMeth = accService.getMethod(
							"getRegisteredAccessibilityServicesMap",
							(Class[]) null);
					accservices = (Map) getAccServicesMeth.invoke(
							accServiceManager, (Object[]) null);

				} catch (Exception e) {
					Utils.println(
							"Unable to getRegisteredAccessibilityServicesMap",
							e);
				}
			}
		}
		return accservices;
	}
}
