/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.launch;

import java.io.File;
import java.util.ArrayList;
import java.util.Map;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.javapp.launch.AspectJValidationLauncher;
import org.eclipse.actf.validation.javapp.launch.ValidationLauncher;
import org.eclipse.actf.validation.javapp.ui.JavaValidationUIPlugin;
import org.eclipse.actf.validation.javapp.ui.preferences.JavaValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.jobs.ValidationReportListener;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.actf.validation.ui.views.ReportManagerView;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IRuntimeClasspathEntry;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaLaunchDelegate;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.PlatformUI;

public class JavaLaunchConfigurationDelegate extends JavaLaunchDelegate
{

	private static final String ACTF_JAVA_LAUNCH = "org.eclipse.actf.validation.javapp.ui.launch.java";

	private static JavaValidationUIPlugin plugin = (JavaValidationUIPlugin.getDefault());

	private static IPreferenceStore store = ValidationUiPlugin.getDefault().getPreferenceStore();

	private static IConfiguration config;

	private ValidationReportListener valListener = null;

	private ValidationLauncher launcher = null;

	private String name = null;

	private DebugPlugin debugPlugin = null;

	//TODO: This needs to be tested under Eclipse 3.1 !!!!!
	public JavaLaunchConfigurationDelegate () throws CoreException {
		super();
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			config = context.getConfiguration();			
		}catch (ConfigurationException e) {
			//Could not obtain a Configuration. We can't do much here but quit.
			throw new RuntimeException(e.getMessage(), e);
		}
	}

	public static boolean isEclipse32 () {
		boolean usesEclipseApp = false;
		try {
			Class.forName("org.eclipse.pde.internal.ui.launcher.EclipseApplicationLauncherTabGroup");
			usesEclipseApp = true;
		}catch (ClassNotFoundException e) {
		}
		return usesEclipseApp;
	}

	public void launch (ILaunchConfiguration configuration, String mode,
						ILaunch launch, IProgressMonitor monitor)
		throws CoreException {
		name = getMainTypeName(configuration);
		config.setSymbolPool(IConfiguration.ACTF_ID);
		String aopfile = store.getString(JavaValidationPreferenceInitializer.P_AOP_ASPECTPATH);
		String libPath = config.getStringParameter(ValidationConstants.ACTF_LIB_PATH_KEY);
		String tempDir = config.getStringParameter(IConfiguration.WORKING_DIR_KEY);
		launcher = new AspectJValidationLauncher(name, aopfile, tempDir);
		launcher.addSystemProperty(RuntimeContextFactory.RUNTIME_CONTEXT_CLASSNAME_KEY, "org.eclipse.actf.validation.javapp.runtime.InterJvmRuntimeContext");		
		if (libPath != null && libPath.trim().length() > 0) {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			String msaaDir  =  context.getResourceLocator().getPath("org.eclipse.actf.accservice.win32.ia2"/* AccessibilityServicePlugin.ACCESSIBILITY_MSAA_BUNDLE */);
			libPath = libPath + File.pathSeparator + msaaDir;
			launcher.addSystemProperty("java.library.path", libPath);
		}
		launcher.addSystemProperty(
			ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY,
			store.getString(ValidationPreferenceInitializer.P_CODE_PROCESSOR_CLASSPATH));
		valListener = new ValidationReportListener();
		valListener.set_source(name);
		debugPlugin = DebugPlugin.getDefault();
		launcher.addReportListener(valListener);
		debugPlugin.addDebugEventListener(this);
		Map props = ValidationPreferenceInitializer.toMap(store.getString(JavaValidationPreferenceInitializer.P_SYSTEM_PROPERTIES));
		launcher.addSystemProperties(props);
		// Determine the JVM version being used to launch the app
		String version = null;
		try {
			// IVMInstall2 not supported in Eclipse 3.0.x
			IVMInstall javaLauncher = getVMInstall(configuration);
			Class iv2Type = Class.forName("org.eclipse.jdt.launching.IVMInstall2");
			if (iv2Type.isAssignableFrom(javaLauncher.getClass())) {
				version = (String) iv2Type.getMethod("getJavaVersion", (Class[]) null).invoke(
					javaLauncher, (Object[]) null);
			}
		}catch (Exception e) {
		}
		boolean isJava14 = version == null || version.indexOf("1.4") >= 0;
		launcher.setUsingJava14(isJava14);
		// Copy the classpath from the launch config to the Validation Launcher
		String[] classpath = getClasspath(configuration);
		StringBuffer cp = new StringBuffer();
		for (int c = 0; c < classpath.length; ++c) {
			cp.append(classpath[c]);
			cp.append(File.pathSeparator);
		}
		launcher.setClasspath(cp.toString());
		launcher.setExecutionArguments("");
		try {
			launcher.prepareForLaunch();
		}catch (Exception e1) {
			throw new CoreException(new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.OK, e1.getMessage() == null ? "Error in launch preparation" : e1.getMessage(), e1));
		}
		ILaunchConfigurationWorkingCopy copy = configuration.getWorkingCopy();
		StringBuffer jvmArgs = new StringBuffer();
		String[] args = launcher.allJvmArguments();
		String launcherCp = launcher.getClasspath();
		if (launcherCp.startsWith("\"")) {
			launcherCp = launcherCp.substring(1, launcherCp.length() - 1);
		}
		// Create an XML formatted memento string for the classpath 
		// so it can be saved as an attribute in the launch configuration
		// The classpath contains both user specified classpath entries
		// and actf required classpath entries.
		String[] paths = launcherCp.split(File.pathSeparator);
		ArrayList classPathMementos = new ArrayList();
		for (int c = 0; c < paths.length; c++) {
			Path path = new Path(paths[c]);
			IRuntimeClasspathEntry classPathEntry = JavaRuntime.newArchiveRuntimeClasspathEntry(path);
			String memento = classPathEntry.getMemento();
			classPathMementos.add(memento);
		}
		copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_DEFAULT_CLASSPATH, false);
		copy.setAttribute(IJavaLaunchConfigurationConstants.ATTR_CLASSPATH, classPathMementos);
		// Note: The code seems to work without setting the bootstrap attribute.  This 
		// attribute was set when launching the Eclipse Runtime Workbench.  
		//		copy.setAttribute("bootstrap", launcherCp);
		//		let the delegate handle the classpath for rtwb -
		//		we don't want a -cp arg
		for (int arg = 0; arg < args.length; ++arg) {
						if (args[arg].equals("-cp")) {
						++arg;
						continue;
						}
			jvmArgs.append(args[arg]);
			jvmArgs.append(' ');
		}
		String vmArgsKey = IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS;
		copy.setAttribute(vmArgsKey, jvmArgs.toString());
		launch.setAttribute(ACTF_JAVA_LAUNCH, "true");
		super.launch(copy, mode, launch, monitor);
	}

	public void handleDebugEvents (DebugEvent[] events) {
		for (int e = 0; e < events.length; ++e) {
			if (events[e].getSource() instanceof IProcess) {
				final IProcess process = (IProcess) events[e].getSource();
				final String tool = "Java Application with ACTF";
				if (process.getLaunch().getAttribute(ACTF_JAVA_LAUNCH).equals(
					"true")
						&& events[e].getKind() == DebugEvent.TERMINATE) {
					final ReportManagerView[] view = new ReportManagerView[1];
					PlatformUI.getWorkbench().getDisplay().syncExec(
						new Runnable() {

							public void run () {
								view[0] = (ReportManagerView) AbstractViewPart.findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
								if (view[0] != null && valListener != null) {
									view[0].createReports(
										valListener.getAllReports(),
										"Report on " + name, name, tool,
										SwtGuiModel.SWT_MODEL);
								}
							}
						});
					if (launcher != null) {
						launcher.removeReportListener(valListener);
					}
					if (debugPlugin != null) {
						debugPlugin.removeDebugEventListener(this);
					}
				}
			}
		}
	}
}
