/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.launch;

import java.io.File;
import java.lang.reflect.Method;
import java.util.Arrays;
import java.util.Map;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.javapp.launch.AspectJValidationLauncher;
import org.eclipse.actf.validation.javapp.launch.ValidationLauncher;
import org.eclipse.actf.validation.javapp.ui.JavaValidationUIPlugin;
import org.eclipse.actf.validation.javapp.ui.preferences.JavaValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.jobs.ValidationReportListener;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.actf.validation.ui.views.ReportManagerView;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IDebugEventSetListener;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.LaunchConfigurationDelegate;
import org.eclipse.jdt.launching.IJavaLaunchConfigurationConstants;
import org.eclipse.jdt.launching.IVMInstall;
import org.eclipse.jdt.launching.JavaRuntime;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.PlatformUI;

public class WorkbenchLaunchConfigurationDelegate extends LaunchConfigurationDelegate
	implements IDebugEventSetListener
{

	private static final String ACTF_RTWB_LAUNCH = "org.eclipse.actf.validation.javapp.ui.launch.rtwb";
	
	private static JavaValidationUIPlugin plugin = (JavaValidationUIPlugin.getDefault());
	private static IPreferenceStore store = plugin.getPreferenceStore();
	private static IConfiguration config;

	private DebugPlugin debugPlugin = DebugPlugin.getDefault();
	private ValidationReportListener valListener = null;
	private ValidationLauncher launcher = null;
	private LaunchConfigurationDelegate _underlyingDelegate;

	public WorkbenchLaunchConfigurationDelegate () throws CoreException {
		super();
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			config = context.getConfiguration();			
		}catch (ConfigurationException e) {
			//Could not obtain a Configuration. We can't do much here but quit.
			throw new RuntimeException(e.getMessage(), e);
		}
		if (isEclipse32()) {
			try {
				_underlyingDelegate = (LaunchConfigurationDelegate) Class.forName(
					"org.eclipse.pde.ui.launcher.EclipseApplicationLaunchConfiguration").newInstance();
			}catch (Exception e) {
				plugin.logException(e);
			}
		}else {
			try {
				_underlyingDelegate = (LaunchConfigurationDelegate) Class.forName(
					"org.eclipse.pde.internal.ui.launcher.WorkbenchLaunchConfigurationDelegate").newInstance();
			}catch (Exception e) {
				plugin.logException(e);
			}
		}
	}

	public static boolean isEclipse32 () {
		boolean usesEclipseApp = false;
		try {
			Class.forName("org.eclipse.pde.internal.ui.launcher.EclipseApplicationLauncherTabGroup");
			usesEclipseApp = true;
		}catch (ClassNotFoundException e) {
		}
		return usesEclipseApp;
	}

	public void launch (ILaunchConfiguration configuration, String mode,
						ILaunch launch, IProgressMonitor monitor)
		throws CoreException {
		String name = "org.eclipse.core.launcher.Main";
		
		config.setSymbolPool(IConfiguration.ACTF_ID);
		String aopfile = store.getString(JavaValidationPreferenceInitializer.P_AOP_ASPECTPATH);
		String libPath = config.getStringParameter(ValidationConstants.ACTF_LIB_PATH_KEY);
		String tempDir = config.getStringParameter(IConfiguration.WORKING_DIR_KEY);
		launcher = new AspectJValidationLauncher(name, aopfile, tempDir);
		launcher.addSystemProperty(RuntimeContextFactory.RUNTIME_CONTEXT_CLASSNAME_KEY, "org.eclipse.actf.validation.javapp.runtime.InterJvmRuntimeContext");		
		if (libPath != null && libPath.trim().length() > 0) {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
			String msaaDir  =  context.getResourceLocator().getPath("org.eclipse.actf.accservice.win32.ia2" /* AccessibilityServicePlugin.ACCESSIBILITY_MSAA_BUNDLE */);
			libPath = libPath + File.pathSeparator + msaaDir;
			launcher.addSystemProperty("java.library.path", libPath);
		}
		launcher.addSystemProperty(
			ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY,
			store.getString(ValidationPreferenceInitializer.P_CODE_PROCESSOR_CLASSPATH));
		valListener = new ValidationReportListener();
		valListener.set_source(name);
		launcher.addReportListener(valListener);
		debugPlugin.addDebugEventListener(this);
		Map props = ValidationPreferenceInitializer.toMap(store.getString(JavaValidationPreferenceInitializer.P_SYSTEM_PROPERTIES));
		launcher.addSystemProperties(props);
		IVMInstall javaLauncher = JavaRuntime.computeVMInstall(configuration);
		//		IVMInstall2 not supported in Eclipse 3.0.x
		Class iv2Type = null;
		String version = null;
		try {
			iv2Type = Class.forName("org.eclipse.jdt.launching.IVMInstall2");
			if (iv2Type.isAssignableFrom(javaLauncher.getClass())) {
				version = (String) iv2Type.getMethod("getJavaVersion", (Class[]) null).invoke(
					javaLauncher, (Object[]) null);
			}
		}catch (Exception e) {
		}
		boolean isJava14 = version == null || version.indexOf("1.4") >= 0;
		launcher.setUsingJava14(isJava14);
		String vmArgsKey = null;
		Method meth = null;
		Object obj = null;
		try {
			if (isEclipse32()) {
				obj = _underlyingDelegate;
				meth = _underlyingDelegate.getClass().getMethod(
					"getClasspath", new Class[] {ILaunchConfiguration.class});
				vmArgsKey = IJavaLaunchConfigurationConstants.ATTR_VM_ARGUMENTS;
			}else {
				meth = Class.forName(
					"org.eclipse.pde.internal.ui.launcher.LauncherUtils").getMethod(
					"constructClasspath",
					new Class[] {ILaunchConfiguration.class});
				vmArgsKey = (String) Class.forName(
					"org.eclipse.pde.internal.ui.launcher.ILauncherSettings").getField(
					"VMARGS").get(null);
			}
			String[] classpath = meth != null ? (String[]) meth.invoke(
				obj, new Object[] {configuration}) : new String[0];
			StringBuffer cp = new StringBuffer();
			for (int c = 0; c < classpath.length; ++c) {
				cp.append(classpath[c]);
				cp.append(File.pathSeparator);
			}
			launcher.setClasspath(cp.toString());
		}catch (Exception e) {
			throw new CoreException(new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.OK, e.getMessage() == null ? "Error in launch preparation"
					: e.getMessage(), e));
		}
		try {
			launcher.prepareForLaunch();
		}catch (Exception e1) {
			throw new CoreException(new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.OK, e1.getMessage() == null ? "Error in launch preparation"
					: e1.getMessage(), e1));
		}
		ILaunchConfigurationWorkingCopy copy = configuration.getWorkingCopy();
		StringBuffer jvmArgs = new StringBuffer();
		String[] args = launcher.allJvmArguments();
		String launcherCp = launcher.getClasspath();
		if (launcherCp.startsWith("\"")) {
			launcherCp = launcherCp.substring(1, launcherCp.length() - 1);
		}
		copy.setAttribute(
			IJavaLaunchConfigurationConstants.ATTR_DEFAULT_CLASSPATH, false);
		copy.setAttribute(
			IJavaLaunchConfigurationConstants.ATTR_CLASSPATH,
			Arrays.asList(launcherCp.split(File.pathSeparator)));
		//		earlier than Eclipse 3.2
		copy.setAttribute("bootstrap", launcherCp);
		//		let the delegate handle the classpath for rtwb -
		//		we don't want a -cp arg
		for (int arg = 0; arg < args.length; ++arg) {
			if (args[arg].equals("-cp")) {
				++arg;
				continue;
			}
			jvmArgs.append(args[arg]);
			jvmArgs.append(' ');
		}
		copy.setAttribute(vmArgsKey, jvmArgs.toString());
		copy.doSave();
		launch.setAttribute(ACTF_RTWB_LAUNCH, "true");
		_underlyingDelegate.launch(configuration, mode, launch, monitor);
	}

	public void handleDebugEvents (DebugEvent[] events) {
		for (int e = 0; e < events.length; ++e) {
			if (events[e].getSource() instanceof IProcess) {
				final IProcess process = (IProcess) events[e].getSource();
				final String name = "Eclipse Application";
				final String tool = "Eclipse Application with ACTF";
				if (process.getLaunch().getAttribute(ACTF_RTWB_LAUNCH).equals(
					"true")
						&& events[e].getKind() == DebugEvent.TERMINATE) {
					final ReportManagerView[] view = new ReportManagerView[1];
					PlatformUI.getWorkbench().getDisplay().syncExec(
						new Runnable() {

							public void run () {
								view[0] = (ReportManagerView) AbstractViewPart.findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
								if (view[0] != null && valListener != null) {
									view[0].createReports(
										valListener.getAllReports(),
										"Report on " + name, name, tool,
										SwtGuiModel.SWT_MODEL);
								}
							}
						});
					if (launcher != null) {
						launcher.removeReportListener(valListener);
					}
					if (debugPlugin != null) {
						debugPlugin.removeDebugEventListener(this);
					}
				}
			}
		}
	}
}
