/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.preferences;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.Dictionary;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.config.Version;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.model.javapp.SwtGuiModel;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.util.resources.IResourceLocator;
import org.eclipse.actf.validation.ValidationConstants;
import org.eclipse.actf.validation.ValidationCorePlugin;
import org.eclipse.actf.validation.javapp.JavaValidationConstants;
import org.eclipse.actf.validation.javapp.JavaValidationPlugin;
import org.eclipse.actf.validation.javapp.launch.ValidationLauncher;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.core.runtime.Platform;
import org.eclipse.jdt.launching.JavaRuntime;
import org.osgi.framework.Bundle;

public class JavaValidationPreferenceInitializer extends ValidationPreferenceInitializer
{

	
	private static final String JAVA_BINARY_PATH = "bin" + File.separator + "java";
	private static final String JAVA_JRE_BIN_PATH = "jre" + File.separator + "bin";
	private static final String JAVA_BIN_PATH = "bin";
	private static final String JAVA_JRE_PATH = "jre";
	
	private static final String JAVA_VERSION_ARG = "-version";
	private static final String JAVA_VERSION_STR = "java version";
	private static final String JAVA_VERSION_14x = "1.4";
	private static final String JAVA_VERSION_15x = "1.5";
	private static final String JAVA_VERSION_50x = "5.0";
	
	private static final String ECLIPSE_SWT_BUNDLE = "org.eclipse.swt";
	
	//keys should be prefaced with their appropriate pool id
	// Configuration.ACTF_ID is default
	//public static final String POOL_ID_DELIMITER = "$";

	/* Keys taken from previous raven.core.config.Configuration */
	
	public static final String ACTF_AOP_PATH_KEY = "actf.aop.path";

	public static final String ACTF_AOP_ENGINE_PATH_KEY = "actf.aop.engine.path";

	public static final String ACTF_AOP_VERBOSE_KEY = "actf.aop.verbose";	

	public static final String JAVA_HOME_KEY = "java.home";


	/* AOP-related keys */
	public static final String P_CREATION_CONTEXTS_PROCESSING = ValidationConstants.VALIDATION_ID
			+ ValidationPreferenceInitializer.POOL_ID_DELIMITER
			+ JavaValidationConstants.CREATION_CONTEXTS_PROCESSING_KEY;

	public static final String P_AOP_VERBOSE = ValidationConstants.VALIDATION_ID
			+ ValidationPreferenceInitializer.POOL_ID_DELIMITER + JavaValidationConstants.ACTF_AOP_VERBOSE_KEY;

	public static final String P_SWT_RUNTIME_HOME = SwtGuiModel.SWT_RUNTIME_HOME_KEY;

	public static final String P_AOP_ASPECTPATH = JavaValidationConstants.ACTF_AOP_PATH_KEY;

	public static final String P_SYSTEM_PROPERTIES = "actf.aop.sys.props";

	public static final String P_AOP_JVM = "actf.aop.jvm";

	public static final String P_OSGI_CLASSLOADER = "osgi.parentClassloader";

	public static final String P_OSGI_LOGFILE = "osgi.logfile";

	public static final String P_OSGI_INSTALL_AREA = "osgi.install.area";

	public static final String JVM_14x = "JVM14x";

	public static final String JVM_15x = "JVM15x";	

	public static final String ACTF_JVM_TRACE = "actf_jvm_trace.out";

	/* keys for <code>JavaToolsPreferencePage</code> */
	

	public static final String P_JAVA_HOME = ValidationLauncher.JAVA_HOME_KEY;
		
	public static final String CREATION_CONTEXTS_PROCESSING_KEY = "creation.contexts.processing";

	public static final String PROCESS_NO_CONTEXTS = "noContexts";
	
	/* server-related keys */
	public static final String RESOURCE_SERVER_HOST_KEY = "resource.server.host";

	public static final String RESOURCE_SERVER_PORT_KEY = "resource.server.port";

	public static final String RESOURCE_SERVER_HOST_DEFAULT = "localhost";

	public static final String RESOURCE_SERVER_PORT_DEFAULT = "20013";		
	
	public static final String OSGI_CLASSLOADER_DEFAULT = "app";

	public static final String OSGI_LOGFILE_DEFAULT = "osgi.log";

	public static final String P_RESOURCE_SERVER_PORT = ValidationConstants.VALIDATION_ID
	+ ValidationPreferenceInitializer.POOL_ID_DELIMITER + RESOURCE_SERVER_PORT_KEY;	
	
	public static final String[][] ccChoices = new String[][] {
			{"Components with violations", JavaValidationConstants.PROCESS_VIOLATION_CONTEXTS},
			{"No components", PROCESS_NO_CONTEXTS}};
	
	public JavaValidationPreferenceInitializer() {
		super();
	}

	public void initializeDefaultPreferences() {
		super.initializeDefaultPreferences();
		updateModelList();

		config.setSymbolPool(IConfiguration.ACTF_ID);
		// set the path to the default AOP libs
		config.setParameter(ACTF_AOP_ENGINE_PATH_KEY, getDefaultAOPEngineClasspath());
		// set the default Java root directory
		preferencesNode.put(
			P_JAVA_HOME, getDefaultJavaPath());
		config.setParameter(
			JAVA_HOME_KEY, preferencesNode.get(
				P_JAVA_HOME, ""));

		preferencesNode.put(
			P_SWT_RUNTIME_HOME, getSWTRuntimeHome());
		config.setParameter(
			SwtGuiModel.SWT_RUNTIME_HOME_KEY, preferencesNode.get(
				P_SWT_RUNTIME_HOME, ""));		
		
		String processorClassPath = preferencesNode.get( ValidationPreferenceInitializer.P_CODE_PROCESSOR_CLASSPATH , "");		
		StringBuffer initialProcessorClasspath = new StringBuffer( getSWTJarLocation());
		
		if ( initialProcessorClasspath.length() > 0 ) {
			initialProcessorClasspath.append(File.pathSeparator);
		}
		initialProcessorClasspath.append(getSWTJarLocation());

		preferencesNode.put(
			ValidationPreferenceInitializer.P_CODE_PROCESSOR_CLASSPATH, initialProcessorClasspath.toString());
		config.setParameter(
			ValidationConstants.CODE_PROCESSOR_CLASSPATH_KEY,
			initialProcessorClasspath.toString());		
		
		
		preferencesNode.put(
			P_CREATION_CONTEXTS_PROCESSING,
			JavaValidationConstants.PROCESS_VIOLATION_CONTEXTS);
		config.setParameter(
			CREATION_CONTEXTS_PROCESSING_KEY,
			JavaValidationConstants.PROCESS_VIOLATION_CONTEXTS);
		preferencesNode.put(
			P_AOP_ASPECTPATH, getDefaultAOPPaths());
		preferencesNode.putBoolean(P_AOP_VERBOSE, false);
		config.setParameter(ACTF_AOP_VERBOSE_KEY, false);
		preferencesNode.put(
			P_AOP_JVM, isJavaRuntime14(preferencesNode.get(
				P_JAVA_HOME, "")) ? JVM_14x : JVM_15x);
		Map m = new HashMap();
		String actfExtJvmDump = tmpdir == null ? File.separator
				+ ACTF_JVM_TRACE : tmpdir + File.separator + ACTF_JVM_TRACE;
		m.put(ValidationLauncher.EXTERNAL_JVM_DUMP, actfExtJvmDump);
		String osgiLog = tmpdir == null ? File.separator + OSGI_LOGFILE_DEFAULT
				: tmpdir + File.separator + OSGI_LOGFILE_DEFAULT;
		m.put(P_OSGI_LOGFILE, osgiLog);
		m.put(P_OSGI_CLASSLOADER, OSGI_CLASSLOADER_DEFAULT);
		
		// TODO should be determined dynamically
		String[] plugins = {
				//"org.eclipse.actf.accservice.win32.msaa", 
				// AccessibilityServicePlugin.ACCESSIBILITY_MSAA_BUNDLE,
							JavaValidationPlugin.MODELJAVACORE_PLUGIN_ID, 
							ValidationCorePlugin.VALIDATION_CORE_PLUGIN_ID 
							};
		m.put( IResourceLocator.ACTF_RESOURCES_PATHS_KEY, getResourcePath( plugins ) );
		
		preferencesNode.put(
			P_SYSTEM_PROPERTIES, ValidationPreferenceInitializer.toStringList(m));
				
		String currentJarPath = config.getStringParameter(ValidationConstants.ACTF_CLASSPATH_KEY );
		String jarPath = getModelJavaJarPath( currentJarPath );
		
		preferencesNode.put(ValidationPreferenceInitializer.P_ACTFPATH, jarPath);
		config.setParameter(ValidationConstants.ACTF_CLASSPATH_KEY,
				preferencesNode.get(ValidationPreferenceInitializer.P_ACTFPATH, ""));		

		preferencesNode.put(
			P_RESOURCE_SERVER_PORT,
			RESOURCE_SERVER_PORT_DEFAULT);
		config.setSymbolPool( ValidationConstants.VALIDATION_ID );
		config.setParameter(
			RESOURCE_SERVER_PORT_KEY,
			RESOURCE_SERVER_PORT_DEFAULT);		
	}

	private String getModelJavaJarPath ( String currentJarPath ) {
		// The default AOP Engine path is the ACTF core directory
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		
		StringBuffer buffer = new StringBuffer( currentJarPath );
		
		String[] jars = { JavaValidationPlugin.JAVA_VALIDATION_CORE_JAR };
		
		for( int i=0; i < jars.length; i++ ) {
			String jarPath = context.getResourceLocator().getPath(jars[i]);
			
			if ( jarPath != null ) {
				if ( buffer.length() > 0 ) {
					buffer.append( File.pathSeparator);
				}
				buffer.append( jarPath );
			}
		}
		
		return buffer.toString();
	}	
	
	private String getResourcePath( String[] resources ) {
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		StringBuffer buffer = new StringBuffer();
				
		for( int i=0; i < resources.length; i++ ) {
			String resourcePath = context.getResourceLocator().getPath(resources[i]);
			
			if ( resourcePath != null) {
				if ( buffer.length() > 0 ) {
					buffer.append( File.pathSeparator);
				}
				buffer.append( resourcePath );
				buffer.append( "resources");
			}
		}
		return buffer.toString();		
	}
	
	/**
	 * Utility to get the version of the Java JDK. The javaRoot is assumed to be
	 * the path to the root directory of the Java install to use for
	 * validations. The subdirectory "bin" will be appended to it.
	 * 
	 * @param javaRoot
	 *            the root directory of a Java installation
	 * @return String of either "1.4", "1.5" or "5.0".
	 */
	private String getJavaVersion (String javaRoot) {
		String javaVersion = null;
		String javaCmd = null;
		if (javaRoot.endsWith(File.separator)) {
			javaCmd = javaRoot + JAVA_BINARY_PATH;
		}else {
			javaCmd = javaRoot + File.separator + JAVA_BINARY_PATH;
		}
		try {
			String[] cmdarray = {javaCmd, JAVA_VERSION_ARG};
			Process proc = Runtime.getRuntime().exec(cmdarray);
			// Create a BufferedReader for the error output of the java -version
			// command.
			// The output is actually returned on the error input stream and not
			// the
			// regular input stream.
			InputStreamReader esr = new InputStreamReader(proc.getErrorStream());
			BufferedReader ebr = new BufferedReader(esr);
			// read the output
			String line;
			while ((line = ebr.readLine()) != null) {
				if (line.startsWith(JAVA_VERSION_STR)) {
					StringBuffer versionsb = new StringBuffer(line.substring(
						line.indexOf('"') + 1, line.lastIndexOf('"')));
					if (versionsb.indexOf(JAVA_VERSION_14x) >= 0) {
						javaVersion = JAVA_VERSION_14x;
					}else if ((versionsb.indexOf(JAVA_VERSION_15x) >= 0)
							|| versionsb.indexOf(JAVA_VERSION_50x) >= 0) {
						javaVersion = JAVA_VERSION_15x;
					}
				}
				Utils.println(IReporter.DETAIL, line);
			}
			// check for ls failure
			if (proc.waitFor() != 0) {
				Utils.println(
					IReporter.ERROR,
					"Failure during exec to obtain jdk version, exit value = "
							+ proc.exitValue());
			}
		}catch (IOException ioe) {
			Utils.println(IReporter.SYSTEM_NONFATAL, ioe.getMessage(), ioe);
		}catch (InterruptedException e) {
			Utils.println(IReporter.SYSTEM_NONFATAL, e.getMessage(), e);
		}
		return javaVersion;
	}

	/**
	 * Utility that returns a boolean value of true if the JVM for launching
	 * external validations is 1.4. False, otherwise.
	 * 
	 * @param javaRoot
	 * @return isJavaRuntime14
	 */
	private boolean isJavaRuntime14 (String javaRoot) {
		boolean isJava14 = false;
		String javaVersion = getJavaVersion(javaRoot);
		if (javaVersion != null) {
			if (javaVersion.equalsIgnoreCase(JAVA_VERSION_14x)) {
				isJava14 = true;
			}
		}else {
			// Something happened in the exec of the process to execute "java
			// -version".
			// Default to Java 1.4 since it will ClassLoader AOP Weaving will
			// work for either 1.4 or 1.5.
			isJava14 = true;
		}
		return isJava14;
	}
	
	/**
	 * Utility to get the path to the root of the Java JDK used by Eclipse. This
	 * is the same JDK as one selected in the Java preferences pages under Java ->
	 * Installed JREs.
	 * 
	 * @return the path to the root of the selected JDK in Eclipse
	 */
	private String getDefaultJavaPath () {
		String javaRoot = null;
		File defaultJVMLoc = JavaRuntime.getDefaultVMInstall().getInstallLocation();
		try {
			javaRoot = defaultJVMLoc.getCanonicalPath();
			Utils.println(
				IReporter.DETAIL,
				"getDefaultJavaPath(), path to default JVM - "
						+ javaRoot);
			if (!validateJavaPath(javaRoot)) {
				Utils.println(
					IReporter.DETAIL,
					"getDefaultJavaPath(), path to default JVM is not java root directory. Looking for jre directory.");
				StringBuffer defaultPath = new StringBuffer(javaRoot.replace(
					'\\', '/'));
				if (defaultPath.toString().endsWith("/")) {
					defaultPath.append(JAVA_JRE_PATH);
				}else {
					defaultPath.append('/');
					defaultPath.append(JAVA_JRE_PATH);
				}
				if (validateJavaPath(defaultPath.toString())) {
					return defaultPath.toString();
				}else {
					Utils.println(
						IReporter.SYSTEM_NONFATAL,
						"Unable to confirm that the default Eclipse JRE path is valid. Returning Eclipse JRE path - "
								+ javaRoot);
				}
			}
		}catch (IOException ioe) {
			Utils.println(IReporter.SYSTEM_NONFATAL, ioe.getMessage(), ioe);
		}
		return javaRoot;
	}

	/**
	 * @param javaPathStr
	 * @return true iff the path contains a valid Java runtime
	 */
	private boolean validateJavaPath (String javaPathStr) {
		String baseDir;
		String exePath;
		boolean pathIsValid = false;
		File javaPathFile, exePathFile;
		if ((javaPathStr == null) || (javaPathStr.trim().length() <= 0)) { return false; }
		try {
			javaPathFile = new File(javaPathStr);
			if (javaPathFile.isDirectory()) {
				baseDir = javaPathStr.replace('\\', '/');
				if (baseDir.endsWith("/")) {
					exePath = baseDir + JAVA_BINARY_PATH;
				}else {
					exePath = baseDir + '/' + JAVA_BINARY_PATH;
				}
				if (Platform.getOS().equalsIgnoreCase(Platform.OS_WIN32)) {
					exePath = exePath + ".exe";
				}
				exePathFile = new File(exePath);
				if (exePathFile.isFile()) {
					pathIsValid = true;
				}
			}
		}catch (Exception e) {
			Utils.println(
				IReporter.SYSTEM_NONFATAL, "Unable to validate Java path - "
						+ javaPathStr, e);
			pathIsValid = false;
		}
		return pathIsValid;
	}	
	
	private String getDefaultAOPEngineClasspath () {
		// The default AOP Engine path is whereever the AspectJ plugins are located
		
		StringBuffer result = new StringBuffer();
		
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		String runtime = context.getResourceLocator().getPath(JavaValidationPlugin.ASPECTJ_RUNTIME_JAR);
		String weaver  = context.getResourceLocator().getPath(JavaValidationPlugin.ASPECTJ_WEAVER_JAR);


		if ( runtime != null && runtime.length() > 0 ) {
			result.append( runtime );
		}
		
		if ( weaver != null && weaver.length() > 0 ) {
			if ( result.length() > 0 ) {
				result.append( File.pathSeparator );
			}
			result.append( weaver );
		}

		return result.toString();
	}
	
	private String getDefaultAOPPaths () {
		StringBuffer buffer = new StringBuffer();
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		
		String swingPath = context.getResourceLocator().getPath("resources/swing-accessibility");		
		String swtPath   = context.getResourceLocator().getPath("resources/swt-accessibility");	
		if ( swingPath != null ) {
			buffer.append( swingPath );
			buffer.append( File.pathSeparator );
		}
		
		if ( swtPath != null ) {
			buffer.append( swtPath );
		}
				
		return buffer.toString();
	}
	
	private Version getSwtVersion () {
		Version swtVersion = null;
		Bundle platformBundle = Platform.getBundle("org.eclipse.swt");
		Dictionary platformDict = platformBundle.getHeaders();
		if (!platformDict.isEmpty()) {
			String version = (String) platformDict.get(Version.bundleVersionKey);
			swtVersion = new Version(version);
		}else {
			swtVersion = new Version("0.0.0");
		}
		Utils.println(
			IReporter.DETAIL,
			"getSwtVersion(), SWT version - "
					+ swtVersion.toString());
		return swtVersion;
	}

	/**
	 * @return SWT runtime home
	 */
	private String getSWTRuntimeHome () {
		String swtRuntimeHome = null;
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();			
			String swtHomeName = ECLIPSE_SWT_BUNDLE + "." + Platform.getWS() + "." + Platform.getOS() + "." + Platform.getOSArch();
			swtRuntimeHome = context.getResourceLocator().getPath(swtHomeName);
		}catch (Exception e) {
			Utils.println(IReporter.SYSTEM_NONFATAL, e);
		}

		if (swtRuntimeHome != null) {
			Utils.println(IReporter.DETAIL, "resolveSWTRuntimeHome(), SWT runtime home - " + swtRuntimeHome);
		}else {
			Utils.println(IReporter.ERROR, "Unable to find SWT bundle - " + ECLIPSE_SWT_BUNDLE);
		}
		
		return swtRuntimeHome;
	}

	/**
	 * @return SWT jar location
	 */
	private String getSWTJarLocation () {
		String swtJarLocation = null;
		try {			
			IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();			
			String bundleName = ECLIPSE_SWT_BUNDLE + "." + Platform.getWS() + "." + Platform.getOS() + "." + Platform.getOSArch();
			swtJarLocation = context.getResourceLocator().getPath(bundleName);
			
		}catch (Exception e) {
			Utils.println(IReporter.SYSTEM_NONFATAL, e);
		}

		if (swtJarLocation == null || swtJarLocation.length() == 0 ) {
			Utils.println(IReporter.ERROR, "Unable to find SWT bundle - " + ECLIPSE_SWT_BUNDLE);
		} else {
			Utils.println(IReporter.DETAIL, "resolveSWTJarLocation(), SWT jar location - " + swtJarLocation);
		}
		
		return swtJarLocation;
	}	
}

