/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.validate;

import java.awt.Frame;
import java.awt.LayoutManager;
import java.awt.Panel;
import java.io.File;
import java.lang.reflect.Constructor;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JPanel;

import org.eclipse.actf.validation.core.Reflector;
import org.eclipse.jdt.core.IClasspathEntry;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.jdt.core.IJavaProject;
import org.eclipse.jdt.core.IMethod;
import org.eclipse.jdt.core.IType;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;

public class JavaElementResolver
{

	public static Set SUPPORTED_GUI_TYPES = new HashSet();
	static {
		SUPPORTED_GUI_TYPES.add(Composite.class);
		SUPPORTED_GUI_TYPES.add(Panel.class);
		SUPPORTED_GUI_TYPES.add(JPanel.class);
		SUPPORTED_GUI_TYPES.add(Frame.class);
		SUPPORTED_GUI_TYPES.add(JFrame.class);
	};

	public static boolean isSupportedGuiType (Class c) {
		boolean found = false;
		for (Iterator iter = SUPPORTED_GUI_TYPES.iterator(); !found
				& iter.hasNext();) {
			found = ((Class) iter.next()).isAssignableFrom(c);
		}
		return found;
	}

	public static final Class[][] SUPPORTED_PARAMETER_TYPE_COMBINATIONS = new Class[][] {
			new Class[0], new Class[] {Frame.class}, new Class[] {Panel.class},
			new Class[] {Composite.class}, new Class[] {Shell.class},
			new Class[] {Composite.class, Integer.TYPE}};

	public static boolean isSupportedMember (Member member) {
		boolean result = false;
		Class[] types = member instanceof Constructor ? ((Constructor) member).getParameterTypes()
				: ((Method) member).getParameterTypes();
		for (int cc = 0; !result
				& cc < SUPPORTED_PARAMETER_TYPE_COMBINATIONS.length; ++cc) {
			result = Arrays.equals(
				types, SUPPORTED_PARAMETER_TYPE_COMBINATIONS[cc]);
		}
		return result;
	}

	protected static Map PRIMATIVE_TYPE_MAP = new HashMap();
	static {
		PRIMATIVE_TYPE_MAP.put("B", Byte.TYPE);
		PRIMATIVE_TYPE_MAP.put("C", Character.TYPE);
		PRIMATIVE_TYPE_MAP.put("D", Double.TYPE);
		PRIMATIVE_TYPE_MAP.put("I", Integer.TYPE);
		PRIMATIVE_TYPE_MAP.put("J", Long.TYPE);
		PRIMATIVE_TYPE_MAP.put("S", Short.TYPE);
		PRIMATIVE_TYPE_MAP.put("Z", Boolean.TYPE);
	}

	private URLClassLoader _loader;

	public JavaElementResolver () {
	}

	public Class findTargetClass (IJavaElement element) throws Exception {
		Class cls = null;
		while (element != null
				&& (element.getElementType() != IJavaElement.TYPE || (element.getElementType() == IJavaElement.TYPE && ((IType) element).isAnonymous()))) {
			IJavaElement last = element;
			element = element.getAncestor(IJavaElement.TYPE);
			if (element == last) {
				element = element.getParent();
			}
		}
		if (element != null) {
			IType type = (IType) element;
			if (type.isClass()) {
				IJavaProject project = type.getJavaProject();
				IClasspathEntry[] entries = project.getResolvedClasspath(true);
				URL[] urls = new URL[entries.length + 1];
				File projLoc = new File(project.getResource().getLocation().toOSString()).getParentFile();
				urls[0] = new File(projLoc, project.getOutputLocation().toOSString()).toURL();
				for (int e = 0; e < entries.length; ++e) {
					urls[e + 1] = entries[e].getPath().toFile().toURL();
				}
				_loader = new IntraJvmWeavingClassLoader(urls, null, getClass().getClassLoader());
				cls = Class.forName(type.getFullyQualifiedName(), true, _loader);
			}
		}
		return cls;
	}

	public Member findTargetMember (IJavaElement element, Class targetClass)
		throws Exception {
		Member member = null;
		if (element.getElementType() == IJavaElement.TYPE) {
			member = findSupportedConstructor(targetClass);
		}else if (element.getElementType() == IJavaElement.METHOD) {
			IMethod method = (IMethod) element;
			String[] paramTypes = method.getParameterTypes();
			String methName = method.getElementName();
			if (method.isMainMethod()) {
				throw new IllegalArgumentException("Use Java GUI View in ACTF External Perspective to launch this class via the main(String[]) method");
			}else if (method.isConstructor()) {
				member = findResolvedMember(targetClass, paramTypes, null);
			}else {
				member = findResolvedMember(targetClass, paramTypes, methName);
			}
		}else {
			throw new UnsupportedOperationException("Validation of element of type "
					+ element.getClass().getName() + " not supported");
		}
		return member;
	}

	protected Member findResolvedMember (Class targetClass,
											String[] paramTypes, String methName)
		throws Exception {
		Reflector reflector = new Reflector();
		Member member = null;
		Member[] candidates = methName == null ? (Member[]) targetClass.getDeclaredConstructors()
				: (Member[]) reflector.findMethods(targetClass, methName).toArray(
					new Member[0]);
		for (int c = 0; member == null & c < candidates.length; ++c) {
			Class[] types = candidates[c] instanceof Constructor ? ((Constructor) candidates[c]).getParameterTypes()
					: ((Method) candidates[c]).getParameterTypes();
			boolean match = types.length == paramTypes.length;
			for (int cc = 0; match & cc < types.length; ++cc) {
				match &= matches(types[cc], paramTypes[cc]);
			}
			if (match) {
				member = candidates[c];
			}
		}
		return member;
	}

	public Constructor findSupportedConstructor (Class targetClass)
		throws Exception {
		Constructor ctor = null;
		if (Panel.class.isAssignableFrom(targetClass)) {
			Constructor[] ctors = targetClass.getConstructors();
			for (int c = 0; c < ctors.length; ++c) {
				Class[] types = ctors[c].getParameterTypes();
				if (types.length == 0) {
					ctor = ctors[c];
					break;
				}else if (types.length == 1
						&& LayoutManager.class.isAssignableFrom(types[0])) {
					ctor = ctors[c];
				}
			}
		}else if (Frame.class.isAssignableFrom(targetClass)) {
			Constructor[] ctors = targetClass.getConstructors();
			for (int c = 0; c < ctors.length; ++c) {
				Class[] types = ctors[c].getParameterTypes();
				if (types.length == 0) {
					ctor = ctors[c];
					break;
				}else if (types.length == 1
						&& String.class.isAssignableFrom(types[0])) {
					ctor = ctors[c];
				}
			}
		}else if (Composite.class.isAssignableFrom(targetClass)) {
			Constructor[] ctors = targetClass.getConstructors();
			for (int c = 0; c < ctors.length; ++c) {
				Class[] types = ctors[c].getParameterTypes();
				if (types.length == 1
						&& Composite.class.isAssignableFrom(types[0])) {
					ctor = ctors[c];
					break;
				}else if (types.length == 2
						&& Composite.class.isAssignableFrom(types[0])
						&& Integer.TYPE.equals(types[1])) {
					ctor = ctors[c];
				}
			}
		}else {
			Constructor[] ctors = targetClass.getConstructors();
			OUTER: for (int c = 0; c < ctors.length; ++c) {
				for (int cc = 0; cc < SUPPORTED_PARAMETER_TYPE_COMBINATIONS.length; ++cc) {
					Class[] types = ctors[c].getParameterTypes();
					if (Arrays.equals(
						types, SUPPORTED_PARAMETER_TYPE_COMBINATIONS[cc])) {
						ctor = ctors[c];
						break OUTER;
					}
				}
			}
		}
		return ctor;
	}

	protected boolean matches (Class cls, String type) {
		Class primType = (Class) PRIMATIVE_TYPE_MAP.get(type);
		boolean result = primType != null && primType.equals(cls);
		if (!result) {
			String clsName = cls.getName();
			String name = clsName.substring(clsName.lastIndexOf('.') + 1);
			result = type.substring(1, type.length() - 1).equals(name);
		}
		return result;
	}
}
