/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.validate;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Window;
import java.lang.reflect.Constructor;
import java.lang.reflect.Member;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.ui.jobs.ValidationJob;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.jdt.core.IJavaElement;
import org.eclipse.swt.SWT;
import org.eclipse.swt.awt.SWT_AWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Shell;

public class JavaValidationJob extends ValidationJob
{
	
	public static final String JAVA_VALIDATION_JOB_NAME = "Java Validation";

	private JavaElementResolver _resolver = new JavaElementResolver();
	private Constructor _constructor;
	private Method _method;
	private IJavaElement _element;

	public JavaValidationJob (String name) {
		super(name);
	}

	public IJavaElement getJavaElement () {
		return _element;
	}

	public void reset (Object resource) throws Exception {
		IJavaElement element = (IJavaElement) resource;
		Class cls = _resolver.findTargetClass(element);
		if (cls != null) {
			Member member = _resolver.findTargetMember(element, cls);
			if (member != null) {
				_constructor = member instanceof Constructor ? (Constructor) member
						: _resolver.findSupportedConstructor(cls);
				_method = member instanceof Method ? (Method) member
						: (Method) null;
				setResource(element.getUnderlyingResource());
				_element = element;
			}else {
				throw new IllegalArgumentException("Could not find appropriate constructor or method for element "
						+ element.getClass().getName());
			}
		}else {
			throw new IllegalArgumentException("Could not find or instantiate class for element "
					+ element.getClass().getName());
		}
	}

	protected void syncClassAndMembers () throws Exception {
		Class cls = _resolver.findTargetClass(_element);
		_constructor = cls.getConstructor(_constructor.getParameterTypes());
		if (_method != null) {
			_method = cls.getMethod(
				_method.getName(), _method.getParameterTypes());
		}
	}

	protected IStatus run (IProgressMonitor monitor) {
		IStatus status = Status.OK_STATUS;
		try {
			boolean isNewAction = ((Boolean) getProperty(NEW_ACTION_QNAME)).booleanValue();
			Platform.getJobManager().join(
				ResourcesPlugin.FAMILY_AUTO_BUILD, null);
			configureReportInstance();
			
			try {
				Class cls = Class.forName("org.eclipse.actf.validation.javapp.aspects.IntraJvmValidationAspect");
				Method meth = cls.getMethod(
					"setCurrentReport", new Class[] {ValidationReport.class});
				meth.invoke(null, new Object[] {currentReport});
			}catch (Exception e1) {
				throw new RuntimeException(e1);
			}			
			
			if (isNewAction) {
				syncClassAndMembers();
			}
			Display.getDefault().syncExec(new Runnable() {
				public void run () {
					refreshAndValidate();
				}
			});
		}catch (Exception e) {
			plugin.logException(e);
			status = new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.ERROR, "Error running validation", e);
		}
		return status;
	}

	protected void refreshAndValidate () {
		Map instanceMap = makeInstance();
		Object instance = instanceMap.keySet().iterator().next();
		Object gui = instanceMap.get(instance);
		if (_method != null) {
			Object methGui = invokeMethod(instance);
			if (methGui != null) {
				gui = methGui;
			}
		}
		if (gui != null) {
			part.setValidatable(gui);
			validate();
		}
	}

	private Map makeInstance () {
		// parameter types for constructor of validated class
		Class[] types = _constructor.getParameterTypes();
		// type of validated class
		Class declaring = _constructor.getDeclaringClass();
		// is the validated class extending a supported type?
		boolean isGuiType = JavaElementResolver.isSupportedGuiType(declaring);
		// either the class extends a known GUI type or
		// its first constructor parameter is a known GUI type
		Class compArgType = !isGuiType && types.length > 0 ? types[0] : null;
		Class guiType = isGuiType ? declaring : compArgType;
		Map instanceMap = new HashMap();
		Object gui = null, instance = null;
		try {
			if (!isGuiType && compArgType == null) {
				// if not extending supported GUI type and constructor is no-arg, simply instantiate
				instance = declaring.newInstance();
			}else if (Composite.class.isAssignableFrom(guiType)) {
				// either extends Composite or takes a Shell or Composite as an argument to the constructor
				// following are simply temporary ancestors --
				// real parent is viewer
				Display display = Shell.class.isAssignableFrom(guiType) ? Display.getCurrent() : null;
				Shell shell = new Shell(display, SWT.SHELL_TRIM);
				Composite composite = new Composite(shell, SWT.BORDER);
				Object[] args = null;
				if (isGuiType) {
					// extends Composite
					args = new Object[types.length];
					args[0] = composite;
					if (types.length == 2) {
						args[1] = new Integer(SWT.NONE);
					}
					gui = instance = _constructor.newInstance(args);
				}else {
					// constructor takes a Shell or Composite
					args = new Object[1];
					if (display != null) {
						composite.dispose();
						composite = shell;
					}
					args[0] = composite;
					instance = _constructor.newInstance(args);
					gui = composite;
				}
			}else if (Component.class.isAssignableFrom(guiType)) {
				// validated class either extends a Swing type or its
				// constructor takes a Swing type as its first parameter
				Composite composite = new Composite(new Shell(SWT.SHELL_TRIM), SWT.BORDER | SWT.EMBEDDED);
				Frame f = SWT_AWT.new_Frame(composite);
				f.setLayout(new BorderLayout());
				parentMap.put(composite, f);
				gui = composite;
				if (isGuiType && !Window.class.isAssignableFrom(declaring)) {
					// GUI is a panel so simply instantiate it
					instance = _constructor.newInstance((Object[]) null);
					f.add((Component) instance, BorderLayout.CENTER);
				}else if (isGuiType && Frame.class.isAssignableFrom(declaring)) {
					// GUI extends a Frame so simply instantiate it
					gui = instance = _constructor.newInstance((Object[]) null);
					composite.dispose();
					parentMap.clear();
				}else if (!isGuiType) {
					// constructor takes a Swing component as 1st parameter
					if (Frame.class.isAssignableFrom(compArgType)) {
						instance = _constructor.newInstance(new Object[] {f});
					}else if (Panel.class.isAssignableFrom(compArgType)) {
						Panel p = new Panel(new BorderLayout());
						f.add(p);
						instance = _constructor.newInstance(new Object[] {p});
					}
				}
			}
		}catch (Exception e) {//e.printStackTrace(System.err)
			plugin.logException(e);
		}
		instanceMap.put(instance, gui);
		return instanceMap;
	}

	private Object invokeMethod (Object instance) {
		Object gui = null;
		if (JavaElementResolver.isSupportedMember(_method)) {
			Class[] types = _method.getParameterTypes();
			Object[] args = new Object[types.length];
			if (types.length == 0) {
				args = null;
			}else if (Shell.class.isAssignableFrom(types[0])) {
				gui = new Shell(Display.getCurrent(), SWT.SHELL_TRIM);
				args[0] = gui;
			}else if (Composite.class.isAssignableFrom(types[0])) {
				gui = new Composite(new Shell(SWT.SHELL_TRIM), SWT.BORDER);
				args[0] = gui;
				if (args.length == 2) {
					args[1] = new Integer(SWT.NONE);
				}
			}else if (Panel.class.isAssignableFrom(types[0])) {
				gui = new Composite(new Shell(SWT.SHELL_TRIM), SWT.BORDER
						| SWT.EMBEDDED);
				Frame f = SWT_AWT.new_Frame((Composite) gui);
				f.setLayout(new BorderLayout());
				parentMap.put(gui, f);
				if (Frame.class.isAssignableFrom(types[0])) {
					args[0] = f;
				}else if (Panel.class.isAssignableFrom(types[0])) {
					Panel p = new Panel(new BorderLayout());
					f.add(p);
					args[0] = p;
				}
			}
			try {
				_method.invoke(instance, args);
			}catch (Exception e) {
				plugin.logException(e);
			}
		}else {
			throw new IllegalArgumentException("Unsupported method: " + _method);
		}
		return gui;
	}
}
