/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.javapp.ui.views;

import java.awt.Frame;
import java.awt.Window;

import org.eclipse.actf.validation.javapp.ui.JavaValidationUIConstants;
import org.eclipse.actf.validation.javapp.ui.validate.JavaValidationElementChangedListener;
import org.eclipse.actf.validation.ui.jobs.IValidatableGuiContainer;
import org.eclipse.actf.validation.ui.jobs.ValidationJob;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.actf.validation.ui.views.PartAdapter;
import org.eclipse.jdt.core.JavaCore;
import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IContributionItem;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IPartListener;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;

public class RenderedGuiView extends AbstractViewPart
	implements IValidatableGuiContainer
{

	public static final String RENDERED_GUI_VIEW_ID = "org.eclipse.actf.validation.javapp.ui.views.renderedGuiView";	
	
	private Composite _view;

	private Object _gui;

	private ValidationJob _job;

	private JavaValidationElementChangedListener _listener;

	private boolean _autoRefresh = true;

	private IPartListener partListener = new PartAdapter() {

		public void partActivated (IWorkbenchPart part) {
			enableToolbarButton(
				JavaValidationUIConstants.RGVIEW_TERMINATE_VALIDATION_ACTION,
				getValidatable() != null);
			enableManualValidationButton(getValidatable());
		}
	};

	public RenderedGuiView () {
		super();
	}

	public void init (IViewSite site) throws PartInitException {
		super.init(site);
		_listener = new JavaValidationElementChangedListener(this);
		JavaCore.addElementChangedListener(_listener);
	}

	public void dispose () {
		getSite().getPage().removePartListener(partListener);
		JavaCore.removeElementChangedListener(_listener);
		disposeGui();
		if (_view != null && !_view.isDisposed()) {
			_view.dispose();
		}
		_view = null;
		super.dispose();
	}

	protected void disposeGui () {
		if (_gui != null && _gui instanceof Shell) {
			((Shell) _gui).dispose();
		}else if (_gui != null && _gui instanceof Frame) {
			((Window) _gui).dispose();
		}
	}

	public Object getValidatable () {
		return _gui;
	}

	public boolean isAutoValidating () {
		return _autoRefresh;
	}

	public void setAutoValidating (boolean auto) {
		_autoRefresh = auto;
		enableManualValidationButton(_gui);
		// Check the button appropriately for the given state
		IToolBarManager tbMgr = getViewSite().getActionBars().getToolBarManager();
		IContributionItem item = tbMgr.find(JavaValidationUIConstants.RGVIEW_AUTO_REFRESH_ACTION);
		if (item instanceof ActionContributionItem) {
			ActionContributionItem aci = (ActionContributionItem) item;
			aci.getAction().setChecked(auto);
		}
	}

	protected void enableManualValidationButton (Object gui) {
		boolean enableManualRefresh = (gui == null) ? false : !_autoRefresh;
		enableToolbarButton(
			JavaValidationUIConstants.RGVIEW_MANUAL_VALIDATION_ACTION,
			enableManualRefresh);
	}

	public ValidationJob getValidationJob () {
		return _job;
	}

	public void setValidationJob (ValidationJob job) {
		_job = job;
	}

	protected String getMessageTitle () {
		return "Java Validation";
	}

	public void setFocus () {
		_view.setFocus();
	}

	public void setValidatable (Object gui) {
		refreshView(gui);
		if (gui instanceof Shell) {
			reportMessage("Validating shell shown in separate window");
			Shell s = (Shell) gui;
			s.setText("ACTF Validation");
			s.pack();
			s.open();
		}else if (gui instanceof Frame) {
			reportMessage("Validating Frame shown in separate window");
			Frame f = (Frame) gui;
			String title = "ACTF Validation";
			if (f.getTitle().trim().length() > 0) {
				title = "ACTF Validation - " + f.getTitle();
			}
			f.setTitle(title);
			f.pack();
			f.setVisible(true);
		}
		_gui = gui;
	}

	public void createPartControl (Composite parent) {
		super.createPartControl(parent);
		_view = new Composite(parent, SWT.BORDER);
		_view.setLayout(new FillLayout());
		getSite().getPage().addPartListener(partListener);
		reportMessage("No GUI being validated");
	}

	protected void refreshView (Object gui) {
		clearView();
		if (gui == null) {
			reportMessage("No GUI being validated");
			disposeGui();
		}else if (gui instanceof Composite) {
			Composite composite = (Composite) gui;
			_view.setSize(composite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
			composite.setParent(_view);
			update(viewParent);
		}
		enableToolbarButton(
			JavaValidationUIConstants.RGVIEW_TERMINATE_VALIDATION_ACTION, gui != null);
		enableManualValidationButton(gui);
	}

	protected void reportMessage (String message) {
		Text msg = new Text(_view, SWT.LEFT | SWT.READ_ONLY);
		msg.setText(message);
		update(_view);
	}

	protected void clearView () {
		Control[] children = _view.getChildren();
		for (int c = 0; c < children.length; ++c) {
			if (!children[c].isDisposed()) {
				children[c].dispose();
			}
		}
		// If gui created an external Frame then dispose of it so we don't 
		// get multiple external frames displayed on each validation
		if (_gui instanceof Frame) {
			Frame frame = (Frame) _gui;
			frame.dispose();
		}
	}

	public void update (Composite c) {
		c.layout();
		c.redraw();
		c.setVisible(true);
	}
}
