/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  {INITIAL AUTHOR} - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.reports;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;

import org.eclipse.actf.util.logging.IReporter;

/**
 * used as the basic reporting unit for a validation report. Hard-coded fields
 * include the level of the violation and its corresponding string
 * representation, the unique id of the component causing the problem, the class
 * name of the component, the message describing the problem, a component name
 * (if available), a long description of the violation, and a timestamp. Other
 * fields can be added.
 * 
 * @author Mike Squillace
 */
public class ValidationReportRecord extends ReportElement implements Comparable {

	static final long serialVersionUID = 5090644786827554826L;

	public static final String NODEID_FIELDNAME = "nodeid";
	public static final String RULEID_FIELDNAME = "ruleid";
	public static final String NAME_FIELDNAME = "compname";
	public static final String LEVEL_FIELDNAME = "level";
	public static final String MESSAGE_FIELDNAME = "message";
	public static final String LONGMESSAGE_FIELDNAME = "longmessage";
	public static final String NODENAME_FIELDNAME = "nodename";
	public static final String LOCATIONPATH_FIELDNAME = "locationpath";
	public static final String TIMESTAMP_FIELDNAME = "timestamp";

	private static final String[] BUILTIN_FIELDS = {
		NODEID_FIELDNAME, RULEID_FIELDNAME, LEVEL_FIELDNAME, MESSAGE_FIELDNAME, LONGMESSAGE_FIELDNAME,
		NAME_FIELDNAME, NODENAME_FIELDNAME, LOCATIONPATH_FIELDNAME, TIMESTAMP_FIELDNAME 
	};

	protected static String sortKey = "level";

	protected static ArrayList fieldNames = new ArrayList();
	static {
		for (int f = 0; f < BUILTIN_FIELDS.length; ++f) {
			addAttributeName(BUILTIN_FIELDS[f]);
		}
	}

	protected int level;
	protected String nodeName, compName;
	protected String id;
	protected String message, longMessage;
	protected Calendar timestamp;
	protected String locationPath;
	protected String ruleId;
	
	protected HashMap fieldMap = new HashMap();
	protected HashMap fieldStringMap = new HashMap();

	/**
	 * create a new record for a validation report
	 * 
	 */
	public ValidationReportRecord() {
		setComponentName("<None>");
		setNodeName("<Unavailable>");
		setMessage("");
		setTimestamp(Calendar.getInstance());
	}

	/**
	 * set the field names for all records. This method should be used before
	 * any individual field is set.
	 * 
	 * @param fieldNames -
	 *            names of fields
	 */
	public static void addAttributeNames(String[] fieldNames) {
		for (int f = 0; fieldNames != null && f < fieldNames.length; ++f) {
			addAttributeName(fieldNames[f]);
		}
	}

	/**
	 * get the field names being used for generated records
	 * 
	 * @return field names being used by generated records
	 */
	public static String[] getAttributeNames() {
		return (String[]) fieldNames.toArray(new String[0]);
	}

	protected static void addAttributeName(String name) {
		if (!fieldNames.contains(name)) {
			fieldNames.add(name);
		}
	}

	/**
	 * set the sort key which must be one of the defined field names or a field
	 * added by user
	 * 
	 * @param field -
	 *            field according to which records are sorted
	 */
	public static void setSortKey(String field) {
		if (!fieldNames.contains(field)) {
			throw new IllegalArgumentException("Field " + field
					+ " does not exist for ValidationReportRecord objects");
		}
		sortKey = field;
	}

	/**
	 * return the sort key, the key according to which records are sorted
	 * 
	 * @return sort key
	 */
	public static String getSortKey() {
		return sortKey;
	}

	/**
	 * Gets the location path for a component.
	 * 
	 * @return the location path
	 */
	public String getLocationPath() {
		return locationPath;
	}

	/**
	 * Sets the location path for this component.
	 * 
	 * @param path -
	 *            the path to set.
	 */
	public void setLocationPath(String path) {
		locationPath = path;
		setAttribute(LOCATIONPATH_FIELDNAME, path);
		setAttributeAsString(LOCATIONPATH_FIELDNAME, path.toString());
	}

	/**
	 * @return Returns the class name of the component.
	 */
	public String getNodeName() {
		return nodeName;
	}

	/**
	 * @param name -
	 *            the node name of the component
	 *            @see org.eclipse.actf.model.IModel#getNodeName(Object)
	 */
	public void setNodeName(String name) {
		nodeName = name;
		setAttribute(NODENAME_FIELDNAME, name);
	}

	/**
	 * @return the node id
	 */
	public String getNodeId() {
		return id;
	}

	/**
	 * id of the component corresponding to this record. The id is used in
	 * conjunction with creation context ids.
	 * 
	 * @param id
	 */
	public void setNodeId(String id) {
		this.id = id;
		setAttribute(NODEID_FIELDNAME, id);
	}

	/**
	 * returns the rule id responsible for generating this record
	 * 
	 * @return the rule id or <code>null</code> if no id was given
	 */
	public String getRuleId () {
		return ruleId;
	}

	/**
	 * id of the rule corresponding to this record
	 * 
	 * @param id rule id
	 */
	public void setRuleId (String id) {
		ruleId = id;
		setAttribute(RULEID_FIELDNAME, id);
	}

	/**
	 * @return Returns the component name.
	 */
	public String getComponentName() {
		return compName;
	}

	/**
	 * @param name -
	 *            the name to set.
	 */
	public void setComponentName(String name) {
		this.compName = name;
		setAttributeAsString(NAME_FIELDNAME, name);
	}

	/**
	 * return the level as a string or label stripped of its prefix
	 * 
	 * @return level as a string or label
	 */
	public String getLevelAsString() {
		String sev = getAttributeAsString(LEVEL_FIELDNAME);
		int dot = sev.indexOf('.');
		if (dot >= 0) {
			sev = sev.substring(dot + 1);
		}
		return sev;
	}

	/**
	 * @return Returns the level.
	 */
	public int getLevel() {
		return level;
	}

	/**
	 * set the level of the violation or message. This should be one of the
	 * levels defined in <code>Reporter</code>.
	 * 
	 * @param level
	 *            The level to set.
	 * @see org.eclipse.actf.util.logging.IReporter
	 */
	public void setLevel(int level) {
		if (level < IReporter.MIN_LEVEL || level > IReporter.MAX_LEVEL) {
			throw new IllegalArgumentException("Illegal value for level:" + level);
		}
		this.level = level;
		setAttribute(LEVEL_FIELDNAME, new Integer(level));
	}

	/**
	 * @return Returns the message.
	 */
	public String getMessage() {
		return message;
	}

	/**
	 * @param message
	 *            The message to set.
	 */
	public void setMessage(String message) {
		this.message = message;
		setAttribute(MESSAGE_FIELDNAME, message);
	}

	/**
	 * @return Returns the long message.
	 */
	public String getLongMessage() {
		return longMessage;
	}

	/**
	 * @param message
	 *            The long message to set.
	 */
	public void setLongMessage(String message) {
		this.longMessage = message;
		setAttribute(LONGMESSAGE_FIELDNAME, message);
	}

	/**
	 * @return Returns the timestamp.
	 */
	public Calendar getTimestamp() {
		return timestamp;
	}

	/**
	 * @param timestamp
	 *            The timestamp to set.
	 */
	public void setTimestamp(Calendar timestamp) {
		this.timestamp = timestamp;
		setAttributeAsString(TIMESTAMP_FIELDNAME, timestamp.getTime()
				.toString());
	}

	/**
	 * set the specified field using the given value
	 * 
	 * @param field -
	 *            field to set
	 * @param val -
	 *            value of field
	 */
	public void setAttribute(String field, Object val) {
		if (!fieldNames.contains(field)) {
			throw new IllegalArgumentException("Field " + field + " does not exist for this record");
		}
		fieldMap.put(field, val);
	}

	/**
	 * get the value for the specified field
	 * 
	 * @param field -
	 *            field of which value is desired
	 * @return value of field
	 */
	public Object getAttribute(String field) {
		if (!fieldNames.contains(field)) {
			throw new IllegalArgumentException("Field " + field
					+ " does not exist for this record");
		}
		return fieldMap.get(field);
	}

	/**
	 * set the specified field using the given String value.
	 * 
	 * @param field -
	 *            field to set
	 * @param val -
	 *            String representation of value of field
	 */
	public void setAttributeAsString(String field, String val) {
		if (!fieldNames.contains(field)) {
			throw new IllegalArgumentException("Field " + field + " does not exist for this record");
		}
		fieldStringMap.put(field, val);
	}

	/**
	 * get the value for the specified field as a String.
	 * 
	 * @param field -
	 *            field of which value is desired
	 * @return value of field as a String
	 */
	public String getAttributeAsString(String field) {
		if (!fieldNames.contains(field)) {
			throw new IllegalArgumentException("Field " + field + " does not exist for this record");
		}
		String val = (String) fieldStringMap.get(field);
		if (val == null) {
			val = fieldMap.get(field).toString();
		}
		return val;
	}

	public int compareTo(Object o) {
		int result;
		if (o instanceof ValidationReportRecord) {
			ValidationReportRecord record = (ValidationReportRecord) o;
			if (sortKey.equals(LEVEL_FIELDNAME)) {
				if (record.getLevel() > this.getLevel()) {
					result = 1;
				} else {
					result = record.getLevel() == this.getLevel() ? 0 : -1;
				}
			} else if (sortKey.equals(NODEID_FIELDNAME)) {
				result = record.getNodeId().compareTo(this.getNodeId());
			} else if (sortKey.equals(NODENAME_FIELDNAME)) {
				result = this.getNodeName() == null || record.getNodeName() == null
					? -1 : this.getNodeName().compareTo(record.getNodeName());
			} else if (sortKey.equals(TIMESTAMP_FIELDNAME)) {
				if (this.getTimestamp().after(record.getTimestamp())) {
					result = 1;
				} else {
					result = this.getTimestamp().equals(record.getTimestamp()) ? 0 : -1;
				}
			} else {
				result = ((Comparable) getAttribute(sortKey)).compareTo(record.getAttribute(sortKey));
			}
		} else {
			throw new IllegalArgumentException(
					"Cannot compare ValidationReportRecord to object of type " + o.getClass().getName());
		}
		return result;
	} // compareTo

	public String toString() {
		StringBuffer sb = new StringBuffer();
		
		if (ruleId != null) {
			sb.append("{IRule:");
			sb.append(ruleId);
			sb.append('}');
			sb.append(' ');
		}
		
		sb.append(getAttributeAsString(LEVEL_FIELDNAME));
		sb.append('[');
		sb.append(getAttributeAsString(NODENAME_FIELDNAME));
		sb.append(':');
		String compName = getComponentName();
		if (compName != null) {
			sb.append(compName);
		}
		if (id != null && id.length() > 0) {
			sb.append('@');
			sb.append(getAttributeAsString(NODEID_FIELDNAME));
		}
		sb.append("] ");
		sb.append(getAttributeAsString(MESSAGE_FIELDNAME));
		
		return sb.toString();
	}

	public int hashCode() {
		return 37 * nodeName.hashCode() + id.hashCode();
	}

	public boolean equals(Object o) {
		boolean result = o instanceof ValidationReportRecord;
		if (result) {
			ValidationReportRecord r = (ValidationReportRecord) o;
			result = id.equals(r.id)
					&& nodeName.equals(r.nodeName)
					&& message.equals(r.message)
					&& ruleId.equals(r.ruleId);
		}
		return result;
	}
} // ValidationReportRecord
