/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.reports;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.MissingResourceException;

import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.AbstractReporter;
import org.eclipse.actf.util.logging.IErrorLogger;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.ValidationConstants;

/**
 * used to store logged records for later retrieval. Records can be retreaved
 * all at once or individually, sorted, and organized for a more readable
 * report. This is also the class that should be used when using List-like GUI
 * components for viewing reports.
 * 
 * @author Mike Squillace
 */
public class ValidationReporter extends AbstractReporter {

	private ArrayList records = new ArrayList();

	private int startRecord = 0, endRecord = -1;

	private boolean reportStarted;

	/**
	 * create a new ValidationReporter
	 */
	public ValidationReporter() {
		super();
	}

	/** {@inheritDoc} */
	public void open() {
	}

	/** {@inheritDoc} */
	public boolean isOpen() {
		return true;
	}

	/** {@inheritDoc} */
	public void close() {
	}

	/** {@inheritDoc} */
	public void flush() {
	}

	/** {@inheritDoc} */
	public void report(int level, String message, Object[] values) {
		if (!isOpen()) {
			throw new IllegalStateException("not open");
		}
		int defaultOutputLevel = getDefaultOutputLevel();
		if (defaultOutputLevel == UNKNOWN || level >= defaultOutputLevel) {
			ValidationReportRecord record = new ValidationReportRecord();
			if (bundle != null) {
				try {
					message = bundle.getString(message);
				} catch (MissingResourceException mre) {
					// do nothing
				}
			}
			if (level != UNKNOWN) {
				if (level < MIN_LEVEL || level > MAX_LEVEL) {
					throw new IllegalArgumentException("invalid level: " + level);
				}
				int lastDot = KEYS[level].lastIndexOf('.');
				String srcId = sourceID == null
					? KEYS[level].substring(0, lastDot) : sourceID;
				String levelStr = bundle != null
					? bundle.getString(KEYS[level])
					: KEYS[level].substring(lastDot + 1);
				record.setLevel(level);
				record.setAttributeAsString(ValidationReportRecord.LEVEL_FIELDNAME, levelStr);
				record.setRuleId(srcId);
			}
			
			if (!categories.isEmpty()) {
				String cat = (String) categories.peek();
				int left = cat.indexOf('[');
				int idStart = cat.indexOf("[@id=");
				if (left != -1) {
					record.setNodeName(cat.substring(0, left).trim());
					record.setComponentName(cat.substring(left + 1, cat.indexOf(']', left)));
				} else {
					record.setNodeName(cat);
					record.setComponentName("<None>");
				}
				if (idStart != -1) {
					record.setNodeId(cat.substring(idStart + "[@id=".length(), cat.indexOf(']', idStart)).trim());
				}
			}
			
			StringBuffer recordMsg = new StringBuffer();
			if (values != null && values.length > 0) {
				// TODO MAS: must fix
				if (values.length >= 1 && values[0] instanceof String) {
					record.setLocationPath((String) values[0]);
				}
				if (message != null && message.length() > 0) {
					//recordMsg.append(MessageFormat.format(message, values));
					recordMsg.append(message);
				}
			} else {
				if (message != null && message.length() > 0) {
					recordMsg.append(message);
				}
			}
			record.setMessage(recordMsg.toString());
			record.setTimestamp(Calendar.getInstance());
			Utils.println(Utils.PROCESS_ALL, "Adding record: "
					+ record);
			records.add(record);
		}
	} // report

	/** {@inheritDoc} */
	public void report(int level, String msg, Throwable t) {
		for (int l = 0; l < errorLoggers.size(); ++l) {
			IErrorLogger logger = (IErrorLogger) errorLoggers.get(l);
			logger.logError(msg, t);
		}
	}

	/** {@inheritDoc} */
	public String getLastReport() {
		return records.isEmpty() ? null : records.get(records.size() - 1).toString();
	}

	/** {@inheritDoc} */
	public void reset() {
		super.reset();
		records.clear();
	}

	/**
	 * return all records logged by this reporter thus far
	 * 
	 * @return all report records
	 */
	public ValidationReportRecord[] getRecords() {
		return (ValidationReportRecord[]) records.toArray(new ValidationReportRecord[0]);
	}

	/**
	 * get the number of report records logged by this reporter
	 * 
	 * @return record count
	 */
	public int getRecordCount() {
		return records.size();
	}

	/** {@inheritDoc} */
	public void startReport(Object comp) {
		if (!reportStarted) {
			startRecord = records.size();
			reportStarted = true;
		}
	}

	/** {@inheritDoc} */
	public void endReport() {
		if (!reportStarted) {
			throw new IllegalStateException(
					"Report has not been started on this reporter");
		}
		endRecord = records.size() - 1;
		reportStarted = false;
	}

	public ValidationReportRecord[] lastReport() {
		if (reportStarted) {
			throw new IllegalStateException("Report has not been completed");
		}
		ValidationReportRecord[] records = new ValidationReportRecord[endRecord - startRecord + 1];
		for (int r = startRecord; r < this.records.size() && r <= endRecord; ++r) {
			records[r - startRecord] = (ValidationReportRecord) this.records.get(r);
		}
		return records;
	}

	/** {@inheritDoc} */
	public void setLastReport(String reportText) {
	}

	/** {@inheritDoc} */
	public void addReportRecords(List records) {
		this.records.addAll(records);
	}

	/** {@inheritDoc} */
	public int getDefaultOutputLevel() {
		int result = IReporter.DEFAULT_LEVEL;
		// Always get the output level from the config
		try {
			IRuntimeContext context = RuntimeContextFactory.getInstance()
					.getRuntimeContext();
			IConfiguration config = context.getConfiguration();
			config.setSymbolPool(ValidationConstants.VALIDATION_ID);
			result = config
					.getIntParameter(ValidationConstants.REPORTER_LEVEL_KEY);
		} catch (Exception e) {
			// On failure do nothing, just return the default level
		}
		return result;
	}

	/** {@inheritDoc} */
	public void addReportRecord(ValidationReportRecord record) {
		ArrayList recordList = new ArrayList();
		recordList.add(record);
		addReportRecords(recordList);
	}
} // ValidationReporter
