/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rulebase;

import org.eclipse.actf.validation.core.ExecutionPointDescriptor;
import org.eclipse.actf.validation.rules.IRuleSet;


/**
 * main interface representing a collection of rules and their metadata in
 * ACTF. Currently, only rulebases described by XML documents (or <i>validation
 * documents</i>) are supported.
 * 
 * @see IDocumentRuleBase
 * @author John Buslawski
 * 
 */
public interface IRuleBase {

	/**
	 * get the name of this rulebase
	 * 
	 * @return name of rulebase
	 */
	public String getName();

	/**
	 * set the name of this rulebase
	 * 
	 * @param name -
	 *            name of rulebase
	 */
	public void setName(String name);

	/**
	 * returns whether or not this rulebase is enabled. None of the validation
	 * rules in a rulebase that is disabled will be processed during
	 * validations.
	 * 
	 * @return <code>true</code> if this rulebase is enabled,
	 *         <code>false</code> otherwise
	 */
	public boolean isEnabled();

	/**
	 * set whether or not this rulebase is enabled. Once a rulebase is enabled,
	 * individual validation rules of that rulebase can be enabled or disabled.
	 * However, if a rulebase is not enabled, none of the validation rules
	 * contained by that rjulebase will be executed.
	 * 
	 * @param enable -
	 *            set whether or not this rulebase is enabled
	 */
	public void setEnabled(boolean enable);

	/**
	 * returns the metadata for this rulebase (if any)
	 * 
	 * @return metadata for this rulebase or <code>null</code> if no metadata
	 *         is available
	 */
	public IRuleBaseMetadata getMetadata();

	/**
	 * set the metadata for this rulebase
	 * 
	 * @param metadata
	 */
	public void setMetadata(IRuleBaseMetadata metadata);

	/**
	 * retreave the rulesets contained within this rulebase. Rulesets are simply
	 * sets of validation rules in ACTF.
	 * 
	 * @return rulesets of this rulebase
	 * @see IRuleSet
	 */
	public IRuleSet[] getRuleSets();

	/**
	 * set the rulesets for this rulebase
	 * 
	 * @param rulesets
	 */
	public void setRuleSets(IRuleSet[] rulesets);

	/**
	 * add a ruleset to this rulebase
	 * 
	 * @param set
	 */
	public void addRuleSet(IRuleSet set);

	/**
	 * remove the ruleset with the given name
	 * 
	 * @param name
	 *            name of ruleset to be removed
	 */
	public void removeRuleSet(String name);

	/**
	 * retreave the model type associated with this rulebase
	 * 
	 * @return model type of this rulebase
	 * @see #setModelType(String)
	 */
	public String getModelType();

	/**
	 * set the model type associated with this rulebase. Each rulebase
	 * contains metadata and rulesets specific to a supported model.
	 * 
	 * @param type
	 *            type of model to be associated with this rulebase
	 */
	public void setModelType(String type);

	/**
	 * get the execution point descriptors of this rulebase
	 * 
	 * @return the array of ExecutionPointDescriptors for this IRuleBase
	 * @see #setExecutionPointDescriptors(ExecutionPointDescriptor[])
	 */
	public ExecutionPointDescriptor[] getExecutionPointDescriptors();

	/**
	 * set the execution point descriptors for this rulebase. Execution point
	 * descriptors inform the rulebase as to when and how validation rules are
	 * to be applied during execution of applications to be validated.
	 * 
	 * @param descriptors -
	 *            an array of ExecutionPointDescriptors for this IRuleBase
	 */
	public void setExecutionPointDescriptors(ExecutionPointDescriptor[] descriptors);

	/**
	 * add an execution point descriptor to this rulebase
	 * 
	 * @param descriptor
	 */
	public void addExecutionPointDescriptor(ExecutionPointDescriptor descriptor);

	/**
	 * remove the execution point descriptor with the given name from this
	 * rulebase
	 * 
	 * @param name
	 *            name of descriptor to be removed
	 */
	public void removeExecutionPointDescriptor(String name);

	/**
	 * get the execution point descriptor with the given name
	 * 
	 * @param name
	 *            name of desired descriptor
	 * @return
	 */
	public ExecutionPointDescriptor getExecutionPointDescriptor(String name);

	/**
	 * retrieves all RuleSets in the IRuleBase that are associated with the
	 * ExecutionPoint with the given name.
	 * 
	 * @param name -- the execution point name to match
	 */
	public IRuleSet[] getRuleSetsForExecutionPoint(String name);
	
}
