/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rulebase;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashSet;
import java.util.Set;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.doc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.doc.rulebase.ReflectionValidationDocument;
import org.eclipse.actf.validation.rules.RuleConstants;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * default implementation for the ACTF validation engine. A rulebase mapping
 * document is located in rulebases.xml files somewhere in the resources
 * classpath.
 * 
 * @see "resources/rulebases.xml"
 * @author John Buslawski
 * 
 */
public class RuleBaseMapDocument
{

	/** root element name for the rulebases.xml file; value is 'rulebases */
	public static final String RULEBASES = "rulebases";
	
	/** tag name for the element defining a rulebase; value is 'rulebase' */
	public static final String RULEBASE = "rulebase";
	
	/** tag name for element defining a target; value is 'target' */
	public static final String TARGET = "target";
	
	/** attribute defining the rulebase factory; value is 'rulebaseFactory' */
	public static final String RULEBASE_FACTORY = "rulebaseFactory";
	
	/** attribute defining the rulebase location for a document rulebase; value is 'location' */
	public static final String LOCATION = "location";
	
	/** attribute defining whether or not rulebases are to be applied polymorphically; value is 'polymorphic' */
	public static final String POLYMORPHIC = "polymorphic";
	
	/** attribute defining the regular expression for a target; value is 'regexp' */
	public static final String REGEXP = "regexp";

	private Element root;
	
	/**
	 * create a rulebase mapping document from the given XML source stream
	 * 
	 * @param xmlSource --
	 *            source stream for XML document
	 * @throws SAXException
	 *             in case of parsing error
	 * @throws IOException
	 *             in case of I/O error
	 * @throws ParserConfigurationException
	 */
	public RuleBaseMapDocument(InputStream xmlSource)
			throws ParserConfigurationException, SAXException, IOException {
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		factory.setNamespaceAware(true);
		factory.setIgnoringComments(true);
		factory.setCoalescing(true);
		
		DocumentBuilder builder = factory.newDocumentBuilder();
		Document document = builder.parse(xmlSource);
		root = document.getDocumentElement();
	} // ctor

	/** {@inheritDoc} */
	public String getDocumentName() {
		return root.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, IValidationDocument.DOCUMENT_NAME);
	}

	/** {@inheritDoc} */
	public String getDocumentVersion() {
		return root.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, IValidationDocument.DOCUMENT_VERSION);
	}

	/** {@inheritDoc} */
	public IRuleBaseDescriptor[] getRuleBaseDescriptors() {
		NodeList nodes = root.getElementsByTagNameNS(IValidationDocument.ACTF_NAMESPACE_URI, RULEBASE);
		ArrayList descriptors = new ArrayList();
		
		for (int i = 0; i < nodes.getLength(); ++i) {
			IRuleBaseDescriptor descriptor = parseRuleBaseDescriptor((Element) nodes.item(i));
			if (descriptor != null) {
				descriptors.add(descriptor);
			}
		}
		
		return (IRuleBaseDescriptor[]) descriptors.toArray(new IRuleBaseDescriptor[descriptors.size()]);
	}

	protected IRuleBaseDescriptor parseRuleBaseDescriptor (Element rbElem) {
		DocumentRuleBaseDescriptor rbDescriptor = new DocumentRuleBaseDescriptor();
		rbDescriptor.setRuleBaseName(rbElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, IValidationDocument.DOCUMENT_NAME));
		String sourceType = rbElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, "type");
		if (!sourceType.equalsIgnoreCase(DocumentRuleBaseDescriptor.DOCUMENT)) {
			Utils.println(IReporter.WARNING,"The <raven:type> element found in IRule Base mapping document is not of type - " + DocumentRuleBaseDescriptor.DOCUMENT);
		}
		rbDescriptor.setType(sourceType);
		rbDescriptor.setRuleBaseFactory(rbElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleBaseMapDocument.RULEBASE_FACTORY));
		rbDescriptor.setSourceLocation(rbElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleBaseMapDocument.LOCATION));
		
		Set targetSet = new HashSet();
		Element[] targetElems = ReflectionValidationDocument.getChildElements(rbElem, "target");
		for (int t = 0; t < targetElems.length; ++t) {
			Element targetElem = targetElems[t];
			String regExpStr = targetElem.getAttributeNS(IValidationDocument.ACTF_NAMESPACE_URI, RuleConstants.REGEXP);
			targetSet.add(regExpStr);
		}
		rbDescriptor.setTargets(targetSet);
		
		return rbDescriptor;
	}
	
} // RuleBaseMapDocument
