/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.doc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.rulebase.IDocumentRuleBaseMetadata;
import org.eclipse.actf.validation.rulebase.IRuleBaseMetadata;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * base implementation of a validation rule in ACTF. Clients should extend this
 * class rather than implementing <code>IRule</code>.
 * 
 * @author <a href="mailto:masquill@us.ibm.com">Mike Squillace</a>
 * 
 */
public abstract class AbstractRule implements IRule {

	private String id, name;
	private String iterName;
	private String msg, desc, hint;
	private int level = IReporter.UNKNOWN;
	private boolean ignoreifnull = false, enable = true;
	private String categoryId;
	private IRuleSet ruleSet;
	private IRule parentRule;
	private List ruleParts = new LinkedList();
	private HashMap attrMap = new HashMap();

	protected static final Map SEVERITY_MAP = new HashMap();
	static {
		SEVERITY_MAP.put("INFO", new Integer(IReporter.INFO));
		SEVERITY_MAP.put("WARNING", new Integer(IReporter.WARNING));
		SEVERITY_MAP.put("ERROR", new Integer(IReporter.ERROR));
		SEVERITY_MAP.put("SEVERE", new Integer(IReporter.SEVERE));
	}

	public AbstractRule() {
		name = toString();
	}

	/** {@inheritDoc} */
	public String getName() {
		return name;
	}

	/** {@inheritDoc} */
	public void setName(String name) {
		this.name = name;
	}

	/** {@inheritDoc} */
	public String getId () {
		return id;
	}

	/** {@inheritDoc} */
	public void setId (String id) {
		this.id = id;
	}

	protected String findRuleId () {
		if (id == null) {
			// Get the id in effect when this rule was
			// evaluated. The id may be defined in this
			// rule or somewhere up the hierarchy.
			IRule rule = this;
			String ruleId = null;
			while (ruleId == null && rule != null) {
				ruleId = rule.getId();
				rule = rule.getParent();
			}
			id = ruleId;
		}
		
		return id;
	}
	
	/** {@inheritDoc} */
	public int getSeverity() {
		return level;
	}

	/** {@inheritDoc} */
	public void setSeverity(int level) {
		this.level = level;
	}

	protected String findMessage() {
		if (msg == null) {
			// Get the message in effect when this rule was
			// evaluated. The message may be defined in this
			// rule or somewhere up the hierarchy.
			IRule rule = this;
			String message = null;
			while (message == null && rule != null) {
				message = rule.getMessage();
				rule = rule.getParent();
			}
			msg = message;
		}
		
		return msg;
	}

	/** {@inheritDoc} */
	public String getMessage() {
		return msg;
	}

	/** {@inheritDoc} */
	public void setMessage(String msg) {
		this.msg = msg;
	}

	/** {@inheritDoc} */
	public boolean getIgnoreIfNull() {
		return ignoreifnull;
	}

	/** {@inheritDoc} */
	public void setIgnoreIfNull(boolean ignoreifnull) {
		this.ignoreifnull = ignoreifnull;
	}

	protected boolean isIgnoringNullTarget (Object target) {
		return target == null && getIgnoreIfNull();
	}
	
	/** {@inheritDoc} */
	public boolean isEnabled() {
		return enable;
	}

	/** {@inheritDoc} */
	public void setEnabled(boolean enable) {
		this.enable = enable;
	}

	/** {@inheritDoc} */
	public Iterator getIterator (Object collection) {
		Iterator iter = null;
		if (iterName != null && iterName.equalsIgnoreCase("none")) {
			iter = new SingleItemIterator(collection);
		} else if (collection.getClass().isArray()) {
			iter = new ArrayIterator(collection);
		} else if (collection instanceof Collection) {
			iter = ((Collection) collection).iterator();
		} else if (collection instanceof NodeList) {
			iter = new DomNodeIterator(collection);
		} else if (iterName != null) {
			try {
				Class iterCls = Class.forName(iterName);
				iter = (Iterator) iterCls
					.getConstructor(new Class[] {Object.class}).newInstance(new Object[] {collection});
			} catch (Exception e) {
				iter = new SingleItemIterator(collection);
			}
		} else {
			iter = new SingleItemIterator(collection);
		}
		return iter;
	}

	/** {@inheritDoc} */
	public void setIteratorName (String clsName) {
		iterName = clsName;
	}

	/** {@inheritDoc} */
	public String getLongDescription() {
		return desc;
	}

	/** {@inheritDoc} */
	public void setLongDescription(String desc) {
		this.desc = desc;
	}

	/** {@inheritDoc} */
	public String getRepairHint() {
		return hint;
	}

	/** {@inheritDoc} */
	public void setRepairHint(String hint) {
		this.hint = hint;
	}

	protected void processActfAttributes(Element elem) {
		NamedNodeMap attrs = elem.getAttributes();
		for (int a = 0; a < attrs.getLength(); ++a) {
			Attr attr = (Attr) attrs.item(a);
			if (!attr.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI)) {
				continue;
			}
			
			String attrName = attr.getLocalName();
			if (attrName.equals(RuleConstants.MESSAGE)) {
				setMessage(getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.SEVERITY)) {
				Integer severity = (Integer) SEVERITY_MAP.get(attr.getValue());
				if (severity != null) {
					setSeverity(severity.intValue());
				} else {
					Utils.println(IReporter.USER, "Illegal value for severity: " + attr.getValue());
				}
			} else if (attrName.equals(RuleConstants.ENABLE)) {
				setEnabled(attr.getValue().equals("true"));
			} else if (attrName.equals(RuleConstants.POLYMORPHIC)) {
				setAttribute(RuleConstants.POLYMORPHIC, attr.getValue());
			} else if (attrName.equals(RuleConstants.IGNOREIFNULL)) {
				setIgnoreIfNull(attr.getValue().equals("true"));
			} else if (attrName.equals(RuleConstants.PARAM)) {
				setAttribute(RuleConstants.PARAM, getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.ID)) {
				setId(getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.NAME)) {
				setName(getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.DESCRIPTION)) {
				setLongDescription(getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.REPAIRHINT)) {
				setRepairHint(getAliasValue(attr.getValue()));
			} else if (attrName.equals(RuleConstants.ITERATOR)) {
				setIteratorName(attr.getValue());
			}
		}
		
		// inherit the message and severity from the parent if not specified
		IRule parent = getParent();
		if (parent != null) {
			if (getMessage() == null && parent.getMessage() != null) {
				setMessage(parent.getMessage());
			}
			if (getSeverity() == IReporter.UNKNOWN) {
				setSeverity(parent.getSeverity());
			}
		}
	}
	
	/**
	 * insures that either the target is not <code>null</code> or that, 
	 * if the target is <code>null</code>, null targets can be ignored 
	 * 
	 */
	public boolean apply(IValidationContext context, Object target) throws Exception {
		// -- presence test --
		// failure if target is null and null targets are not ignored
		boolean passed = true;
		if (target == null && !getIgnoreIfNull()) {
			String ruleMsg = findMessage();
			report(context, ruleMsg == null ? "Target is null" : ruleMsg + " [target is null]");
			passed = false;
		} else if (isIgnoringNullTarget(target)) {
			Utils.println(Utils.PROCESS_INFO, "Skipping rule " + getName() + " (ignoreifnull == true).");
		}
		return passed;
	}

	/** {@inheritDoc} */
	public List getRuleParts() {
		return ruleParts;
	}

	/** {@inheritDoc} */
	public void setRuleParts(List ruleParts) {
		this.ruleParts = ruleParts;
	}

	/** {@inheritDoc} */
	public void addRulePart(IRule rule) {
		if (rule != null) {
			if (rule.getSeverity() == IReporter.UNKNOWN) {
				rule.setSeverity(getSeverity());
			}
			ruleParts.add(rule);
		}
	}

	protected String findComponentCategory() {
		if (this.categoryId == null) {
			// Get the category id in effect when this rule was
			// evaluated. The categoryId may be defined in this
			// rule or somewhere up the hierarchy.
			IRule rule = this;
			String categoryId = null;
			while (categoryId == null && rule != null) {
				categoryId = rule.getComponentCategory();
				rule = rule.getParent();
			}
			this.categoryId = categoryId;
		}
		
		return this.categoryId;
	}

	/** {@inheritDoc} */
	public String getComponentCategory() {
		return categoryId;
	}

	/** {@inheritDoc} */
	public void setComponentCategory(String categoryId) {
		this.categoryId = categoryId;
	}

	/** {@inheritDoc} */
	public IRule getParent() {
		return parentRule;
	}

	public void setParent(IRule parent) {
		parentRule = parent;
	}

	/** {@inheritDoc} */
	public IRuleSet getRuleSet() {
		return ruleSet;
	}

	public void setRuleSet(IRuleSet iRuleSet) {
		ruleSet = iRuleSet;
	}

	public String toString () {
		StringBuffer sb = new StringBuffer();
		sb.append(name == null ? getClass().getName() : name);
		if (id != null) {
			sb.append('[');
			sb.append(id);
			sb.append(']');
		}
		sb.append(':');
		sb.append(msg);
		return sb.toString();
	}

	protected void report (IValidationContext context, String msg) {
		IReporter reporter = (IReporter) context.getParameter(ValidationContextConstants.REPORTER);
		INodeLocator locator = (INodeLocator) context.getParameter(ValidationContextConstants.ELEMENT_LOCATOR);
		Object target = context.getParameter(ValidationContextConstants.CURRENT_TARGET);
		
		if (reporter != null) {
			String oldId = reporter.getSourceID();
			String ruleId = findRuleId();
			if (ruleId != null) {
				reporter.setSourceID(ruleId);
			}
			
			String path = null;
			Object[] values = null;
			if (locator != null) {
				path = locator.locate(target, null);
				if (path != null) {
					values = new Object[] {path};
				}
			}
			reporter.report(getSeverity(), msg, values);
			if (ruleId != null) {
				reporter.setSourceID(oldId);
			}
		}
	}
	
	protected String getAliasValue(String name) {
		String aliasValue = name;
		if (ruleSet != null) {
			IRuleBaseMetadata metaData = ruleSet.getRuleBase().getMetadata();
			if (metaData instanceof IDocumentRuleBaseMetadata) {
				if (name.startsWith("%")) {
					aliasValue = ((IDocumentRuleBaseMetadata) metaData).getAliasValue(name.substring(1));
				}
			}
		}
		return aliasValue;
	}

	protected IRuleFactory getRuleFactory() {
		return ruleSet.getRuleFactory();
	}
	
	protected String getTextTrim (Element elem) {
		String textContent = "";
		NodeList nodes = elem.getChildNodes();
		for (int n = 0; textContent.length() == 0 && n < nodes.getLength(); ++n) {
			Node node = nodes.item(n);
			if (node.getNodeType() == Node.TEXT_NODE) {
				textContent = node.getNodeValue();
			}
		}
		
		return textContent.trim();
	}
	
	public String getAttribute (String name) {
		return (String) attrMap.get(name);
	}
	
	public void setAttribute(String name, String value) {
		attrMap.put(name, value);
	}
	
} // AbstractRule
