/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import java.util.Iterator;
import java.util.List;

import org.eclipse.actf.validation.validators.IValidationContext;


/**
 * an interface to be implemented to add rules to the validation engine.
 * 
 * @author <a href="mailto:masquill@us.ibm.com">Mike Squillace</a>
 *
 */
public interface IRule {

	/**
	 * return the name of the rule. This might also serve as a short
	 * description of the purpose of the rule or what it is testing.
	 * 
	 * @return name of rule
	 */
	public String getName();

	/**
	 * set the name for this rule. The name need not be unique but should be a short
	 * description of the rule or of what it is testing.
	 * 
	 * @param name
	 */
	public void setName(String name);
	
	/**
	 * get the unique id for this rule
	 * 
	 * @return
	 */
	public String getId ();
	
	/**
	 * set the id for this rule. The id for a rule should be unique within a given rulebase.
	 * 
	 * @param id
	 */
	public void setId (String id);
	
	/**
	 * returns the enable attribute for this rule
	 * 
	 * @return whether this rule is enabled (true | false )
	 */
	public boolean isEnabled();

	/**
	 * 
	 * set the enable attribute for this rule. If enable is true then this rule
	 * is evaluated. If enable is false then this rule is skipped.
	 * 
	 * @param enable -
	 *            true | false
	 */
	public void setEnabled(boolean enable);

	/**
	 * returns the message resulting from violating this rule
	 * 
	 * @return message to be received when rule is violated
	 */
	public String getMessage();

	/**
	 * set the message to be received upon a violation of this rule. This
	 * message is typically short and can be supplemented by the long
	 * description.
	 * 
	 * @param msg -
	 *            message to be received upon a violation of this rule
	 * @see #setLongDescription(String)
	 */
	public void setMessage(String msg);

	/**
	 * returns the long description received upon a violation of this rule
	 * 
	 * @return long description or message received upon a violation of this
	 *         rule
	 */
	public String getLongDescription();

	/**
	 * set the long description or message to be received upon a violation of
	 * this rule. The long description can supplement the shorter message for
	 * this rule.
	 * 
	 * @param desc -
	 *            long description to be received upon a violation of this rule
	 * @see #setMessage(String)
	 */
	public void setLongDescription(String desc);

	/**
	 * returns the repair hint for this rule
	 * 
	 * @return repair hint
	 */
	public String getRepairHint();

	/**
	 * set the repair hint for this rule. The repair hint provides information
	 * about how to fix violations of this rule.
	 * 
	 * @param hint
	 */
	public void setRepairHint(String hint);

	/**
	 * return the severity level for a violation of this rule.
	 * 
	 * @return severity level
	 */
	public int getSeverity();

	/**
	 * set the severity level for a violation of this rule. The level will
	 * typically be one of the levels specified in
	 * <code>org.eclipse.actf.util.logging.Reporter</code>.
	 * 
	 * @param level
	 * @see org.eclipse.actf.util.logging.IReporter
	 */
	public void setSeverity(int level);

	/**
	 * initialize this rule with any content that might be necessary for its 
	 * application to a target during the invocation of the <code>apply</code> method.
	 * 
	 * <p>
	 * The specified content may take any form. It may be an XML element, a
	 * .properties file, a string, etc. This method may be invoked anytime prior to calling 
	 * <code>apply</code>.
	 * 
	 * @param content - initialization content for the rule
	 * @see #apply(IValidationContext, Object)
	 * @throws Exception
	 */
	public void initialize(Object content) throws Exception;

	/**
	 * apply this rule to the given target using the given 
	 * context. The returned value indicates whether or not a violation of
	 * some kind has taken place.
	 * 
	 * <p>Note that the context object can be used to retreave a great deal of information about the current validation 
	 * process. In particular, it can be used to update the validation report being generated with 
	 * the violations that result from an application of this rule to the given target.
	 *
	 * @param context context under which rule is being evaluated (supplied by engine)
	 * @param target object to which rule is to be applied
	 * @return whether or not the target violates the rule
	 * @see #initialize(Object)
	 * @see org.eclipse.actf.validation.validators.IValidationContext
	 * @see org.eclipse.actf.validation.validators.ValidationContextConstants
	 * @throws Exception
	 */
	public boolean apply(IValidationContext context, Object target) throws Exception;

	/**
	 * returns the rule parts or nested rules for this rule
	 * 
	 * @return list of rule parts
	 */
	public List getRuleParts();

	/**
	 * set the rule parts or nested rules for this rule.
	 * 
	 * @param ruleParts
	 *            list of rule parts
	 */
	public void setRuleParts(List ruleParts);

	/**
	 * adds a rule to the list of rule parts or nested rules that are executed
	 * when this rule is evaluated.
	 * 
	 * @param rule
	 *            validation rule
	 */
	public void addRulePart(IRule rule);

	/**
	 * return the parent rule of this rule.
	 * 
	 * @return instance of the parent rule
	 */
	public IRule getParent();

	/**
	 * set the parent rule of this rule. Rules can be nested. There are some
	 * fields that are set only on the parent rule for all nested rule parts.
	 * 
	 * @param parent
	 *            rule
	 */
	public void setParent(IRule parent);

	/**
	 * return the IRuleSet containing this IRule.
	 * 
	 * @return instance of the containing IRuleSet
	 */
	public IRuleSet getRuleSet();

	/**
	 * Set the containing IRuleSet for this rule. A IRule is contained by only one
	 * IRuleSet.
	 * 
	 * @param iRuleSet
	 *            parent rule
	 */
	public void setRuleSet(IRuleSet iRuleSet);

	/**
	 * return the unique component category identifier in effect at the time
	 * this rule is evaluated. The category identifier is used in validation
	 * reports.
	 * 
	 * @return category identifier of component
	 */
	public String getComponentCategory();

	/**
	 * returns the actf:ignoreifnull attribute for this rule
	 * 
	 * @return whether apply(Object) returns true or false when this rule is
	 *         evaluated with a null object
	 */
	public boolean getIgnoreIfNull();

	/**
	 * returns the iterator used for this rule should the rule involve any sort of collection 
	 * or array. Note that a <code>ArrayIterator</code> is provided for 
	 * arrays and a <code>DomNodeIterator</code> is provided for iterating through the 
	 * nodes of a W3C DOM level 2 <code>NodeList</code>.
	 * 
	 * <p>Other iterators may be specified using the actf:iterator attribute.
	 *
	 * @param collection with which to initialize iterator
	 * @return iterator used for collections or <code>null</code> if no 
	 * iterator is required
	 */
	public Iterator getIterator (Object collection);

	/**
	 * set the component category identifier for this rule. The parent rule, the
	 * first child of a first child of a <actf:rulesets> element, will
	 * typically contain this value since all rules are evaluated for this
	 * component. This value is used when creating a report record and provides
	 * a unique identifier for this particular component.
	 * 
	 * @param categoryId
	 */
	public void setComponentCategory(String categoryId);

	/**
	 * 
	 * set the actf:ignoreifnull attribute for this rule. If ignoreifnull is true
	 * then this rule is ignored if the target object to evaluate in the
	 * apply(Object) method is null. If ignoreifnull is false then apply(Object)
	 * will fail if the target object is null.
	 * 
	 * @param ignoreIfNull -
	 *            true | false
	 * @see #apply(Object)
	 */
	public void setIgnoreIfNull(boolean ignoreIfNull);

	/**
	 * set the class name of the iterator to be  used for iterating through any sort of 
	 * collection. This attribute is set using the <code>RuleConstants.ITERATOR</code> attribute.
	 *
	 * @param clsName - class name of iterator to be used
	 * @see #getIterator(Object)Object)
	 * @see RuleConstants#ITERATOR
	 */
	public void setIteratorName (String clsName);
	
	/**
	 * retreaves the attribute with the given name for this rule
	 * 
	 * @param name name of attribute
	 * @return attribute with given name or <code>null</code>
	 */
	public String getAttribute (String name);
	
	/**
	 * set an arbitrary attribute for this rule
	 * 
	 * @param name name of attribute
	 * @param value value of attribute
	 */
	public void setAttribute (String name, String value);
	
}