/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.validation.rulebase.IRuleBase;

/**
 * A factory for rules. A factory instance, once obtained, can be used to find
 * rules for a given type of object using a <code>IRuleMapping</code>. Implementations should be specified using the &lt;actf:ruleFactory&gt; 
 * attribute in documents or programatically. In either case, rule factories should always be atached to a <code>IRuleSet</code>.
 *
 * @see IRuleSet#setRuleFactory(IRuleFactory)
 * @see IRuleMapping
 * @author <a href="mailto:masquill@us.ibm.com">Mike Squillace</a>
 */
public interface IRuleFactory {

	/**
	 * set the rulebase for this factory
	 * 
	 *
	 */
	public void setRuleBase (IRuleBase ruleBase);
	
	/**
	 * 
	 * @return rulebase
	 */
	public IRuleBase getRuleBase ();
	
	/**
	 * set the ruleset for this factory
	 * 
	 *
	 */
	public void setRuleSet (IRuleSet ruleSet);
	
	/**
	 * 
	 * @return ruleset
	 */
	public IRuleSet getRuleSet ();
	
	/**
	 * registers a validation rule class for the given type
	 * 
	 * @param name -
	 *            name under which the rule is being registered
	 * @param ruleClassName -
	 *            the class name of the rule implementation
	 */
	public void registerRuleClassName(String name, String ruleClassName);

	/**
	 * gets an instance of the validation rule for the given name. The rule must
	 * have been previously registered via the
	 * <code>registerRuleClassName</code> method or must be one of the
	 * standard ACTF validation rules.
	 * 
	 * @param name -
	 *            name of the rule
	 * @return the registered rule for the given name or <code>null</code> if there was no
	 *         rule registered under that name
	 *         @see #registerRuleClassName(String, String)
	 */
	public IRule getRuleInstance(String name);

	/**
	 * creates and initializes a validation rule for the given name. The rule
	 * must have been previously registered via the
	 * <code>registerRuleClassName</code> method or must be one of the
	 * standard ACTF validation rules.
	 * 
	 * @param name -
	 *            name of the rule
	 * @param parent -
	 *            parent rule instance
	 * 
	 * @return the registered rule for the given name or <code>null</code> if there was no
	 *         rule registered under that name
	 *         @see #registerRuleClassName(String, String)
	 */
	public IRule getRuleInstance(String name, IRule parent);

	/**
	 * Makes an instance of the rule using the object.
	 * 
	 * @param obj -
	 *            Object from which the rule is derived
	 * @return the rule based on the input object
	 */
	public IRule makeRuleInstance(Object obj);

	/**
	 * Makes an instance of the rule using the object having the specified parent.
	 * 
	 * @param obj -
	 *            Object from which the rule is derived
	 * @param parent -
	 *            parent rule instance
	 * 
	 * @return the rule based on the input object
	 */
	public IRule makeRuleInstance(Object obj, IRule parent);
	
	/**
	 * retreaves a rule mapping that maps a type of object to a set of rules. The mapping is created 
	 * lazily based on the token representing the ruleset with which this factory is associated.
	 * 
	 * @param rulesetToken - ruleset (or token of ruleset) associated with this factory
	 * @return rule mapping
	 */
	public IRuleMapping getRuleMapping (Object rulesetToken);
	
}
