/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import org.eclipse.actf.validation.rulebase.IRuleBase;

/**
 * captures a set of rules in ACTF. A ruleset is a collection of rules in a
 * rulebase. Each ruleset can be enabled or disabled and, if enabled, each rule
 * within the ruleset can be enabled or disabled. Rulesets are also associated
 * with execution points so that collections of rules can be applied at
 * different points throughout a process.
 * 
 * @see org.eclipse.actf.validation.core.ExecutionPoint
 * @see IRuleBase
 * @author John Buslawski
 * 
 */
public interface IRuleSet {

	/**
	 * get the name of this ruleset
	 * 
	 * @return name of ruleset
	 */
	public String getName();

	/**
	 * set the name of this ruleset
	 * 
	 * @param name
	 */
	public void setName(String name);

	/**
	 * return whether or not this ruleset is enabled
	 * 
	 * @return <code>true</code> if this ruleset is enabled,
	 *         <code>false</code> otherwise
	 * @see #setEnabled(boolean)
	 */
	public boolean isEnabled();

	/**
	 * set whether or not this ruleset is enabled. If a ruleset is enabled, each
	 * of the validation rules within the set can be enabled or disabled.
	 * Disabling this ruleset disables all validation rules within the set.
	 * 
	 * @param enabled
	 */
	public void setEnabled(boolean enabled);

	/**
	 * add a rule to this ruleset
	 * 
	 * @param rule
	 */
	public void addRule(IRule rule);

	/**
	 * remove the rule with the given name
	 * 
	 * @param name
	 *            name of rule to be removed
	 */
	public void removeRule(String name);

	/**
	 * return all of the rules within this ruleset
	 * 
	 * @return all rules within this ruleset
	 */
	public IRule[] getRules();

	/**
	 * get the rule with the given name
	 * 
	 * @param name
	 *            name of desired rule
	 * @return rule with the given name or <code>null</code> if no such rule
	 *         exists in this ruleset
	 */
	public IRule getRule(String name);

	/**
	 * get the rule factory responsible for creating rules in this ruleset
	 * 
	 * @return rule factory
	 */
	public IRuleFactory getRuleFactory();

	/**
	 * set the rule factory for this ruleset. The rule factory set should
	 * generate all rules in this ruleset.
	 * 
	 * @param factory
	 */
	public void setRuleFactory(IRuleFactory factory);

	/**
	 * get the name of the execution point descriptor that triggers rules in
	 * this ruleset
	 * 
	 * @return name of execution point that triggered this ruleset
	 */
	public String getExecutionPointName ();

	/**
	 * set the name of the execution point that triggered this ruleset \
	 * 
	 * @param name
	 *            name of executionPoint
	 */
	public void setExecutionPoint(String name);

	/**
	 * get the rulebase to which this ruleset belongs
	 * 
	 * @return rulebase to which this ruleset belongs
	 */
	public IRuleBase getRuleBase();

	/**
	 * set the rulebase to which this ruleset belongs
	 * 
	 * @param iRuleBase
	 */
	public void setRuleBase(IRuleBase iRuleBase);
}
