/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  John Buslawski - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.rules;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.actf.core.adapt.DefaultAdaptorFactory;
import org.eclipse.actf.core.adapt.IAdaptor;
import org.eclipse.actf.core.adapt.IAdaptorFactory;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.model.IGuiModel;
import org.eclipse.actf.model.IModel;
import org.eclipse.actf.model.InvalidComponentException;
import org.eclipse.actf.model.locate.AbstractNodeLocator;
import org.eclipse.actf.model.locate.INodeLocator;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.AbstractReporter;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.core.Parameter;
import org.eclipse.actf.validation.core.Reflector;
import org.eclipse.actf.validation.doc.codeproc.ICodeProcessor;
import org.eclipse.actf.validation.doc.rulebase.IValidationDocument;
import org.eclipse.actf.validation.doc.rulebase.ReflectionDocumentRuleFactory;
import org.eclipse.actf.validation.validators.IValidationContext;
import org.eclipse.actf.validation.validators.ValidationContextConstants;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

/**
 * @author John Buslawski
 * 
 */
public class ReflexiveRule extends AbstractRule {

	private Reflector reflector = new Reflector();
	private String[] methodNames = null;
	private Class[] srcClasses = null;
	private Object[] srcObjects = null;

	
	/** {@inheritDoc} */
	public void initialize(Object content) throws Exception {
		if (content instanceof Element) {
			Element elem = (Element) content;
			processActfAttributes(elem);
			buildReflectionParameters(elem);
			processRules(elem);
			processScriptText(elem);
		}
	}

	/** {@inheritDoc} */
	public boolean apply(IValidationContext context, Object target) throws Exception {
		// execute the presence test
		boolean result = super.apply(context, target);
		// if not enabled, presence test failed,
		// or ignoring null target, then return
		if (isEnabled() && result && !isIgnoringNullTarget(target)) {
			buildReflectionTargets(context, target);
			result = invokeMethod(context, target);
		}
		
		return result;
	}

	private void buildReflectionParameters(Element elem) {
		String propAlias = getAliasValue(elem.getLocalName());
		methodNames = new String[] { propAlias, "get" + Utils.firstCharToUpper(propAlias) };
	}

	/**
	 * 
	 * Use the IAdaptorFactory for this model to find other candidate
	 * classes for this component when using reflection to find a method. This
	 * technique uses the Eclipse IAdaptable pattern.
	 * 
	 * @param component
	 *            to evaluate
	 */
	private void buildReflectionTargets(IValidationContext context, Object component) {
		// The default list of classes and objects contain the component
		// itself at a minimum.
		srcClasses = new Class[] { component.getClass() };
		srcObjects = new Object[] { component };
		
		// Find the IAdaptorFactory for this model and instantiate it.

		IAdaptorFactory adaptorFactory = DefaultAdaptorFactory.getInstance();
		
		if (adaptorFactory != null) {
			// Create some temporary collections and seed them with the
			// target class types and target object.
			List classList = new ArrayList();
			classList.add(component.getClass());
			List objectList = new ArrayList();
			objectList.add(component);
			// get the possible Adaptors for this type
			IAdaptor[] adaptors = adaptorFactory.getAllAdaptors(component.getClass());
			Class[] classTypes = null;
			if (adaptors.length > 0) {
				for (int i = 0; i < adaptors.length; i++) {
					classTypes = adaptors[i].getSupportedTypes();
					// Iterate through the supported class types and try to
					// adapt the target
					// to the class type. If it can be adapted then put it on
					// the list of
					// reflection candidates.
					for (int j = 0; j < classTypes.length; j++) {
						Object adaptedObject = null;
						try {
							adaptedObject = adaptors[i].adapt(component, classTypes[j]);
						} catch (Exception e) {
							Utils.println(IReporter.SYSTEM_NONFATAL,
									"Could not adapt " + component + " to type " + classTypes[j].getName(), e);
						}
						
						if (adaptedObject != null) {
							Utils
									.println(
											Utils.PROCESS_ALL,
											"ReflexiveRule.buildReflectionTargets(Object), adding class type: "
													+ classTypes[j].getName()
													+ " to list of reflection targets.");
							classList.add(classTypes[j]);
							objectList.add(adaptedObject);
						}
					}
				}
			}
			srcClasses = (Class[]) classList.toArray(new Class[classList
					.size()]);
			srcObjects = objectList.toArray(new Object[objectList.size()]);
		}
	}

	private void processRules(Element elem) {
		NamedNodeMap attrs = elem.getAttributes();
		Map attrMap = new HashMap();
		for (int a = 0; a < attrs.getLength(); ++a) {
			Attr attr = (Attr) attrs.item(a);
			if (attr.getNamespaceURI().equals(IValidationDocument.ACTF_NAMESPACE_URI)) {
				attrMap.put(attr.getLocalName(), attr.getValue());
			}
		}
		
		if (!attrMap.isEmpty()) {
			Iterator iter = attrMap.keySet().iterator();
			while (iter.hasNext()) {
				String attrKey = (String) iter.next();
				// if the attribute is not a rule then go to the next one
				ReflectionDocumentRuleFactory ruleFactory = (ReflectionDocumentRuleFactory) getRuleFactory();
				if (ruleFactory.isRuleAttribute(attrKey)) {
					continue;
				}
				
				IRule validRule = ruleFactory.getRuleInstance(attrKey);
				if (validRule != null) {
					Utils.println(Utils.PROCESS_ALL,
							"ReflexiveRule.processRules(Element), IRule element name - "
									+ elem.getTagName() + ", found rule name - " + attrKey);
					try {
						validRule.setRuleSet(getRuleSet());
						validRule.initialize(elem);
						validRule.setParent(this);
						validRule.setName(attrKey);
						addRulePart(validRule);
					} catch (Exception e) {
						Utils.println(IReporter.SYSTEM_NONFATAL,
								"ReflexiveRule.processRules(Element), unable to initialize rule: " + validRule.getName(), e);
					}
				} else {
					Utils.println(IReporter.SYSTEM_NONFATAL,
									"ReflexiveRule.processRules(Element), IRuleFactory unable to create a rule instance from attribute " + attrKey + "] for IRule element name " + elem.getTagName());
				}
			}
		}
	}

	private void processScriptText(Element elem) {
		if (elem != null || (elem == null && !getIgnoreIfNull())) {
			IRule scriptRule = null;
			String textContent = getTextTrim(elem);
			if (textContent.length() > 0) {
				try {
					IRuleFactory ruleFactory = getRuleFactory();
					scriptRule = ruleFactory.getRuleInstance(RuleConstants.SCRIPT);
					scriptRule.setRuleSet(getRuleSet());
					scriptRule.initialize(elem);
					scriptRule.setParent(this);
					scriptRule.setName(RuleConstants.SCRIPT);
					addRulePart(scriptRule);
				} catch (Exception e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"ReflexiveRule.processScriptText(Element), unable to initialize rule: "
									+ scriptRule.getName(), e);
				}
			}
		}
	}

	private boolean invokeMethod(IValidationContext context, Object component) throws InvalidComponentException {
		final Reflector.Result[] methResult = new Reflector.Result[1];
		final Throwable[] throwable = new Throwable[1];
		final Object[] result = new Object[1];
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		ICodeProcessor codeProcessor = (ICodeProcessor) context.getParameter(ValidationContextConstants.CODE_PROCESSOR);
		Parameter[] params = null;
		String param = getAttribute(RuleConstants.PARAM);
		Object propValue = null;
		int mn = 0; // index for method names array
		int index = 0;
		boolean passed = true;
		
		// algorithm for finding getter method to verify property:
		// call method with name of element on control
		// call method "get" + firstCharToUpper(name of element) on control
		// Note: provision for one param given as 'actf:param' in xml doc
		
		CLASS_LOOP: for (int cl = 0; cl < srcClasses.length; ++cl) {
			for (mn = 0; mn < methodNames.length; ++mn) {
				try {
					if (param != null) {
						params = new Parameter[] {new Parameter(param, codeProcessor) };
					} else {
						params = new Parameter[0];
					}
					
					methResult[0] = reflector.findMethod(
							srcObjects[cl], methodNames[mn], params);
					if (methResult[0] != null) {
						final int clx = cl;
						Runnable runnable = new Runnable() {
							public void run() {
								try {
									result[0] = methResult[0].invoke(srcObjects[clx]);
								} catch (Throwable e) {
									throwable[0] = e;
								}
							}
						};
						
						if (model instanceof IGuiModel) {
							((IGuiModel) model).invokeOnUIThread(runnable);
						} else {
							runnable.run();
						}
						
						if (throwable[0] != null) {
							Throwable t = throwable[0];
							if (t instanceof InvalidComponentException) {
								throw (InvalidComponentException) t;
							} else {
								throw new RuntimeException(t.getMessage());
							}
						}
						propValue = result[0];
						Utils.println(Utils.ALL,
								"Successful invokation of method" + methResult[0].getMember() + " returned value: " + propValue);
						
						if (propValue != null) {
							Utils.println(Utils.ALL,
									"Breaking with value " + propValue + " for property " + getName());
							break CLASS_LOOP;
						} else {
							Utils.println(Utils.PROCESS_ALL,
									"No value returned for property/method " + getName() + (param != null ? " and parameter " + param : ""));
							break CLASS_LOOP;
						}
					} else {
						// attempt to treat property as referring to a public field
						propValue = reflector.getFieldValue(methodNames[mn],
								srcObjects[cl]);
						if (propValue != null) {
							Utils.println(Utils.ALL,
									"Breaking with field value " + propValue);
							break CLASS_LOOP;
						}
					}
					index = mn;
				} catch (InvalidComponentException e) {
					throw e;
				} catch (Exception e) {
					Utils.println(IReporter.SYSTEM_NONFATAL,
							"Exception caught while attempting to locate - "
									+ srcObjects[cl] + "." + methodNames[mn], e);
					continue;
				}
			}
		}
		
		// now that we have the result of the getter, we are ready to
		// validate the property
		if (propValue != null) {
			// determine the iterator to use if propValue is any form of collection
			Iterator propIter = getIterator(propValue);
			INodeLocator locator = (INodeLocator) context.getParameter(ValidationContextConstants.ELEMENT_LOCATOR);
			while (propIter.hasNext()) {
				Object thisProp = propIter.next();
				codeProcessor.setLocal("propertyValue", thisProp, "true");
				for (Iterator iter = getRuleParts().iterator(); iter.hasNext(); ) {
					IRule rule = (IRule) iter.next();
					if (!(propIter instanceof SingleItemIterator)) {
						setRuleProperties(context, thisProp, rule, locator);
					}
					try {
						passed = rule.apply(context, thisProp);
					} catch (Exception e) {
						Utils.println(IReporter.SYSTEM_NONFATAL, "Exception caught while evaluating results from method call ", e);
					}
				}
			} // next propValue
		} else if (propValue == null && !getIgnoreIfNull()) {
			// not leaf but propValue == null
			String message = findMessage();
			if (message == null || message.length() == 0) {
				StringBuffer msgBuff = new StringBuffer();
				msgBuff.append("No value returned for thisComponent.");
				msgBuff.append(methodNames[index]);
				msgBuff.append(param != null ? " with parameter " + param : "");
				message = msgBuff.toString();
			}
			Utils.println(Utils.ALL,
					"ReflexiveRule.invokeMethod(), adding report record, message - " + message + ", severity - " + getSeverity());

			report(context, message);
			passed = false;
} else {
			// propValue == null && getIgnoreIfNull()
			// do nothing
		}
		
		return passed;
	}
	
	private void setRuleProperties (IValidationContext context, Object component, IRule rule, INodeLocator locator) {
		IModel model = (IModel) context.getParameter(ValidationContextConstants.MODEL);
		IReporter reporter = (IReporter) context.getParameter(ValidationContextConstants.REPORTER);
		if (model != null && locator != null) {
			String nodeName = model.getNodeName(component);
			String nodeId = model.getNodeId(component);
			String desc = AbstractNodeLocator.descriptionAsString(locator.describe(component));
			rule.setComponentCategory(
				AbstractReporter.CATEGORY_FORMATTER.format(new Object[] {nodeName, desc, nodeId}));
			rule.setParent(this);
			rule.setSeverity(getSeverity());
		}
	}

}
