/*******************************************************************************
* Copyright (c) 2004, 2007 IBM Corporation.
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
* Contributors:
*  Mike Squillace - initial API and implementation
*******************************************************************************/ 

package org.eclipse.actf.validation.ui.jobs;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.actf.core.config.ConfigurationException;
import org.eclipse.actf.core.config.IConfiguration;
import org.eclipse.actf.core.runtime.IRuntimeContext;
import org.eclipse.actf.core.runtime.RuntimeContextFactory;
import org.eclipse.actf.ui.preferences.RootPreferencePage;
import org.eclipse.actf.util.Utils;
import org.eclipse.actf.util.logging.IReporter;
import org.eclipse.actf.validation.core.ExecutionPoint;
import org.eclipse.actf.validation.core.ValidationProcessConstants;
import org.eclipse.actf.validation.core.ValidationProcessException;
import org.eclipse.actf.validation.core.ValidationProcessManager;
import org.eclipse.actf.validation.events.IReportModifyListener;
import org.eclipse.actf.validation.events.ReportModifyEvent;
import org.eclipse.actf.validation.reports.CreationContext;
import org.eclipse.actf.validation.reports.ValidationReport;
import org.eclipse.actf.validation.reports.ValidationReportRecord;
import org.eclipse.actf.validation.ui.ValidationUIConstants;
import org.eclipse.actf.validation.ui.ValidationUiPlugin;
import org.eclipse.actf.validation.ui.preferences.ValidationPreferenceInitializer;
import org.eclipse.actf.validation.ui.views.AbstractViewPart;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PlatformUI;

public class ValidationJob extends Job
{

	public static final QualifiedName NEW_ACTION_QNAME = new QualifiedName(ValidationUiPlugin.getDefault().getPluginId(), "newAction");
	public static final QualifiedName BUNDLE_RESOURCE_QNAME = new QualifiedName(ValidationUiPlugin.getDefault().getPluginId(), "bundleResource");

	protected static ValidationUiPlugin plugin = ValidationUiPlugin.getDefault();
	
	protected Map parentMap = new HashMap();
	protected IResource resource;
	protected String resourceName;
	protected ValidationReport currentReport;
	protected Map contextMap = new HashMap();
	protected IValidatableGuiContainer part;

	private IReportModifyListener rmListener = new IReportModifyListener() {
		public void reportModified (ReportModifyEvent e) {
			if (e.addedElements instanceof CreationContext[]) {
				for (int i = 0; i < e.addedElements.length; ++i) {
					CreationContext context = (CreationContext) e.addedElements[i];
					plugin.trace(IReporter.CONFIG,
						"reportModify(ReportModifyEvent): Caching " + context);
					contextMap.put(context.getComponentID(), context);
				}
			}
		}
	};

	public ValidationJob (String name) {
		super(name);
		setUser(true);
		setPriority(Job.BUILD);
		setProperty(NEW_ACTION_QNAME, Boolean.TRUE);
	}

	public ValidationReport getCurrentReport () {
		return currentReport;
	}

	public IResource getResource () {
		return resource;
	}

	public void setResource (IResource resource) {
		this.resource = resource;
	}

	public void setSourcePart (IValidatableGuiContainer part) {
		this.part = part;
	}

	protected IStatus run (IProgressMonitor monitor) {
		IStatus status = Status.OK_STATUS;
		if (part != null) {
			try {
				if (((Boolean) getProperty(NEW_ACTION_QNAME)).booleanValue()) {
					configureReportInstance();
				}
				PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {

					public void run () {
						validate();
					}
				});
			}catch (Throwable e) {
				plugin.logException(e);
				status = new Status(IStatus.ERROR, plugin.getPluginId(), IStatus.ERROR, "Error running validation", e);
			}
		}
		return status;
	}

	protected void validate () {
		double startTime = System.currentTimeMillis(), elapsedTime = 0.0;
		IWorkbenchPart iwPart = (IWorkbenchPart) part;
		iwPart.setFocus();
		AbstractViewPart avp = (AbstractViewPart) AbstractViewPart.findView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
		IPreferenceStore store = plugin.getPreferenceStore();
		int traceLevel = RootPreferencePage.mapToLevel(store.getString(ValidationPreferenceInitializer.P_VALIDATION_LEVEL));
		IConfiguration config = null;
		IRuntimeContext context = RuntimeContextFactory.getInstance().getRuntimeContext();
		
		try {
			config = context.getConfiguration();
		}catch (ConfigurationException e) {
			//Could not obtain a Configuration. We can't do much here but quit.
			throw new RuntimeException(e.getMessage(), e);
		}
		
		String sourceName = "<unknown>"; //fallback 
		if (resource != null) { //for where a resource is associated with the validation
			sourceName = resource.getName();
		}else if (resourceName != null) { //for where there is no resource (Inspection View or URL) 
			sourceName = resourceName;
		}
		
		ValidationProcessManager vpm = ValidationProcessManager.getInstance();
		ExecutionPoint eventExPt = vpm.createMethodExecutionPointForTarget(
			ValidationProcessConstants.VISIBILITY, part.getValidatable(), null, null);
		try {
			vpm.initiateValidation(eventExPt, parentMap);
		}catch (ValidationProcessException vpe) {
			plugin.logException(vpe);
		}
		
		final ValidationReportRecord[] records = (vpm.getReporter()).getRecords();
		PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
			public void run () {
				try {
					currentReport.addRecords(records);
				}catch (RuntimeException e) {
					plugin.logException(e);
				}
			}
		});
		
		try {
			updateMarkers(records);
		}catch (Exception e) {
			plugin.logException(e);
		}
		elapsedTime = (System.currentTimeMillis() - startTime) / 1000.0;
		Utils.println(
			Utils.PROCESS_ALL, "Validation finished, elapsed time - "
					+ elapsedTime);
		String model = "<unknown>"; //TODO find the GuiArch and set it here or back when the job was created
		String tool = getName();
		if (avp == null) {
			avp = (AbstractViewPart) AbstractViewPart.activateView(ValidationUIConstants.REPORT_TREE_VIEW_ID);
		}
		avp.createReport(
			currentReport, "Report on " + sourceName, sourceName, tool, model);
	}

	protected void configureReportInstance () {
		if (currentReport != null) {
			currentReport.removeReportModifyListener(rmListener);
		}
		currentReport = new ValidationReport();
		contextMap.clear();
		currentReport.addReportModifyListener(rmListener);
	}

	protected void updateMarkers (ValidationReportRecord[] records) throws Exception {
		if (resource != null) {
			resource.deleteMarkers(null, true, IResource.DEPTH_ZERO);
			for (int r = 0; r < records.length; ++r) {
				ValidationReportRecord record = records[r];
				String id = record.getNodeId();
				CreationContext context = (CreationContext) contextMap.get(id);
				if (context != null) {
					IMarker marker = resource.createMarker(ValidationUIConstants.VALIDATION_MARKER_ID);
					StackTraceElement[] elems = context.getStackTrace();
					int level = record.getLevel();
					int index = 0;
					for (int t = 0; t < elems.length; ++t) {
						if (resource.getName().indexOf(elems[t].getFileName()) >= 0) {
							index = t;
							break;
						}
					}
					marker.setAttribute(IMarker.MESSAGE, record.getMessage());
					marker.setAttribute(
						IMarker.LOCATION, elems[index].getLineNumber());
					marker.setAttribute(
						IMarker.LINE_NUMBER, elems[index].getLineNumber());
					if (level == IReporter.ERROR) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_ERROR);
					}else if (level == IReporter.WARNING) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_WARNING);
					}else if (level == IReporter.INFO) {
						marker.setAttribute(
							IMarker.SEVERITY, IMarker.SEVERITY_INFO);
					}
				}
			}
		}
	}

	public String getResourceName () {
		return resourceName;
	}

	public void setResourceName (String resourceName) {
		this.resourceName = resourceName;
	}
}